import unittest
from datetime import datetime

from caplib.market import *
from caplib.analytics import *
from caplib.cmanalytics import *


class TestCmAnalytics(unittest.TestCase):

    def setUp(self):
        # Create calendar
        cal_cnbe = 'CAL_CNBE'
        hol_serial_numbers = [44654, 44954]
        sbd_serial_numbers = [44655]
        # Convert list of serial numbers to datetime objects
        holidays = [datetime.fromordinal(sn) for sn in hol_serial_numbers]
        specials = [datetime.fromordinal(sn) for sn in sbd_serial_numbers]
        create_calendar(cal_cnbe, holidays, specials)

        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and ANALYTICAL method
        self.bsm_analytical_pricing_settings = create_pricing_settings(
            'CNY', False,
            create_model_settings('BLACK_SCHOLES_MERTON'),
            'ANALYTICAL',
            create_pde_settings(),
            create_monte_carlo_settings()
        )

        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and PDE method
        self.bsm_pde_pricing_settings = create_pricing_settings(
            'CNY', False,
            create_model_settings('BLACK_SCHOLES_MERTON'),
            'PDE',
            create_pde_settings(201, 401, -5, 5, 'MMT_NUM_STDEVS', 0.001, 'ADAPTIVE_GRID', 'CUBIC_SPLINE_INTERP'),
            create_monte_carlo_settings()
        )

        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and MONTE_CARLO method
        self.bsm_mc_pricing_settings = create_pricing_settings(
            'CNY', False,
            create_model_settings('BLACK_SCHOLES_MERTON'),
            'MONTE_CARLO',
            create_pde_settings(),
            create_monte_carlo_settings(8096, 'SOBOL_NUMBER', 1023, 'BROWNIAN_BRIDGE_METHOD',
                                        'INVERSE_CUMULATIVE_METHOD', False, 1)
        )

        # Create Pricing Settings with Duprie Local Vol model and PDE method
        self.duprie_pde_pricing_settings = create_pricing_settings(
            'CNY', False,
            create_model_settings('DUPIRE_LOCAL_VOL_MODEL', [201, 401, 4, 0.001]),
            'PDE',
            create_pde_settings(201, 401, -5, 5, 'MMT_NUM_STDEVS', 0.001, 'ADAPTIVE_GRID', 'CUBIC_SPLINE_INTERP'),
            create_monte_carlo_settings()
        )

        # Create Pricing Settings with Duprie Local Vol model and MONTE_CARLO method
        self.duprie_mc_pricing_settings = create_pricing_settings(
            'CNY', False,
            create_model_settings('DUPIRE_LOCAL_VOL_MODEL', [201, 401, 4, 0.001]),
            'MONTE_CARLO',
            create_pde_settings(),
            create_monte_carlo_settings(8096, 'SOBOL_NUMBER', 1023, 'BROWNIAN_BRIDGE_METHOD',
                                        'INVERSE_CUMULATIVE_METHOD', False, 201)
        )

        # Create Risk Settings
        self.risk_settings = create_cm_risk_settings(
            create_ir_curve_risk_settings(
                delta=True, gamma=False, curvature=False,
                shift=1.0e-4, curvature_shift=5.0e-1,
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK',
                scaling_factor=1.0e-4, threading_mode='SINGLE_THREADING_MODE'),
            create_price_risk_settings(
                delta=True, gamma=True, curvature=False,
                shift=1.0e-2, curvature_shift=5.0e-1,
                method='CENTRAL_DIFFERENCE_METHOD',
                scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'),
            create_vol_risk_settings(
                vega=True, volga=True,
                shift=1.0e-2,
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK',
                scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'),
            create_price_vol_risk_settings(
                vanna=True,
                price_shift=1.0e-2, vol_shift=1.0e-2,
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK',
                price_scaling_factor=1.0e-2, vol_scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'),
            create_dividend_curve_risk_settings(
                delta=True, gamma=False,
                shift=1.0e-4,
                method="CENTRAL_DIFFERENCE_METHOD", granularity="TOTAL_RISK",
                scaling_factor=1.0e-4, threading_mode="SINGLE_THREADING_MODE"),
            create_theta_risk_settings(
                theta=True, shift=1, scaling_factor=1. / 365.)
        )

        # Create Scenario Analysis Settings
        self.scenario_analysis_settings = create_scn_analysis_settings(
            scn_analysis_type='PRICE_VOL_SCN_ANALYSIS',
            min_underlying_price=-20e-2,
            max_underlying_price=20e-2,
            num_price_scns=11,
            price_scn_gen_type=1,
            min_vol=-5.e-2,
            max_vol=5.e-2,
            num_vol_scns=12,
            vol_scn_gen_type=0,
            threading_mode='SINGLE_THREADING_MODE'
        )

        # currency
        self.currency = 'CNY'
        # underlying
        self.underlying = 'CU.SHFE'

        # Create Market Data
        self.as_of_date = datetime(2020, 2, 21)

        # Create IR Yield Curve
        self.ir_curve = create_ir_yield_curve(
            as_of_date=self.as_of_date,
            currency=self.currency,
            term_dates=[
                datetime(2020, 2, 26),
                datetime(2020, 3, 2),
                datetime(2020, 3, 9),
                datetime(2020, 3, 24),
                datetime(2020, 5, 24),
                datetime(2020, 8, 24),
                datetime(2020, 11, 24),
                datetime(2021, 2, 24),
                datetime(2022, 2, 24),
                datetime(2023, 2, 24),
                datetime(2024, 2, 24),
                datetime(2025, 2, 24),
                datetime(2027, 2, 24),
                datetime(2030, 2, 24)
            ],
            zero_rates=[
                0.0135486283791684, 0.0197762034605164, 0.0197686053073393, 0.0224838821372655, 0.0241740300538751,
                0.0256822601972516, 0.0265096948143765, 0.0271330931714993, 0.0274314991366822, 0.0284834397783798,
                0.0297276346662025, 0.0308410887945891, 0.032692683803743, 0.034410206396147
            ],
            curve_name='CNY_SHIBOR_3M'
        )

        # Create Option Quote Matrix
        self.cm_option_quote_matrix = create_cm_option_quote_matrix(
            exercise_type='AMERICAN',
            underlying_type="FUTURE_UNDERLYING_TYPE",
            as_of_date=self.as_of_date,
            term_dates=[
                datetime(2020, 3, 12),
                datetime(2020, 4, 13),
                datetime(2020, 5, 12),
                datetime(2020, 6, 12),
                datetime(2020, 7, 10),
                datetime(2020, 8, 12),
                datetime(2020, 9, 11)
            ],
            payoff_types=[
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL',
                 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL',
                 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL',
                 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'],
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL',
                 'CALL', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL']
            ],
            strikes=[
                [54000, 55000, 56000, 57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000,
                 68000, 69000, 70000, 71000, 72000, 73000, 74000, 75000, 76000, 77000, 78000, 79000, 80000, 82000],
                [54000, 55000, 56000, 57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000,
                 68000, 69000, 70000, 71000, 72000, 73000, 74000, 75000, 76000, 77000, 78000, 79000, 80000, 82000],
                [54000, 55000, 56000, 57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000,
                 68000, 69000, 70000, 71000, 72000, 73000, 74000, 75000, 76000, 77000, 78000, 79000, 80000, 82000],
                [56000, 57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000, 68000, 69000,
                 70000, 71000, 72000, 73000, 74000, 75000, 76000, 77000, 78000, 79000, 80000, 82000],
                [57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000, 68000, 69000, 70000,
                 71000, 72000, 73000, 74000, 75000, 76000, 77000],
                [56000, 57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000, 68000, 69000,
                 70000, 71000, 72000, 73000, 74000, 75000, 76000, 77000, 78000, 79000, 80000],
                [57000, 58000, 59000, 60000, 61000, 62000, 63000, 64000, 65000, 66000, 67000, 68000, 69000, 70000,
                 71000, 72000, 73000, 74000, 75000, 76000]
            ],
            prices=[
                [14.00, 16.00, 20.00, 26.00, 38.00, 64.00, 86.00, 130.00, 198.00, 282.00, 400.00, 600.00, 878.00,
                 1070.00, 660.00, 394.00, 236.00, 142.00, 90.00, 52.00, 32.00, 22.00, 18.00, 14.00, 10.00, 6.00, 6.00],
                [126.00, 126.00, 160.00, 200.00, 250.00, 312.00, 396.00, 478.00, 600.00, 776.00, 974.00, 1230.00,
                 1602.00, 1736.00, 1276.00, 910.00, 642.00, 460.00, 312.00, 212.00, 150.00, 110.00, 80.00, 58.00, 40.00,
                 28.00, 38.00],
                [172.00, 192.00, 236.00, 292.00, 426.00, 478.00, 574.00, 702.00, 892.00, 1206.00, 1368.00, 1632.00,
                 2140.00, 2218.00, 1762.00, 1350.00, 1026.00, 762.00, 562.00, 404.00, 294.00, 210.00, 128.00, 102.00,
                 58.00, 56.00, 106.00],
                [414.00, 486.00, 584.00, 700.00, 958.00, 1186.00, 1308.00, 1634.00, 2008.00, 2436.00, 2914.00, 2814.00,
                 2396.00, 2026.00, 1698.00, 1412.00, 758.00, 964.00, 446.00, 346.00, 256.00, 188.00, 138.00, 102.00,
                 82.00, 44.00],
                [676.00, 798.00, 936.00, 1064.00, 1354.00, 1780.00, 2138.00, 2538.00, 2978.00, 3474.00, 3324.00,
                 2902.00, 2516.00, 1544.00, 1256.00, 984.00, 806.00, 622.00, 488.00, 382.00, 282.00],
                [678.00, 802.00, 906.00, 1058.00, 1246.00, 1456.00, 2066.00, 2446.00, 2862.00, 3314.00, 3814.00,
                 3590.00, 3168.00, 2784.00, 2446.00, 2136.00, 1852.00, 1612.00, 1390.00, 630.00, 1026.00, 448.00,
                 338.00, 278.00, 196.00],
                [1006.00, 1146.00, 1312.00, 1558.00, 1790.00, 2118.00, 2688.00, 3118.00, 3578.00, 4078.00, 3828.00,
                 3408.00, 3024.00, 2680.00, 2358.00, 2078.00, 1822.00, 1586.00, 1388.00, 666.00]
            ],
            underlying=self.underlying
        )
        # print(self.cm_option_quote_matrix)
        # Build Volatility Surface
        self.vol_surf = cm_vol_surface_builder(
            as_of_date=self.as_of_date,
            smile_method='SVI_SMILE_METHOD',
            wing_strike_type='DELTA',
            lower=-1e-5,
            upper=1e-5,
            option_quote_matrix=self.cm_option_quote_matrix,
            underlying_prices=[66810.00, 66750.00, 66620.00, 66620.00, 66490.00, 66460.00, 66380.00],
            discount_curve=self.ir_curve,
            fwd_curve=None,
            building_settings=[1, 0.5],
            underlying=self.underlying
        )

        self.mkt_data_set = create_cm_mkt_data_set(
            as_of_date=self.as_of_date,
            discount_curve=self.ir_curve,
            underlying_price=66380.00,
            vol_surf=self.vol_surf,
            underlying=self.underlying)

    def test_cm_vol_surface_builder(self):
        expected = b'\n \x08\x01\x10\x03\x18\x01 \x01(\x010\x028\x01I\xf1h\xe3\x88\xb5\xf8\xe4\xbeQ\xf1h\xe3\x88\xb5\xf8\xe4>\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x1a\xd6\x04\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19O\xae6\xa3\x07\x14\xe4@!\x00i\xbc\x05n\x94\xf5@*\xeb\x01\n\xe8\x01O\xae6\xa3\x07\x14\xe4@\x00\x00\x00\x00\x00^\xea@\x00\x00\x00\x00\x00\xdb\xea@\x00\x00\x00\x00\x00X\xeb@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x00\x00\x00\x00\x00\x0b\xf3@\x00\x00\x00\x00\x80I\xf3@\x00\x00\x00\x00\x00\x88\xf3@\x00i\xbc\x05n\x94\xf5@0\x039\xe0\xc0\x81\x03\x07\x0e\xac?B\xeb\x01\n\xe8\x01\xe2_\x911\x03\x8b\xdf?\x9f\xb8\xb5hzC\xd8?{\x85\xd8\x83:\xc6\xd6?\xcc\x843\xcd\xd2\x85\xd5?\x9d\x8a\xb5\xa5\x8f[\xd4?\x91\xb4\xf2\xf1\x8e\x83\xd3?\xfa\x16=\x8d\xd2#\xd3?DJ\xaach\x00\xd2?\xac\x97\x92\xd6:F\xd1?\xef\xef\xc5\xc9\x07\x9a\xd0?+\xda\xa8W\x9f?\xcf?9\r\xcfa\x85,\xcd?i\x97E\xd9 \xcb\xcb?x\x90\x98(fA\xca?\x14\x1a4\xbdS\x19\xc7?\x08_q\xa6\n\x03\xc7?\xa2\x83"\\\xb8Z\xc7?\xbdR\xbcg\x9f\x1d\xc8?\x939u\xc9\x1f\x07\xc9?\xdc$\xfa\xf50A\xca?\xf6>\x94\xed^\xed\xca?\x0b\xbb\xa7\xb0\x0c\xe7\xcb?\xce\x97\xd0G\xdaG\xcd?\xca\x94\x0fM\xc1B\xcf?\xc2\xb4\x05\xd7Ms\xd0?-\x92\xa1\x16Q\x0b\xd1?\x813\x92O\x11F\xd1?F\x0f\xbb\xdf\xe2l\xd2?\xa8\x18MfH\xcf\xd1?J*\n(W\xa2?O\x9d2[?\xc2\xc0\xaa\x11\xd2\xef\x90?~\x0c!\x80\x15\xdc\xda\xbf\xc4\xb297\x03\xd3\x83?\xe0\xa7\xf5\x14\x88\xceD?R"\n \xe0\xc0\x81\x03\x07\x0e\xac?|\x19YaFT\xf0@O\xae6\xa3\x07\x14\xe4@\x00i\xbc\x05n\x94\xf5@X\x01\x1a\xd6\x04\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x00\x00\x00\x00\xe0K\xe0@!\x99g\x1f\x14\xe1\xa5\xf7@*\xeb\x01\n\xe8\x01\x00\x00\x00\x00\xe0K\xe0@\x00\x00\x00\x00\x00^\xea@\x00\x00\x00\x00\x00\xdb\xea@\x00\x00\x00\x00\x00X\xeb@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x00\x00\x00\x00\x00\x0b\xf3@\x00\x00\x00\x00\x80I\xf3@\x00\x00\x00\x00\x00\x88\xf3@\x99g\x1f\x14\xe1\xa5\xf7@0\x039\xc5#\xe1[Q<\xc2?B\xeb\x01\n\xe8\x01\xb6\x03\xd5\x07/\x9a\xdf?[\x8d\xf3f\x80\x14\xd5?\xd86\xd0]\x8f\x94\xd3?)Z\xf3\xe5&\xff\xd2?\xb9\xdaH\xefQZ\xd2?A\xb0\'\xb9(\xb5\xd1?^H\xf5=|\r\xd1?\x16\x16\xc1\xf2iz\xd0?#6WJ\xbcG\xcf?\xab\xa1\x9b\x03\xf6\xfc\xcd?$\x96\xc8\xe9\x12\x14\xcd?\xcf\xb1+\x8e\x07\xd3\xcb?\xa8U(\xbb\x03\xb2\xca?\xd8\xb0}ljJ\xca?3-)[\xcdL\xc6?\xa8u\x04\xf8\x92\x03\xc6?\xa6o\x0e\xd3\xc7\xcd\xc5?t\x18J\x926\xd9\xc5?\x03\xc2\x1b\xb5\xde=\xc6?\xf3,\x86\xbf\xdbP\xc6?\xe5\x060R\xee\x89\xc6?#\xa8\x92\xc3\xf9\x0c\xc7?!\xd8\xb6\x88\xb7\xbb\xc7?_\xdc\x964\x12W\xc8?po\x7f\xd4\x89\xe7\xc8?\xed\xbf\x01\xe4\xc6?\xc9?\x1b\xa2]{<\xa6\xc9?\xbe\x83"r\x02\x81\xcc?\xcb\xf5WduP\xcd?J*\n(\xad\x15\xa5:\x1c\x03p?\x10\xcd\x85\x8d\xd6@\x9b?\r\xa3\x86\xf9;\xba\xe3\xbf?\x0b\xb9z\xcd\x96\x94?,\x86\x97\xf9\xbb\xbaw?R"\n \xc5#\xe1[Q<\xc2?\x9a\xc7\xe1\x12\x97Y\xf0@\x00\x00\x00\x00\xe0K\xe0@\x99g\x1f\x14\xe1\xa5\xf7@X\x01\x1a\xd6\x04\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x00\x00\x00\x00\xc0C\xe0@!\x0b\x8b\xd75Y\x13\xf8@*\xeb\x01\n\xe8\x01\x00\x00\x00\x00\xc0C\xe0@\x00\x00\x00\x00\x00^\xea@\x00\x00\x00\x00\x00\xdb\xea@\x00\x00\x00\x00\x00X\xeb@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x00\x00\x00\x00\x00\x0b\xf3@\x00\x00\x00\x00\x80I\xf3@\x00\x00\x00\x00\x00\x88\xf3@\x0b\x8b\xd75Y\x13\xf8@0\x039}\xc6Y\x80\xcdg\xcc?B\xeb\x01\n\xe8\x01\x02T\\\x8a\xf9e\xda?\x90\x8es\x82\xde\xf8\xd1?\xf2\xaf\xb2\xeb\n\x1c\xd1?\xd4\x8c\xe5\x058\x99\xd0?Dy!\xb1-\x1e\xd0?@\xdf\x14\x95\xcad\xd0?P\xdb+\x8e\xcb\xfd\xce?\xb7\x8aqq\x03\xd2\xcd?_y\xf9\xe8\x82\xd6\xcc?\xf6\xe5\xfa\xc7\xd1W\xcc?\xe6\xc1A\x087\xe6\xcc?>X5\x87\x9d\x02\xcb?(0\x0b^\x15\xdd\xc9?\xdf\xa4\x07$q\xbd\xca?#\xc5*\xb2\x14\xd0\xc6?\xd8\xd1\xeb\'B\xb0\xc6?E( \x11\xb4L\xc6?\x07\xfe\x81h\xdb"\xc6?\xea\xd7\xa1\x83a\xf5\xc5?5\xda]J\'\xea\xc5?x31\x97\xf0\xd1\xc5?\x8d\xc9\xfc\xc1u\xed\xc5?\xab\xf1\xd3\xdbM\xff\xc5?v\xe9\xb6\xea\xa5d\xc5?\xbeP\x93i\\\x16\xc6?c\xd9\xef\xc9\x1cu\xc5?\x1aT\xfbM\x81\xcb\xc6?xo\x1f\xb7\xc6\xfd\xca?\xdf\xd8uM\xf0\xba\xc8?J*\n(\x92%[)p\xd9y?\x8dW3n\xb7\xcd\x98?\xc4.\xfcK2\x98\xe8\xbf@\xd0w;44\xa2?\x8fX9\xadh*\x84?R"\n }\xc6Y\x80\xcdg\xcc?aP\xff\x94\xd6Y\xf0@\x00\x00\x00\x00\xc0C\xe0@\x0b\x8b\xd75Y\x13\xf8@X\x01\x1a\xc6\x04\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xf4\xfb\x7f\xc1\x11C\xf0@!\xfc\xcd\x9c\xe1^D\xf0@*\xe3\x01\n\xe0\x01\xf4\xfb\x7f\xc1\x11C\xf0@\x00\x00\x00\x00\x00X\xeb@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x00\x00\x00\x00\x00\x0b\xf3@\x00\x00\x00\x00\x80I\xf3@\x00\x00\x00\x00\x00\x88\xf3@\x00\x00\x00\x00\x00\x05\xf4@\xfc\xcd\x9c\xe1^D\xf0@0\x0397:AOk\xa3\xd3?B\xe3\x01\n\xe0\x01\xcd\xc2\xbf"\x1e\xe2\xca?\x18\xb1\xc0\xba\xc6i\xd0? \x18\x01\x07\xcd\xd0\xcf?\x07\x9a\x11\xaf\xf3\x08\xcf?\xe2\x94\x12}aA\xce?\xe6QK\xb5(\x16\xcf?\n\xe2\x99\x15\xb4\xf5\xce?\x1e"\xa0C\xb3D\xcd?\xb9\xf2^\xf7\x00x\xcd?Y>lkU\xac\xcd?R_\x83\xf7B\xee\xcd?\xe5CZf\x0e4\xce?\xbfW\xc1\x8b`\xf7\xc7?T\xe1\xd0\xb8\xbbH\xc8?\x83D\xf1\xc8\xf1\x8c\xc8?\xa4W/p\xf1\xbe\xc8?\xc5Rr\x938\xe7\xc8?(\x18\x12\xb7\xb5\x9e\xc4?y\x8aj\xde\xd3?\xc9?\\\xb7\xfec*\xcd\xc4?m\xf7\xd4\xae\xba\n\xc5?8|\xff@[\x0b\xc5?k\xff\xc3\x154\x11\xc5?k\x8a\xc8\x18&"\xc5?\x18\xee\xf3C7B\xc5?\xd4\x81\xb8\xd9\x12\xb6\xc5?+*\x84O5\xee\xc5?\r)F\x12\xa4\xde\xca?J*\n(:n\xdc4P\xba\x80?\x15\xd3\r\xa9\xf3\x9a\x9a?NXSJ\xd4\xe1\xe5\xbfe\xb7\xdbd{q\xbd?\x00\x00\x00\x00\x00\x00\x00\x00R"\n 7:AOk\xa3\xd3?\xeb.\x13\xce&c\xf0@\xf4\xfb\x7f\xc1\x11C\xf0@\xfc\xcd\x9c\xe1^D\xf0@X\x01\x1a\xf6\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x8fLe\xac\x02;\xf0@!\x102\xd3\xd9-<\xf0@*\xbb\x01\n\xb8\x01\x8fLe\xac\x02;\xf0@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x102\xd3\xd9-<\xf0@0\x039\xc4\x88\x11#F\x8c\xd8?B\xbb\x01\n\xb8\x01\x9c\xf7\x04\xfd"\xf6\xca?\x86\xcb\x14C\xfc\x86\xcf?\xc6-S\xb4\xa5\xd9\xce?c\xbcN\xc3\x19"\xce?\xdb\xa4D#;\x10\xcd?\x14\x8f\xf1a6{\xcd?\xd5\xfa\xa2\xc6U\xc1\xce?\xac\xf3\x98\xc3\xbb\xeb\xce?J%HDl\x17\xcf?\xdb\xa7\x9c\xd2\x08@\xcf?ly\xaa\xea6\x87\xcf?V\xcc\xa7\xe5#p\xc9?\xae\xcbZ<\xc5\xae\xc9?B\xae\xcc\xab\xa5\xd9\xc9?\xf2\xf3Uc\xa1\xdd\xc4?\x116U\xa9\xe8\xe9\xc4?\xabhN\xb9\xe4\xae\xc4?\x0fb\xad\xbf\xe6\xf6\xc4?`\xd6\x88O\x8b\xcf\xc4?L\x03"\xd4\xcd\xdc\xc4?o\xe6\xff\xd5,\xf2\xc4?\xc6\xda!Op\xc0\xc4?Q\xa6X\xdf\x88\xf2\xca?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00\xf3\x0c\xb2\xf9s\x19\xac?\xbcg\xb4N\x8fQ\xc3\xbf\xc9)5Ks\x9d\xd0?]M\xc6\xf4,\x11\x7f?R"\n \xc4\x88\x11#F\x8c\xd8?j\x18il\x93c\xf0@\x8fLe\xac\x02;\xf0@\x102\xd3\xd9-<\xf0@X\x01\x1a\xb6\x04\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x93?\xfeB49\xf0@!S\x1b\xbdG<:\xf0@*\xdb\x01\n\xd8\x01\x93?\xfeB49\xf0@\x00\x00\x00\x00\x00X\xeb@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@\x00\x00\x00\x00\x80\xcc\xf2@\x00\x00\x00\x00\x00\x0b\xf3@\x00\x00\x00\x00\x80I\xf3@\x00\x00\x00\x00\x00\x88\xf3@S\x1b\xbdG<:\xf0@0\x039Y\xe5\xfd.\x91U\xde?B\xdb\x01\n\xd8\x01\x99R\xe7\xcd\xc3`\xcb?\x10\xc2\xfc\xf6\x03\xca\xce?>(\xdc\xc7\xe9G\xce?c4\xc9\xb8\xc6Q\xcd?\xdf}\xb7\x13\xf6\xb9\xcc?B\x13\xf0\x00@F\xcc?\xa4V\x19\x88\x07\xc8\xcb?6-Tf]j\xce?1\x8eE\xea\x1b\xa2\xce?V\x8e\x7f\xa1\xaa\xd4\xce?J\xbe\xe0_5\x03\xcf?jk\x85\x17-E\xcf?u_\xc6\xfa)M\xc8?\xefl\x95\xbf\xaf\x8d\xc8?\x97JY{\xb2\xc4\xc8?\xcc:\xea\x00\x92\x05\xc9?w\xb4\xbf\xc1w4\xc9?.\xd5}\xca\x86Q\xc9?9ah-j\x85\xc9?!\xca\xbd\xacf\xa1\xc9?f\xf8qe3+\xc4?\xde\xb8bq\xa3\xda\xc9?17\xfa\xf7\x13\xed\xc4?\xad]Y\'c\xaa\xc4?\x1a\x03\x1e\xd7@\xe9\xc4?\xb6\xe3Z\x04Lt\xc4?\xd6\xb1r\xf9\xf0^\xcb?J*\n(\xa4\x02\xa7\x05!\x9ep?\xea\x08%M\xb6\xdb\xa4?\xaag\x1c>a\x89\xbd\xbf!\x00\xb4p\xa3\x0b\xd8?\x00\x00\x00\x00\x00\x00\x00\x00R"\n Y\xe5\xfd.\x91U\xde?$\xbd\xec\xb0:l\xf0@\x93?\xfeB49\xf0@S\x1b\xbdG<:\xf0@X\x01\x1a\xe6\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xd7\x13\x8e\xd8<4\xf0@!5\xe7<\xa435\xf0@*\xb3\x01\n\xb0\x01\xd7\x13\x8e\xd8<4\xf0@\x00\x00\x00\x00\x00\xd5\xeb@\x00\x00\x00\x00\x00R\xec@\x00\x00\x00\x00\x00\xcf\xec@\x00\x00\x00\x00\x00L\xed@\x00\x00\x00\x00\x00\xc9\xed@\x00\x00\x00\x00\x00F\xee@\x00\x00\x00\x00\x00\xc3\xee@\x00\x00\x00\x00\x00@\xef@\x00\x00\x00\x00\x00\xbd\xef@\x00\x00\x00\x00\x00\x1d\xf0@\x00\x00\x00\x00\x80[\xf0@\x00\x00\x00\x00\x00\x9a\xf0@\x00\x00\x00\x00\x80\xd8\xf0@\x00\x00\x00\x00\x00\x17\xf1@\x00\x00\x00\x00\x80U\xf1@\x00\x00\x00\x00\x00\x94\xf1@\x00\x00\x00\x00\x80\xd2\xf1@\x00\x00\x00\x00\x00\x11\xf2@\x00\x00\x00\x00\x80O\xf2@\x00\x00\x00\x00\x00\x8e\xf2@5\xe7<\xa435\xf0@0\x039\xc2\x1c\xd3?\x19\xcc\xe1?B\xb3\x01\n\xb0\x01U\xc7\x80\x96@!\xcb?0\x18\xff\xb5\xa8k\xce?\xd6\xa3\\\xb4\xe1\xbb\xcd?4s\x96~\xe3 \xcd?\x19\xcfw\xdd\xbc\n\xcd?\xfeBX~\x1e\x97\xcc?\x1eS\xd3\xe8T\xab\xcc?A)L\xcc\xe5=\xce?\x8e\xbe\xfaM\xf4w\xce?!6\xd5\xc7z\xa7\xce?\x07\xe0\xb9Op\xdf\xce?[\x18.\xe4\xca\xc8\xc7?\xe7c\xe9i\x84\t\xc8? \xd6\xe2\x93\x1cB\xc8?\xecr:\xafY}\xc8?\xdb4z\x05\xc5\xa0\xc8?%:\xce\xdf\xc9\xd2\xc8?\x11*4vZ\xf8\xc8?\x8d\x04\xca5\x11\r\xc9?\x98\x8a\xa9\xd8x8\xc9?`\x88"\xfd\x94P\xc4?\x12\xea\xc8\xa1\x84\x1f\xcb?J*\n(\x97\xa8,\xf7\x00\xc7\x8b?\xe0\xf6\xb8\x00>l\xa7?\r-\xa1O\x06\xf2\xc7\xbf\xb16.\x05w\xc2\xc8?b\xd6}\x90\xd5\xa7\xa1?R"\n \xc2\x1c\xd3?\x19\xcc\xe1?\xfe\xba\x1c&\xd0p\xf0@\xd7\x13\x8e\xd8<4\xf0@5\xe7<\xa435\xf0@X\x01"\x07\x08\xe4\x0f\x10\x03\x18\x0c"\x07\x08\xe4\x0f\x10\x04\x18\r"\x07\x08\xe4\x0f\x10\x05\x18\x0c"\x07\x08\xe4\x0f\x10\x06\x18\x0c"\x07\x08\xe4\x0f\x10\x07\x18\n"\x07\x08\xe4\x0f\x10\x08\x18\x0c"\x07\x08\xe4\x0f\x10\t\x18\x0b*\x07CU.SHFE'

        result = self.vol_surf

        # print(result.SerializeToString())
        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_european_option_pricer(self):
        expected = b'\n\xf4\x06\t8\x91Uu\xfd)\xedA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xbf}=\xaf\x8d\xed\x1fA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x90\xb3\xa32\xa0\xec\xdc\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08D\xe9%2q\xb2\xb4A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00 )\x93\xde\xb1\x07\xc1\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08^\xbc\x9b\xfa2\xbdC@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x80E\xdd\xc9\xe3\x96pA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xc0H\xbd\xa1\xfel\x86\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08Nx\xdf\xbc\x03\xc9\x00A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\xfb\x8dN\xe5\xda+A\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x1b\x19\xe3\x8fb\xf6\x10B\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xef\xec,\xb8@\xb6\xa5A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xd5\x9c}\'\xeczm\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00JU[p&\x98\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08ld\x0c\rvz\x8fA\x8d\x1a\xcc+a\xfd\xa6A\xf4\xec\x94X\xec\x04\xbcA\xd7\xa0~\x95\xf1P\xcdA\xe2\xab\x1f\x9bk\xfd\xdaA\xe6\xbbVi\xb3V\xe6A\xb0,&v8\xef\xf0A\xcaW\xd3H\xe8\xe5\xf7AT\xfa\x8d\xaf,\xd3\xffA\x18\xc0\x98\xbbx:\x04BuX\xa1e\x1f\xc8\x08B\xd1\x907\xa6\xd2\xab\x95A\xac\xdeszJ\x17\xadA\x9f(\xc0(\xfd\x98\xc0Af6\xaa\xdc?\x83\xd0A\xf2\x03\xe1\xf4\xffG\xddAN\x07\xacU\x89\x94\xe7A4\x16\xbe\x14\x8f\x88\xf1Ar\xa3\x8e\x9b\\k\xf8A\xff\\3x\x90\x1e\x00B\xf5v2a8a\x04B 9\xda\x9eu\xe2\x08B\xd6\x85\xc6\x0f\xc0\xc3\x9cA0V%\x8e\xb4\xf2\xb1A\x9b\x1f\xf2\xa3\x88W\xc3A\x16\x0fp[\xf9i\xd2A\xeaCr~\xff\x95\xdfA\x9bF\n\x88O\xd2\xe8A\xcb,AA\xb8"\xf2A\xd3\x0c\x8d\x01\xab\xf3\xf8A\xfb\xd5]\xe0\tV\x00BU\xc4!\x94(\x8b\x04B\x9b\x16\xb4\xea"\x00\tB\x8a\xd4\xf9\xd29\x84\xa2A\xc0\x8es\xee1\xaf\xb5A\xc0\xe2\xfbFD:\xc6A\xc5\xb0\xe4E([\xd4AP\xee"\xacu\xf3\xe0A\x92\xde\x11-\x05\x10\xeaA\xf1\xf8lC\x95\xbd\xf2A\xe6?f!v~\xf9A\xc6`\r\x8b\xbe\x8f\x00Bs\x114\x83\t\xb8\x04B\x8b\xc7&\xae\x02!\tB\xa0#\xef_U<\xa7A\xd8E\xda\xb8#\xbc\xb9A\xf6\xc5\'\x9d\xb7=\xc9A\xf4\xbf\x15v\x88U\xd6A\x07o_D-\x1d\xe2A\x88P\x84q\xa9M\xebA\xefDr\xb3\x0cY\xf3A\x94\xb5\x7f\x08o\x0b\xfaA\x7f\xf6acr\xcb\x00B\\\xf2w_\x9e\xe7\x04B\x7f\xdd\xbb\x97\xebD\tB4\x83C\xa9\xbf\x87\xacA2\xb4\x07Y{\x14\xbeA\xb3/$\x8c\xc2^\xccA\xba%\x12\xfb\x04X\xd8A\xdd\x85\xc8\xb7\xfaG\xe3AP\xe0E\x82;\x8b\xecA\xb1\xbb@^\t\xf5\xf3A\xb9\xbdO\x9aR\x9a\xfaA\x93\xa8.p\xf0\x08\x01B\x17$r)\xae\x19\x05B\x8e\xfc\x86\x02\xb2k\tB\xafM\x0f\xa3U1\xb1A\x8f\x8e/\xd1\x9dY\xc1A\xf5{\xb5T\x97\x9a\xcfA\x18\x81P0\xb0a\xdaA\xc0\x8e\xe2&\xb9s\xe4A\xa28_\x8c\xba\xc8\xedA2)\x82my\x91\xf4Aq X\x7f\xe7*\xfbA\x95I\xb6\x01\nH\x01B\x04\xea\x08\x0c\x04N\x05B~\x0f$\xac)\x95\tB\'\xbag?*d\xb4A\xa9.\xad\xf5\xc8\xc9\xc3A;\xc7\'\xbfYw\xd1A|aCD\xbdq\xdcA\xf8\x02\x9aLI\xa0\xe5A@\x10\xff\xbc%\x06\xefA\xae\xe9F\xc1M.\xf5A\x7f\x8f\x1az\xfc\xbc\xfbA\xc4$\x04\xed\x95\x88\x01Bt\x190no\x84\x05B\x19\xe6s\xe4&\xc1\tB\xbe\x8e\x02?\xa9\xd9\xb7A7\xf2\xce\xc6rX\xc6A\xe01\x1c@l,\xd3A\x84\x81m\xeaz\x87\xdeA+\r\xb9s\x90\xcd\xe6A\x82\xe6\xbd \xbe!\xf0A\xba\xf9\x9fny\xcb\xf5A\xd5\x8b?\rgP\xfcAv\x90\xca\xd9o\xca\x01B@qj\xd7\xc3\xbc\x05B\xa0\xe0/X\x7f\xef\tB\xcaz\xd2\xbb\xeb\x8e\xbbA\x96\x14ZIp\x03\xc9A\x8f\xce\x11P\x82\xeb\xd4A\x87,E\x80\'Q\xe0An\xf7Y\xa5w\xfb\xe7A\x04\x94\xaa\xa3^\xc0\xf0Ay\x94\xfaY\xf1h\xf6Af\xb9db\x02\xe5\xfcA\xa1\xd8(\xa4w\r\x02BJ\xef\xfa\xb9\xd8\xf6\x05BQ\x94\xff\x90\n \nB\xdel\xa7\x1a\xf1\x80\xbfA\x97\xd2\x90\xd0\xb9\xc8\xcbA\xec\xab\xf3"\xb6\xb3\xd6A\x9e\x90R|\xd9`\xe1A\xab\x83\xb0\x02\xeb)\xe9A\x1e\xe8\x1a\xfe\xf3^\xf1Av\\\x91\xe6\xab\x06\xf7A\xa2\x08~d\xaez\xfdA\xebB\xd6\xd1\x90Q\x02BW\xd44/\x892\x06B\xf2x\xc6?\xa2R\nB\xdeH\xe3\xe2X\xd6\xc1A\x93\xd7\x9fWk\xa6\xceA@+\xcb\xf09\x84\xd8A\\lbq\x98r\xe2A\x12\xb1\x1b@\xd9X\xeaAB\x86\xfa\xc6}\xfd\xf1A\xd2c\n\xb6\xa0\xa4\xf7A\xfd>y\x03O\x11\xfeA\x9dBA\x19\xa2\x96\x02Br\xe8\xf2\xad\xb3o\x06B\xe2\xed\xd9`"\x87\nB \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'
        inst = create_european_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike=66380.00,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_european_option_pricer(
            instrument=inst,
            pricing_date=self.as_of_date,
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )

        # print('test_cm_european_option_pricer:', result.SerializeToString())
        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_american_option_pricer(self):
        expected = b'\n\xf4\x06\t\x9f\xc3O\x9f\xe0=\xedA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xcc\xfb\x9a\xcb\xff\x02 A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x80A\xfd\x97Y?\xd5\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08t\xd5\xab\xd5I\xc2\xb4A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x003^\xeag\x01\xc1\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xd5\x058\xc5\xf5\xd7C@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x7f}{pa\xadpA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xc0q\x01_j\xa1\x86\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x1bR\x07\x7f@\xd5\x00A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xffw\x94\x1e4\xef+A\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08t<\xaeP\x03\x02\x11B\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x0f\x95\xa1\xf6"\xc5\xa5A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xcc\xa4\xba+\x1b\x8fm\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x80`>-\xf96\x98\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08.J&(\x84\xb0\x8fA\xe5$\x1cf\xdf\x16\xa7A\x8b-\xacgD\x1c\xbcAt\x86\x11\x9b\xb8e\xcdA\xc9"XIl\x0f\xdbAF\x89\xcd\xa8\xeae\xe6A\xd6\xa5\x04\x8a\xcb\xfb\xf0AYW4\xd9F\xfa\xf7A{\xf4\xc2\xd8\x94\xf3\xffAH\x92\x83\x95\xd5S\x04B\xedi\x07?<\xef\x08Bt\xd8\xb5\xca\xde\xcc\x95A\xdd\x15A\xfaz5\xadA=X\xc1gd\xa6\xc0A\xa1\xdaZJ\xd5\x8e\xd0AhX6\xe2\x84[\xddA\x1b\xeaQ\xd4\x99\xa4\xe7A\x0c\xd7X\x8b\x80\x95\xf1A\xb4\xf4&\x10\xd3\x7f\xf8A\t\xb2\x1b\xfbv.\x00Bi#f\xe7\x8cy\x04B\xe9\xf14\xe6)\x07\tB\xa9\xe7\t\xbcu\xeb\x9cA\x16\xb7\\\xbaS\x04\xb2Af\x94\xfc\xd5\xc0f\xc3A\x05\xca\x1d\xa8\xc8v\xd2A\xf0\xdb^\xef\x0b\xab\xdfA_\x99\x84Z9\xe3\xe8A\xa6\x8aO\x11\n0\xf2A\xf0<\xbf\rE\x08\xf9A\xb2\x16n\x81\xb6e\x00B\xb0a\x16\x8e\xa8\xa2\x04B\x95\xff\xfa\xdf\xe4"\tB\xfd\xda\x01\xe4\xb9\x9b\xa2A@+\xf1\x85\x83\xc3\xb5AX\xe32\x81aK\xc6A$\xcb\x8c\x9c8i\xd4A\x10~7\x18\xc1\xfe\xe0AuJ\x19i\xc8!\xeaA\x94\xc7\xb5\x14I\xcb\xf2A]\x81x^=\x93\xf9A\xdbz \x15A\x9f\x00B\xbe\xd3\x12\x9c\xdd\xce\x04B8!#U-B\tB\xc15\xba\x00\xc9W\xa7A\xb6\x0e\xa9\xf4P\xd3\xb9A\x89\xfc\xbdp\xccP\xc9A\xf2\xc7\xd7Q\xe0d\xd6A\x8d(1-?)\xe2A\xa4M\xe4-F`\xebA\xc7h\x13\xef#g\xf3A\x93\x11\xf5nk \xfaA\x02\x89\x94\xaf\xd7\xda\x00B\x82R\xb0\x0c\xe7\xfd\x04B%\x03\xcaB\xc6d\tB4\xca\xf3\xc8r\xa7\xacA0\xd0D:\xab.\xbeAJ\xac6\'\xe0s\xccA\x00t\xd9<\xaah\xd8ANK\tD\xd4T\xe3A\x050\xdc\xd6\xb1\x9e\xecA3>\x02;\x85\x03\xf4A\xc8\x96$\xdb\x8a\xaf\xfaAy\x13\x16\x14C\x18\x01BY\x84\xbb\xb1\x85/\x05B\xf95\xa1yu\x8a\tB\xbf\xfd\xf5\xa0sC\xb1A\xf7f/\x83Ih\xc1A\x02\x0f\x0c\xa1\xcd\xb1\xcfA\xf0\x90t1\xa8s\xdaA(\x9d\xfc_[\x81\xe4A\xfaY\x1b\x92\n\xdd\xedA<\xabM\xf9Z\xa0\xf4A\xad\x8c\xc2Fa@\xfbAl~\x96\xceRW\x01BD\xc3\x00\xfa\x7fc\x05B\x93\x90e\xc7\x03\xb3\tB\xea\xe7&\x04\xb0x\xb4A\xe7\xad\xd3\xd1\x19\xda\xc3A0\xb4\xe7\x9e\x08\x84\xd1A#G\x94\xe6\x0c\x85\xdcA\x88\xe6\xef"\xb5\xae\xe5A\x8e\xae\xe1\x8dO\x1b\xefA\xe7\x98\x85\xe8\x95=\xf5A\xd5\x1a>\xa2\xbc\xd2\xfbA@9RN\xdc\x97\x01B\x10\x02 \x9e\xa1\x99\x05B\x9fV\xe4\x1e=\xde\tB\x0bm\xe3\xc7\xb8\xf0\xb7A\x11\xa3\x13=yj\xc6A\xef\xce\x7f\xb35:\xd3A\xaeV\x10\xa6&\x9c\xdeA\xd22.\xc2\xc6\xdc\xe6AF\x14K\xfc\xbf,\xf0A\x82\xb8\x07\x02)\xdb\xf5A\xc8H9\xc5qf\xfcA\xfa\x01~\x1c\xba\xd9\x01B\x11\xf2\r?\xbb\xd1\x05B\x9cb\x1b\xaf\xf0\x0b\nB\x01\xa3o\xaa\xa5\xa8\xbbA\xc3\xb8{\xde;\x17\xc9A\xb4\xc6.\xb6l\xfa\xd4AW\x05\xfdw-\\\xe0AV\x87G4y\x0b\xe8Az<d\x80\xcd\xcb\xf0A\x80-,\x13\ty\xf6A\xd0m\xbeL[\xfb\xfcA\xd5\xbaO-\xcb\x1c\x02B\xea\xa0\xbe\xfe\xa1\x0b\x06B9;\xa8\xe6\xf0;\nB\x9ef\xe5\xb8t\x9d\xbfA7\x93\xbc+Y\xde\xcbA\xe2\x04\xbcj\xc7\xc3\xd6A\xaa]\x8fq\x91l\xe1A\xbecb\x8a\xb8:\xe9A\xd8Q\x99\xea\xcfj\xf1A\xab\xdd\xcek,\x17\xf7A\xcc\xae2\xafX\x91\xfdA\x19\x8d\xe3H\xf2`\x02B\x06\x16\xcf\x18/G\x06B\xcc\x10\xe0e\x13n\nB\'\xfan\x86\x0e\xe6\xc1A\x85!\xe0O\xec\xbd\xceAi\x84\xe1\xa2w\x95\xd8A\x11\x90\xad"\x04\x7f\xe2A\xb8\xa3\xebjsj\xeaA\x05N]\xd2\xc6\t\xf2A\xcb\x06:\x9d\x8a\xb5\xf7A\xa9t\x19{M(\xfeA\xd4\x13:\x89\x15\xa6\x02B\xaep\xeb~?\x84\x06B\x14\xdd\x89\xe30\xa2\nB \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'
        inst = create_american_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            strike=66380.00,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_american_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_american_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_asian_option_pricer(self):
        expected = b'\n\xf4\x06\t\xc0}3\x8a\x14\xcf\xdfA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08G\xbd\xe0ugq\x1bA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00AC\xfc\x87_\xcf\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x80oC\xd1)\xca\xb1A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x00\xa2+n\xb3\xf9\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x80-\x06\x9d)!R@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfe\x8c`\xee2y~A\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x98\xdd$\xa4\xd2\xdd\xa7\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xe7\x88\x08\xfd\x93\xff#A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xbfb!L\xf8\x97PA\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x087\xe7\xa1*\x88\xef\x02B\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08p<\xcf@\xd7<\x98A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x1c\x9an\xe7\xb2\xe6?\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00I\xa2\xe1("j\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x00\x00\x00\x00\x00\x00\x00\x00\xd5\xcf\x88u\xa6\xc1AA\xdcg\xe0\x9b*\xe5zA\x87\x8a\x7fb\xa2;\xa3A,\x18\xcb(\xc5\xfd\xc1A\x81\xa1H\xaf\xd6\xe9\xd7A\xfe\xb3\xc0\xa8UO\xe8A\xe5\x95F\xa3\xf1G\xf4Ai\x9d\x90\xbfEy\xfdA\xcd!\xed$\xb5\x8b\x03B\'\x16o (k\x08B}\x88\xa6\x04\x8a\r\xf5@\xae\xc4\xe9\xceZ\xbbPA\xf0\x1a\x00:\x96\x99\x83A\xfc\x1e\x9e\x9e\xbf\xec\xa7A\xf68\x10\xd5\xe0H\xc4A\x0e\x96\xacFeY\xd9A\xb9}4Je\xe9\xe8A\x12#\x81\x8ftt\xf4A\x96R\xe1Gm\x8b\xfdA:\xf4\xc3\xc1,\x8e\x03B\xb4@\xe0Gvk\x08B\xc1\xb88Q\xc7>\x14At\xdc]0Xr^A$\x03\xb7Uo"\x8bA0\x94\xb6z\xcf!\xadASx\xcf@\xb6\xa1\xc6A\xa8\xef\x04\x08\xdf\xc8\xdaA\xed\x19\\\x99\xcd\x86\xe9A^#\xd0z\xaf\xa4\xf4A\x94y\xfcS\xff\xa0\xfdA\x83\xa0\xe3W\xea\x91\x03B\x85\x1f\x9aQ\x1cl\x08B\xe4\xbd\x96)\x9d\xcd,A\xd5\xe7\xe8\xb1k\xe7hA\xf7\xefV\xfc\xf9\x04\x92A\xa2\xb8\x0e3\x1dY\xb1Ay\xcc;C"\x07\xc9A\x83w\x9b\xd8V8\xdcA\x0b\xc2\xcb\x96\xe6&\xeaAH\x86P\xcdZ\xd8\xf4AD\xca[\x8e\x9c\xba\xfdA\xf2\xf0\xe3\x81\xc1\x96\x03B\x96\xa3g\xc8&m\x08B\xc0b8\xee\x18-@AE\xc2GP\xb8\x80rA\xf6\x8a\xd2\x98Z8\x97A8\xf8;\x8e[U\xb4A\xe0\xae\x0b\xd4\xc8v\xcbA\xba\xdb\x86\xe0\xd1\xa7\xddAP-\xa8X3\xc9\xeaA?\r\x8br\x10\x0f\xf5A\xa0\x9a&D7\xd8\xfdA\x01\xe9\x1d\xa3\xcc\x9c\x03B\xe6}\x0b\x81\xc9n\x08BT\xb9~\xc3\xdc\x7fKA"\x92\xd1\xb8lSzA\x8dc\x17\xf5\x98=\x9dA\x02\x05\xf4k\x7f}\xb7AQr\xc2\x1a\x88\xef\xcdAj\xc5\xbc)U\x17\xdfAO;\xee\x88\xf8m\xebA\xa2\xbd\xa3\xf1~H\xf5Az\xc4N\xf7?\xf9\xfdA\xc5\xd5\x19\x96\x1b\xa4\x03Bo[\x1c\x86Mq\x08B\xd2\xb3 \xb0?=VA\xd6r\x95Vd\x1f\x82A\xda\xf3\'9\x87\xfa\xa1A(\\\xb2\x99\xb5\xcf\xbaA\x93\x1a\xf0\x92\x048\xd0A\x9c4\x95biC\xe0Ag\xb2O\xd0\xa3\x14\xecAb\xdd\x01\x1b\xc3\x84\xf5A\xedf`\x92\x08\x1d\xfeA\xf1\x91\x8e\x14\xd3\xac\x03B\xb25\x05>\xdct\x08B\xee\x97\xf2\xad\x1a\xf1aAK(\x86\x97\x8e\xec\x87A"\x0b\xd1\x8e\xce\xa9\xa5A6\xa6\xd7\xbdJK\xbeA\xabf\xae\x15\x88{\xd1A\xd0\x16\xce\xbc"\xfb\xe0A\x7f)\x82\xd9\x12\xbd\xecA\xb1\xa1\x92U\x80\xc3\xf5A\xf2e\xdcc\xcfC\xfeA\xf9)\x0c\xea\xd2\xb6\x03B\x1emh\x1bXy\x08B\xa72S\xdbY.kAP\x80\xb33x\xc5\x8eA\xe5\xed\x8bA \xa3\xa9A%\x01"\xd0U\xf4\xc0A\xd2\xb1n\xd5\xe8\xc1\xd2A\x7fR/U\xd8\xb2\xe1A!\xdfI\xb6\x1fg\xedA\xed\xfdx\xd87\x04\xf6Ad\xab\x04\x7fGm\xfeA\x86\x80J\xc15\xc2\x03BX\xf4\xe5\x81\xbc~\x08B6\x10\x1a\xe3D\x05sA\x1a\x0e\xb0B1^\x93A\x8e\x8f\xc2 \xc1\xee\xadA\x9b\x80x\xa1I\xd4\xc2AHM\x91az\n\xd4A\xd9\xe2\xc8R\x84j\xe2A\xd9\x8c\x9eWh\x12\xeeA\xa27#\xb1\xd9F\xf6A\xc8\x82ot\xd5\x99\xfeA\xa1\xf4\x8c\x02\x10\xcf\x03B\x9f\x0b1W0\x85\x08B\xcc\x0c\xea\x93\x1e\x7fyA\x85\xb3\xfc\xa4\x00\xd0\x97A\xc7m\xfb\xed\xe3A\xb1A\x87\xf6h3\xca\xc3\xc4A(\xae\x116FU\xd5A\xf4U\xfe\x96-"\xe3AA5\x93\x1e\xca\xbe\xeeA\xab\x8f\xd2\xa6)\x8b\xf6A\x8c\xc5\xcc\x81\x0b\xc9\xfeA|5\xbf\x07\xa6\xdd\x03B\xed\xa3\xab\xa1\xac\x8c\x08B\xf8\xd5\x10\xff#\xc1\x80A\xb9\xb4\xf58D\xc4\x9cA\xe7\xff\xdd\x18\xab\xb2\xb3A\x8cC\x0bJ0\xbf\xc6A\xa3x.\x0b0\xa2\xd6A\x08\xfa\x84\xfd\xd2\xd9\xe3A\xf9\x04\x1d\x91\x0cl\xefA\xee\x0c\xffR\xf6\xd0\xf6A\xd5D~\x98\xe6\xfa\xfeA\x1f\xd3\xc7]\xa0\xed\x03B%y\xe0\x91Q\x95\x08B \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_asian_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike_type='FIXED_STRIKE',
            strike=66380.00,
            avg_method='ARITHMETIC_AVERAGE_METHOD',
            obs_type='DISCRETE_OBSERVATION_TYPE',
            fixing_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_asian_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_asian_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_digital_option_pricer(self):
        expected = b'\n\xf4\x06\t\xc3\xb1s\x8b{\xad B\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08l\xc9\x9c\xbc\n\x0fHA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x80qiw\x8d\xbc\x0f\xc2\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08W\x12"\x05\x1a1\xdfA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x00\x93,\xa2\xff9\xc1\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08_/\x02\'+\xbbO@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfd3Q\x91\x00\xabzA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x80.\x95y\x84J\xd0\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08v\xf4\x9e\xaa\xecvh\xc1\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x7fa\xb3]\xa9L\x94\xc1\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x080\x17\xbc\x14\xff\xea\xfbA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00kO\xe4\x13\xde\x91A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xd7\xbe\x9d \x1b:\xb4\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08 `C\x87\xe7\x91\xe0\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x93\x1e\xe0\x8b\xbd\x18\xd7A>\xf0y\xe9_r\xedA8I\xeaE\xa21\xffA\xfa\x08\xbb\xf1U>\x0cBg\x92\xb1\xb8.l\x16BA\xd5\xe4\xd8\xe4\xf4\x1fB\xbb\x87\xde\x0cl\xc6$B\xe9\xaf\x13f\x01G)B;y5\x12\xd7\x1c-B\xc3\xdc\xd3\xf5\xf8#0BRP\xb8\x06Ad1B}\xa95\xd9\xae5\xddA\x81\x97+\x98{9\xf1A\xbc\xa8\xfa\xa5\x92/\x01B\x0f\xa8\x1a\xfc\x1d\xbb\rB\xa0#\xea&\xe9\xeb\x16BR\x11\xa4=\'\x03 B\xaf\xe4\x02\xac\xb7\xa0$BR5\x06gW\xf4(Bi\x96\xbe]8\xbb,B\xb9\xa0\x14z\xed\xe4/B1z\xc4&\xd591B\xceoB\x83\xd3\xec\xe1A%\xde\xd1_\xc5\xb6\xf3A\x92\x89\xc5\x99\xb6\xb0\x02B\xb1\xbd\x86\x1a<%\x0fB\x98\xc5(N\xc1Z\x17B\x0f\xf0\x03j<\n BM\xca\xb2\x95\x98x$B\x18A^\xden\xa7(B\xd1\xf3\x9a\x89C_,BGy>J\x8c\x80/B`,\xbeN0\x101B\xc9\xd7<\xbfgj\xe5Aaf\t\xe5\x9b6\xf6A\\Q\xeb\xc8\xcc\x14\x04B\xdc\xca\x9d\x19\x92B\x10B\x1b\xb6\xa1\xf33\xc4\x17B\xab\xf5\x8c\xa7\x03\x14 B=\xc6F5{X$B\xc3\xeb\xbc\x19\xb7](B\x19gK\xd1\x95\x04,B\xdf\xb34V../B\xfe\xf8\xaa\xc5\xf1\xe60B<g\x11\xd5\x91\x10\xe9A\x88v\xe2\xb0y\xbb\xf8A\x87\xc6\xd9\xcc\xb0\x80\x05B\xca-e\x02\xb4\xd2\x10B \xda\xb6\xd5S0\x18BA]\xd9\xe1\xf7\x1d B1\x15d\x1b\xec5$B|\xb50\xc1\xc9\x1f(Bz\xa8\xfb=*\xb0+B\x00\xf7~}\x17\xce.B\x12\xb5\xc1\xf7\x84\xbf0B\xbb\x95?d\x19\xcb\xecA\xa8\xed\xf3,p(\xfbAN\xae2\x83\x1d\xcf\x06B\xa3\xe2\xa5\xc1\xc2o\x11B\xd9\x11<_\xca\x88\x18B\x8cYP\x8f[) B\xda\x14q\x020\x19$B\x91"]D\xbd\xea\'BP|`\x91\xf3a+B\x11\xe5c\x13$z.B\x96\xc9e\xb4\x17\x980By\xce\xb1m\xc9U\xf0A\xbbb\xa5\xeb\x9a\x80\xfdA\x8c\x9eY\xaaU\x1c\x08B\xc2`+\x11\xe4\xf3\x11B\xd4g7\x08_\xe1\x18B\x7f5_6>0 B\xb2\xdb\xd4\xb6\xc9\x04$B\x9b\x0fZ\x98$\xb0\'B/Q\xee-\xf0\x1e+B\x1f\xb2\xd3=\xcc+.Bl\xbfI/qr0B\xb4\xfa\xf3|\x8aA\xf2A\xd4C\x1e\xbf&\xdc\xffA\xa8nA\xcd<@\tB\x82\xe1\xd0Q\x8au\x12B\xe3\x02>\x87\xc61\x19B\x1e\xb99\x0c\x10= B\xf0/\x97\xec\xea\xed#B5\xec\xa1|\xe4\x80\'BX@U\x7f \xda*B=\xdf\xf2\xf4\xda\xe0-B\x95\xd37\xeb\x06L0B\xc8*m\xfe\x0f,\xf4A~\xf2\x1d\xf6t\x0e\x01B\xf7M|v\xffb\nB\x8b\xb9\x04(\x16\xef\x12B\x9e\x9cAg&|\x19B\xbdyS}-G Bj\xa5\xf2\xe7\xba\xd6#B\xd6\xa5\x8cm\x91W\'B\xf0\x98\x14\xb9p\xa0*B\x8c\xed\xd4e\xfd\x99-B7I\xc5\x15m(0BL\xb2!x\x82\x17\xf6AM<.T&(\x02B\x1e\xf7\x9e\xb1\x9cx\x0bBc\x9a:1\x8dc\x13B\x10\xdb.n\xf6\xc6\x19B%\xed\x84\x1e\xb0M BE\x1dp\xf17\xc8#B\xee\x1f\xecI\x83+\'BW\xa3?\x02@\\*B\x1a\xcb\xd7!qV-B\xb3\x9e\x8e\xbeu\x050B\xd3\xd5)\r\x93\x06\xf8Ao\xbaZ\xd9\xaa:\x03B\xa0\x11\xf2\xf1\xd8\x83\x0cB}\xfd\xc6\x93\xd2\xd1\x13B\x13\x88\xc7\xcd\xf6\x0e\x1aBR\x8f\ni;Y B\xd1=\xe3G8\xb7#B\xecc\xb5hx\x02\'B\xf2F\xcfe\xe2(*BZ\xbc\xe8\xd0U\x18-B\x8c\xe3\xcbe\xdc\xbc/B\xcf\xfa\x03\x0f\r\xe0\xf9A\x9d\xac\x90\xb068\x04BcU\xde\x9cq\x82\rB\xeb\xafDZn8\x14B\xde=\xcc/DQ\x1aB\\l\xf9\xee\x87a B@\x83\xe4\xed\xc4\xa8#B\xd9\xb5|\xa1/\xe2&B\x97\x95\x99\xfc\xa3\xf8)B\x00.\xc4\xa8\xd6\xd4,B\x16\xf3\x07\xd3\x13z/B \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_digital_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike=66380.00,
            cash=1.0,
            asset=0.0,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_digital_option_pricer(
            instrument=inst,
            pricing_date=self.as_of_date,
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )

        # print('test_cm_digital_option_pricer:', result.SerializeToString())
        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_single_barrier_option_pricer(self):
        expected = b'\n\xf4\x06\t\xde\xc1.\xf8\xe8\x0f\xedA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xdae\xef\xe6s\r A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xf0Y\x8c\x95\xc2\xd2\xdc\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xc0\xf9\x80]\xd7\xcf\xb4A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xe0!&\xae\x9c\x07\xc1\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\n\t\x9a\xd3D\x17C@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff#.\xa0o\x0bpA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xc0\xde\xa0-U\xcd\x86\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x16=Y\x04\x10\x10\xf6@\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\x17\xac\xdbzN"A\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08Q\xc3\x10\xf8\x7f$\x11B\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08 \x7f\xec\xa3G\xf1\xa5A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x83\xc4\xdd\xb5.\xe2m\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00f\x90\x8b\x07{\x98\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\xd1\x8c\x8c\xae\x0c\xb1\x8aA\xa1\x10\xec\x06\x03\x83\xa4A\tp%\x84\x9e\x02\xbaA\xcd\xea0V\x95\x08\xccA\xb7V8\xf5\xb5[\xdaA\x9fT$M:\x1f\xe6A\xf4\xa7_\x99-\xe9\xf0A\xcaW\xd3H\xe8\xe5\xf7AT\xfa\x8d\xaf,\xd3\xffA\x18\xc0\x98\xbbx:\x04BuX\xa1e\x1f\xc8\x08B\x87\xfb\\\xd8\xd8\xf9\x92A=\xe0\xddQ\x14\x8b\xaaA\xa4\xf2\x0e\x1d\xffD\xbfA \x87\x8d\x81\x9e\xdb\xcfAF\x8b4\xc4\xf3\xb9\xdcA\x1fe[\xaf\xd5d\xe7A\x82KRXh\x83\xf1Ar\xa3\x8e\x9b\\k\xf8A\xff\\3x\x90\x1e\x00B\xf5v2a8a\x04B 9\xda\x9eu\xe2\x08BRB\x0f\xfeE\xd3\x99A\x9e\xe6\x7f\xe9\xbb\xa9\xb0A\xbay\xdc\x14]m\xc2A\x99\xfd\xed\xf7\'\xe2\xd1Aw\x01\x8b\xad\xc5\x18\xdfA\x97\x1f \x8e\x04\xa9\xe8A^;\xe3\x18K\x1e\xf2A\xd3\x0c\x8d\x01\xab\xf3\xf8A\xfb\xd5]\xe0\tV\x00BU\xc4!\x94(\x8b\x04B\x9b\x16\xb4\xea"\x00\tBv\x19[\x92?\xf4\xa0AX.h\x8f8h\xb4A\xd1\xab\xe34D]\xc5A\xfb\x9e;\x19\xc9\xdf\xd3A\xd6)\xb3d\x0f\xbc\xe0At7\x1f\x0f\x0e\xec\xe9AF\x0f\x84\x93\xc0\xb9\xf2A\xe6?f!v~\xf9A\xc6`\r\x8b\xbe\x8f\x00Bs\x114\x83\t\xb8\x04B\x8b\xc7&\xae\x02!\tB\xf5\x84@\x9e\xbc\x9b\xa5A\xad\x06.\xd9\xe9z\xb8A\x8e\x02\xc3\xb8\'n\xc8A\xeekr\xb9r\xe5\xd5A\x02=:h\xfb\xeb\xe1AxQ\xd1\x87(.\xebA?\xbf4k\xb6U\xf3A\x94\xb5\x7f\x08o\x0b\xfaA\x7f\xf6acr\xcb\x00B\\\xf2w_\x9e\xe7\x04B\x7f\xdd\xbb\x97\xebD\tB\x07T\x9f\x0e\xf4\xdc\xaaA$2\xd3\x8e\xc8\xdb\xbcA\xd7\xb0\xaem\x80\x9c\xcbA\xf3\x88\xa9\xda\x1e\xf2\xd7A\x04\x15\xb8\xab#\x1c\xe3Aw\xd1r\xd1}o\xecA\xd7x\xc5\xbf\x1c\xf2\xf3A\xb9\xbdO\x9aR\x9a\xfaA\x93\xa8.p\xf0\x08\x01B\x17$r)\xae\x19\x05B\x8e\xfc\x86\x02\xb2k\tBS\xfc&\xc0\x9fY\xb0A\x83\x9d\x88\xc1\x83\xc2\xc0Ay\xdd\n\xec;\xe5\xceA\x96\x1abF\xf4\x04\xdaA\x95=\xb19\x85L\xe4A\xde\xcd\x18i.\xb0\xedA\x8d\xe6\xcd\xcd\xe5\x8e\xf4Aq X\x7f\xe7*\xfbA\x95I\xb6\x01\nH\x01B\x04\xea\x08\x0c\x04N\x05B~\x0f$\xac)\x95\tBl\xa0KX|\x8c\xb3A\x0f\xea\xd2\xc3\x958\xc3A\x8f\x8eg2\xd6"\xd1Arw?\xd2<\x1d\xdcA\xe5o1\x0f\x1d}\xe5A\x8c\x17\xb9\xb6S\xf0\xeeA\xf9\x1bq\x9c\x05,\xf5A\x7f\x8f\x1az\xfc\xbc\xfbA\xc4$\x04\xed\x95\x88\x01Bt\x190no\x84\x05B\x19\xe6s\xe4&\xc1\tB.N\xafT\xee\x03\xb7A\xb5}\x7fr\x8b\xcd\xc5AI\xa4w\x18\xbd\xdd\xd2A\x11:\xb2\x1b^:\xdeAw\xf0\x1b\x00\xe8\xad\xe6A\x86\xb5\xf9\xdc\x00\x18\xf0AkQ\xc7\xb7q\xc9\xf5A\xd5\x8b?\rgP\xfcAv\x90\xca\xd9o\xca\x01B@qj\xd7\xc3\xbc\x05B\xa0\xe0/X\x7f\xef\tB\x92_I\x11\xab\xbc\xbaA\xaf\xe0\xc6r\x06\x7f\xc8A\xea\xa2\xfc\xa7K\xa2\xd4AC\xab\xf6\x05\xea-\xe0A\xb97$\xb4\xe2\xde\xe7AF\xbc%$\xa4\xb7\xf0A\xd36Y\xf6 g\xf6Af\xb9db\x02\xe5\xfcA\xa1\xd8(\xa4w\r\x02BJ\xef\xfa\xb9\xd8\xf6\x05BQ\x94\xff\x90\n \nB\xf6jq\xf3Y\xb3\xbeA\xcb\x07\\D\xd8J\xcbA:\x84\x95_\x9ao\xd6A\x922\xeaS\x96@\xe1A\xdc\xe7\xd5\xae\t\x10\xe9A\xc5+\xb8\xfd\x19W\xf1A\x1a\x86\rH\x0b\x05\xf7A\xa2\x08~d\xaez\xfdA\xebB\xd6\xd1\x90Q\x02BW\xd44/\x892\x06B\xf2x\xc6?\xa2R\nB<\x9f\xea\xe7Sr\xc1Ac\xfch2\x00/\xceA\x8a\xde3\xbb\xdcD\xd8A\x0f\xaf\xbbi\x02U\xe2A%\xe0\x8b[YA\xeaA\x1a\x8d\x98rg\xf6\xf1A\xb0F\t\x8d)\xa3\xf7A\xfd>y\x03O\x11\xfeA\x9dBA\x19\xa2\x96\x02Br\xe8\xf2\xad\xb3o\x06B\xe2\xed\xd9`"\x87\nB \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_single_barrier_option(
            payoff_type='CALL',
            strike=66380.00,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            barrier_type='UP_IN',
            barrier_value=66380.00 * 1.05,
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[], [], []],
            payment_type='PAY_AT_MATURITY',
            cash_rebate=0.0,
            asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_single_barrier_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_single_barrier_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_double_barrier_option_pricer(self):
        expected = b'\n\xf4\x06\t5\xeaI\x12\x9f)\xedA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08l\x1e\xa6%\xdd\xe2\x1fA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08`\xc81\x96B\xec\xdc\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08lo\xc1A\x83\xab\xb4A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xc0|\xe3\x91\xb1\x07\xc1\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08/\xc52W=\xe4B@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfe\xae\x055\x19\xc1oA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00f/\xee\xcb\x8f\x86\xc1\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x081\x96\x9f\xf1* \x01A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\x93\xd8\xc0\x86k,A\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x10\x1c:\x81\xf4\xf4\x10B\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\xd2\x87*l\xb4\xa5A\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\r\x9b\x87\xb1\x08{m\xc2\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\xbap\xbc\x87&\x98\xc1\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08ld\x0c\rvz\x8fA\x8d\x1a\xcc+a\xfd\xa6A\xf4\xec\x94X\xec\x04\xbcA\xd7\xa0~\x95\xf1P\xcdA\xb1xd\xc6\x0b\xfc\xdaA\x02\x06oE\xa3T\xe6A\xb3\x17\xa4\xe7\xe8\xee\xf0A\xcaW\xd3H\xe8\xe5\xf7AT\xfa\x8d\xaf,\xd3\xffA\x18\xc0\x98\xbbx:\x04BuX\xa1e\x1f\xc8\x08B\xd1\x907\xa6\xd2\xab\x95A\xac\xdeszJ\x17\xadA\x9f(\xc0(\xfd\x98\xc0Af6\xaa\xdc?\x83\xd0Am\x89k\x976G\xddA\xc9\x96Um\x89\x93\xe7A\xb8\xd1\x87\xe7h\x88\xf1Ar\xa3\x8e\x9b\\k\xf8A\xff\\3x\x90\x1e\x00B\xf5v2a8a\x04B 9\xda\x9eu\xe2\x08B\xd6\x85\xc6\x0f\xc0\xc3\x9cA0V%\x8e\xb4\xf2\xb1A\x9b\x1f\xf2\xa3\x88W\xc3A\x16\x0fp[\xf9i\xd2Aw\xce\x7f\xada\x95\xdfA\xe3\xd8`\xb8\xd0\xd1\xe8A\xc0\xb3\t\x17\xa6"\xf2A\xd3\x0c\x8d\x01\xab\xf3\xf8A\xfb\xd5]\xe0\tV\x00BU\xc4!\x94(\x8b\x04B\x9b\x16\xb4\xea"\x00\tB\x8a\xd4\xf9\xd29\x84\xa2A\xc0\x8es\xee1\xaf\xb5A\xc0\xe2\xfbFD:\xc6A\xc5\xb0\xe4E([\xd4A\x16\xf5\x91z\x19\xf3\xe0Ao\xa5\xfex\xba\x0f\xeaA\xd8\xc8a\xd3\x8b\xbd\xf2A\xe6?f!v~\xf9A\xc6`\r\x8b\xbe\x8f\x00Bs\x114\x83\t\xb8\x04B\x8b\xc7&\xae\x02!\tB\xa0#\xef_U<\xa7A\xd8E\xda\xb8#\xbc\xb9A\xf6\xc5\'\x9d\xb7=\xc9A\xf4\xbf\x15v\x88U\xd6A`\xb8F\xca\xa9\x1c\xe2A\x12D\x08\x92hM\xebA\t\n:\x07\x06Y\xf3A\x94\xb5\x7f\x08o\x0b\xfaA\x7f\xf6acr\xcb\x00B\\\xf2w_\x9e\xe7\x04B\x7f\xdd\xbb\x97\xebD\tB4\x83C\xa9\xbf\x87\xacA2\xb4\x07Y{\x14\xbeA\xb3/$\x8c\xc2^\xccA\xba%\x12\xfb\x04X\xd8A\x90$\x94n:G\xe3A\xc4}\xe7a\xeb\x8a\xecA\xf6\n\xf8$\x02\xf5\xf3A\xb9\xbdO\x9aR\x9a\xfaA\x93\xa8.p\xf0\x08\x01B\x17$r)\xae\x19\x05B\x8e\xfc\x86\x02\xb2k\tB\xafM\x0f\xa3U1\xb1A\x8f\x8e/\xd1\x9dY\xc1A\xf5{\xb5T\x97\x9a\xcfA\x18\x81P0\xb0a\xdaA\xb95nA\xa9r\xe4A\x0c\x17\xe0RJ\xc8\xedA\x8fe\xa3\x97o\x91\xf4Aq X\x7f\xe7*\xfbA\x95I\xb6\x01\nH\x01B\x04\xea\x08\x0c\x04N\x05B~\x0f$\xac)\x95\tB\'\xbag?*d\xb4A\xa9.\xad\xf5\xc8\xc9\xc3A;\xc7\'\xbfYw\xd1A|aCD\xbdq\xdcA\'\xd3\xfap\xd9\x9e\xe5A\xe5;\xf4\xb7\x88\x05\xefABx\x82\xd8?.\xf5A\x7f\x8f\x1az\xfc\xbc\xfbA\xc4$\x04\xed\x95\x88\x01Bt\x190no\x84\x05B\x19\xe6s\xe4&\xc1\tB\xbe\x8e\x02?\xa9\xd9\xb7A7\xf2\xce\xc6rX\xc6A\xe01\x1c@l,\xd3A\x84\x81m\xeaz\x87\xdeA\xd2\x1bk\x0c\xb3\xcb\xe6A\x0c.b(T!\xf0A\xa4@\x7fPf\xcb\xf5A\xd5\x8b?\rgP\xfcAv\x90\xca\xd9o\xca\x01B@qj\xd7\xc3\xbc\x05B\xa0\xe0/X\x7f\xef\tB\xcaz\xd2\xbb\xeb\x8e\xbbA\x96\x14ZIp\x03\xc9A\x8f\xce\x11P\x82\xeb\xd4A\x87,E\x80\'Q\xe0A\xf4\xbb&I"\xf9\xe7A\xb3\xcd\xdc6\xd5\xbf\xf0AOU\x17!\xd8h\xf6Af\xb9db\x02\xe5\xfcA\xa1\xd8(\xa4w\r\x02BJ\xef\xfa\xb9\xd8\xf6\x05BQ\x94\xff\x90\n \nB\xdel\xa7\x1a\xf1\x80\xbfA\x97\xd2\x90\xd0\xb9\xc8\xcbA\xec\xab\xf3"\xb6\xb3\xd6A\x9e\x90R|\xd9`\xe1A\xd0\xfe\x02\xb6\x16\'\xe9A>l\xf2+H^\xf1A\xd9\xf2-\xe6\x8b\x06\xf7A\xa2\x08~d\xaez\xfdA\xebB\xd6\xd1\x90Q\x02BW\xd44/\x892\x06B\xf2x\xc6?\xa2R\nB\xdeH\xe3\xe2X\xd6\xc1A\x93\xd7\x9fWk\xa6\xceA@+\xcb\xf09\x84\xd8A\\lbq\x98r\xe2A<]H|\x82U\xeaA^\x84\xb0\xaf\xad\xfc\xf1A\xa4\x9eIzy\xa4\xf7A\xfd>y\x03O\x11\xfeA\x9dBA\x19\xa2\x96\x02Br\xe8\xf2\xad\xb3o\x06B\xe2\xed\xd9`"\x87\nB \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_barrier_option(
            payoff_type='CALL',
            strike=66380.00,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=66380.00 * 0.95,
            upper_barrier_type='UP_IN',
            upper_barrier_value=66380.00 * 1.05,
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[], [], []],
            payment_type='PAY_AT_MATURITY',
            lower_cash_rebate=0.0,
            lower_asset_rebate=0.0,
            upper_cash_rebate=0.0,
            upper_asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_double_barrier_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_double_barrier_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_one_touch_option_pricer(self):
        expected = b'\n\xf4\x06\t\x0f_\xd3-\xb1\xe2%A\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x089\xa8\xbb\xd8\x16\xacS@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08p\xf4[\x80\xa4\xb4\x15\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\xe40\x056\x81\xe9@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xe0&i\x00\xc8A\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xe5Z\r\xdf\xcb\xf8b?\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\xd7\x9f\xd8\xa6\xe3\x8f@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x9b:\xba\xe5\xaa\xa6\xc0\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08V0\xe29v\xe7r\xc0\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08~\x91\x85>\x16_\x9f\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xc3\xb5\xacf\x85K1A\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08`\x07\x06\x1d:#\xc6@\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xca\xf2\'\x98\xa4:\x91\xc1\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x01\x17C\xbce:\xbc\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x97\xbf\xe8\xa3\xe0\xe2\xcd@X\x16k\xaa8M\xe6@\xbd\x06\x9b\xbc\x11D\xfb@5\x9e\xc5\xa7\xda\x19\x0cA\x9fK=\x1e\xc2\x08\x19A\xb9\x15(\x96D\xb6#At;\x11\xbd\xfc\xfd+A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x92%\xfb6\'\xcd\xd4@\xf4G\xd2Xf+\xec@\x7f\x19\x99\xa5\xe5\xe7\xff@P\xd7\xa3\xea\x96\x02\x0fA4\x90\xd1B\x06s\x1aAl9\xc9\xe9\xa11$An\xae\\o\xc8\x18,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xa5O\x1d7\xa4\xad\xdb@\x06i\xd2\x93,>\xf1@\xb0\xbc\xd0u\xe1I\x02A\x1d\x8cU\x07\x19\xe6\x10A\x1fu\xf7J\x88\xc2\x1bA\x97,1F\x94\xa1$A\xd9\x96t\x05\xe60,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xb4_\xb6Q\xea\xbe\xe1@^\xea(\xfa\xe6\x92\xf4@\x07t\x04\x04e\x9d\x04A!\x9fxm\xb6:\x12A\x9f[\xe1r\x9c\xf9\x1cA4\xdb\xcd\xca\x93\x07%A\xb5\x016I\xb7F,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x00\x08}0\xb0\x11\xe6@\xc6\xd1G/\xdd\x07\xf8@xd\x9a0\x8d\xe9\x06A\x93I\x87\xaa\x07\x7f\x13A\xe7\xfflVp\x1a\x1eA\xee\xbe=\xac\xdcd%A\x03?\x8cJ\x8cZ,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AP\x96*M\xa7\xc0\xea@w6\x96yn\x92\xfb@\x13\xd1z\xdc\xa9*\tA\xef\xfdK\x14J\xb3\x14A\x91\xd1D\x17\x05\'\x1fA\x13O\xb0?z\xba%A\t\xd0f6\xa7l,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xcdM\x92\x81\x05\xbd\xef@\xa5\xb6\xf3\x18t)\xff@G{\xaa\x88\x0f^\x0bAx\xd4_\xb2\xf5\xd7\x15A{(\xe8\x0f\x97\x10 A\xaa\xd2\xc5\xc7O\t&A8\x10\xf58?},A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x7f\xef\x00\x0b5|\xf2@Z\x82\x13\xb0\x9bb\x01A\xc5aG\xe1\xe2\x81\rA\x06\x0e\x82T\xa7\xed\x16A o\x93>I\x85 AL\xc5\xfb}\x1fR&AkK]\xaf\x82\x8c,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x12\xf26\xac\xaa2\xf5@\xb9\xbdQ\x0b\xab/\x03AD/:X\xec\x94\x0fA\t\t@\x12\x11\xf5\x17A\x1c\xe9\xc2\xcaW\xf2 A\x9e+\x7f9\x90\x95&A\xf0\xf0B\xd8\x98\x9a,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xc4\x896H\xb7\xfb\xf7@=\xbd-+O\xf9\x04A>\xf4e\xc59\xcb\x10AH\x8f:u\xef\xee\x18Ac\xc1>?nX!A\x9e\xb0\xef\xfe1\xd4&A\xad\x05z$\xa3\xa7,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A%\x10d\xb1\xd2\xd1\xfa@\xb2\xe4X\xdfs\xbd\x06A\x1d\xc1.\xcf\x10\xc3\x11AHVX\x02\x02\xdc\x19A[\x0c\xcf\xc1&\xb8!A,\xdd\xcb\xb3\x81\x0e\'A\xffr\xc7?\xbe\xb3,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xdc\xf8\xe5\xd1\x1c\xb0\xfd@\xe4\xff~\xbdvz\x08A"?b\xc8\xf4\xb1\x12A\xd0\xc5\x86.\x06\xbd\x1aAq@\x15\xcf\x0b\x12"Aq\x92)#\xecD\'A\xe49\xfe\xe2\x02\xbf,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_one_touch_option(
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            barrier_type='UP_IN',
            barrier_value=66380.00 * 1.05,
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[], [], []],
            payment_type='PAY_AT_MATURITY',
            cash=1.0,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_one_touch_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_one_touch_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_double_touch_option_pricer(self):
        expected = b'\n\xf4\x06\t\xcc\xa6g+\xbb!.A\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08G.\x98\xcd\x16P ?\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x080\x14\xba\xd9T\xe2\x1d\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\xff\xbf\xf6;&\xb5?\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00`\x1e\xca&{H\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x93u\xe1\x7fu\x95\xd1>\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfd\xff\x07\x0b`\x8e\xfd?\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x00p\x9f\'\x0c\x18\xc0\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xe7\xdc\xb0\xab\xe1[\xa0\xbf\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\xff\x9f\x15\xca%\xcb\xbf\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\xfcP\xb0\xfc\x06\xa8@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\x80\x0bqC\xc1>@\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x80a\xcfS\x1b\xc7\x18\xc1\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\x00s\xe6ELD\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xe4\x02\n\xb2%\x1d.AEt\xb5\xca\xea\x12.A\'\x886q]\x1d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.Ag/ \x02\xa1\x1f.A$\x07\xc9\xed\xc1\x1a.A\x18\xce\xd3\x8d\xbb\x1f.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xe90\xa4\\\xda .A4oG\xe7\x9f\x1e.A\x0b\x9a<\x81\xe6 .A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xb6\xa3\x80un!.A\x0c5\xf9\xc8s .A_\x91\x7f\xcbs!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xbe\x17\x8d\x8b\xb1!.A0\xba\xc2\xbaG!.A\x03\xdb+\xcc\xb3!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xf4\xcf\xa2\xad\xce!.A\xeb\x04\x02\xc5\xa3!.A\x9bEu\x97\xcf!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AAi$\xcf\xda!.A}K3\x18\xca!.A\xbe\x9e9*\xdb!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x19"@\xa7\xdf!.A\xaa\xa7\x0ff\xd9!.A\xcc{U\xc9\xdf!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A#\x11:\x82\xe1!.A\xe9\x0f\xa6B\xdf!.A\x9f\x8fz\x8e\xe1!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x8c\x1e\xbd0\xe2!.A\x86y\xfbi\xe1!.A\x8d3\xf84\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xe5\x81@n\xe2!.AK\x07R,\xe2!.A\xad\xc9\xa7o\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x9bS\x0e\x83\xe2!.A\xef\xd4\x0bn\xe2!.A\x9a\xd0\x80\x83\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_touch_option(
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=66380.00 * 0.95,
            upper_barrier_type='UP_IN',
            upper_barrier_value=66380.00 * 1.05,
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[], [], []],
            payment_type='PAY_AT_MATURITY',
            cash=1.0,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_double_touch_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_double_touch_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_single_shark_fin_option_pricer(self):
        expected = b'\n\xf4\x06\t\xf3-\x8e\xe9|\xffs@\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08%s\x8c=\x8fl\xb3\xbf\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x080~\xd8\xc8~\xccd\xc0\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xaf\xfdK\x82\xd8.I\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00`2\x96\xd3\t\x91\xbf\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xdf\x03bo~\x11\x15?\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08/\x12\x13\xbe\xe2\xb4A@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08 \xa9g\xd0\xc4w)@\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08H.`zwq\xe9?\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x1fE~\x8b\x8c\x1c\x15@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08I\xfb\xb42\xc2\x19\xaf\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08l:\xda\x86\x86\xe7C\xc0\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xd0\xe3w\x991\x0c\x0fA\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08`\xa7\x9f\n)o9@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x1f-E\xcd:Cg@c\x8e\x9d\xc5\xb1\x93x@\x14\x88\x97\xc1\xc6n\x84@\xd9<QD\xee\xde\x8a@?U\xc4\xbe\xf5|\x8b@F\xcc>C\x83\xa5\x83@q\x80S\xc0\x89\xc7h@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x9e\xce\xdf%\x9e\xb4j@\x85f\xd6\x12x\xc8y@\xe9\xdf\xce}\xb2\xfb\x83@\xadf\xc6<\xfd\xf2\x88@\x83\xa2wX&\xa2\x88@\x9eL\xa3h\x11-\x81@\x8c\x96`fG\xeee@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x9fG]n\x94\xb1m@\xff~<q\xcc\x87z@\xf3\x83\x87\x0b\x84\\\x83@\xcc\xff&\x94\xef\x1e\x87@\xde3O\x06\xd4%\x86@\xb6?\x04Rv4~@\x95\x88\n#>\x8ec@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00P-\xa7S/\x17p@\x85\xb7(\xfb=\xe3z@\xa8\\1\xcd\xba\xa0\x82@\xc8\xa7]QJh\x85@1\xef!\xef\xe0\xfa\x83@\xe0&\x0f22\xb2z@~\x8d\x86\xcbe\x8ea@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xa5\x91t*\xed\x14q@\xd7\xeev=\xa2\xecz@=\'\x10\xaa&\xd4\x81@\xf2A\x82\xf7Z\xd1\x83@\xea,\x0e\x1f\xc7\x15\x82@\xa2g$\x02s\xb4w@\xa3\xe9\x96\x97z\xb7_@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x04\xe0N9\x12\xd5q@\'\xb5\xbd\x89\x9c\xb4z@\xdaL7J\x8a\xff\x80@\x15\xcf_xeZ\x82@\xb8\xa6\xecU\x91l\x80@\xc2\x9c\xeb_\x0b#u@\x14\x0es>R\xcf\\@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00,\x074\xb8\xe0\\r@NW\x8c\xfc\xe3Iz@.\xd5H\x1b<)\x80@Mx\xa9@s\x02\x81@\xb9\x93P\x05w\xed}@\xb7\xcf\xb5\xb8\xe5\xear@y\xdb3\xf5\x96MZ@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00=\xe4\xe1#\xbc\xb2r@\xd4\xbbmZ\x08\xb9y@\\\xf0k\x0fg\xab~@W`\x88I\xb2\x8f\x7f@\x9a\xce\xb6\x96\x0eZ{@J\xfbz@\xd0\xfcp@\xa8u(\xedi X@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x001\x8a\x88-e\xddr@\x00^\x0fMy\x0cy@\xac\xf7\x08\xe7\xf7\x0f}@-\x10\xed~!Q}@\x8b\xbd\xae\xc6\xa0\x12y@"i\r\xa84\x99n@G\xb6^\x13\x9d9V@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x8f\xb31\x92z\xe3r@\xeb\x01\x15\xbe\xb5Lx@8\\@\xdc#\x84{@\x9b\x19\xb4\x90\xe0D{@\x04\xa2\xeb\x82\xc3\x0cw@U\t\x8a\x83\xd4\xa0k@h\xd8\xd7\x88\xd2\x8dT@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00B\xcc\xa9\xaf,\xcbr@\xd7/d\xef\x8a\x80w@\x9b\xe7\xe2KW\nz@\x96g&\x84\x9cfy@\xe2\x1d\x1df\xa2?u@&\xd7rmu\x00i@\xd0\x16\xd1\xc5\xdb\x13S@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x16\x08\xd0Q\x13\x9ar@a\xb0\x8deT\xadv@IA\xa4\x7f\xdd\xa3x@\x967(1*\xb2w@"\x9e\xd8M\xbf\xa3s@jD\xed\x14\xf2\xaaf@\xa2\xe8\xb7\x18>\xc4Q@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_single_shark_fin_option(
            payoff_type='CALL',
            strike=66380.00,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            gearing=1.0,
            performance_type='RELATIVE_PERFORM_TYPE',
            barrier_type='UP_OUT',
            barrier_value=66380.00 * 1.05,
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            payment_type='PAY_AT_MATURITY',
            cash_rebate=0.0,
            asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_single_shark_fin_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_single_shark_fin_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_double_shark_fin_option_pricer(self):
        expected = b'\n\xf4\x06\t}\xd5\xf9y+M=\xc1\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08~z\x13Y>\xf8K@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xc0\x8aj\xbb\x01\x98\r\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xa8\x93\xa4j\x92!\xe2@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x80-\xc6C>8\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xa8\xfcK\\3A(@\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08+\x95\x8bVobTA\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xa0\xado\x00O\x83\xfa@\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xf4=\xd4\xc2\xea\xa5\x90\xc0\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08>9\xeb\xbc\xa6\xa0\xbb\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xbe\xe9\x0c-\xb9\xfc\x87\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xf3 \xaa9 \xb4\x1e\xc1\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xef\x8e\x18\xeb\xc9\x9c\xf0A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xaf\xa1A\xe2\xc47\x1bA\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x005C>\x92!GOA\xec\xbf\xb8\xa1\xb2\x07\\A\xf2\xf2\x8e\xeb&\xcbNA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00H\xef\xc4a\x92\xd2CA\xe9[?\xff\xe07KAk#:5\xe0\x8dCA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00#\xb5i*\x84q9A.\x94s\xd4\xf6\x9f2A\xa6?V(w&9A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x001\x05H\xff*\xc70A3\r1Z|,\x0c\xc1\xcc\x8e;\x83^\x9e0A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xbc\xc0e@x\x19\'A\x99\xb5\\B\xd3S1\xc1kx&\xec\xb5\xec&A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x86|6\xccy\xd2 A\x02>nz\xf0%:\xc1\x7f`\xa5\xfaE\xb9 A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00U3d\xea\x8e\x14\x1aA/\xc3\xc6B\xae\x05@\xc1\xf4\xbe\xfc\x07\x1a\xf7\x19A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00O\xa8\xc4\xe3\x1bt\x15A\n\xf44A" B\xc1g\x96\xa6zQb\x15A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xc1\xd1\xeb\x8c\x10\x8a\x12A\xc5\x9a+\x87\x13\xbfC\xc1\xcb\xd4I\xdca\x7f\x12A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00Z\xc0\xc0\x80\xe9\x9e\x10A\x9b1\xc4\x13\xe6\x17E\xc1\xf6\xbd\xbb19\x99\x10A\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x1a\xb9R\xcb\x91\x0eA\xc9\xe5\x14\xe1\xd8HF\xc1\'c\x177k\x8e\x0eA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x8c\xdd-.\x8f\x97\x0cA#\xdc\x7f\xdb\xc5bG\xc1\xac\xb5\x16v?\x9b\x0cA\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_shark_fin_option(
            lower_strike=66380.00,
            upper_strike=66380.00,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_participation=1.0,
            upper_participation=1.0,
            performance_type='ABSOLUTE_PERFORM_TYPE',
            lower_barrier=66380.00 * 0.95,
            upper_barrier=66380.00 * 1.05,
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            payment_type='PAY_AT_MATURITY',
            lower_cash_rebate=0.0,
            lower_asset_rebate=0.0,
            upper_cash_rebate=0.0,
            upper_asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_double_shark_fin_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_double_shark_fin_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_range_accrual_option_pricer(self):
        expected = b'\n\xf4\x06\t\xc4\x911\xb7ez\xb2@\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfbk\xc6*\xbbd\xa2\xbf\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08 \\\x14q\x84S\xa2\xc0\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\x0c\xf5\xdb\xcb\xd87\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x80\xb8\xff\x94\x06\xce\xbf\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x94\xb6\xa4\x9eKJ%\xbf\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08~\xc1\xf2\xac\x9f\xe4Q\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x19Fd8\x8d9@\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x9c\xd4\x12\x86?\xce\xdc?\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\xa3\xf5J\xc8\xe6\x07@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08P2\xd4\xdb\xa4k\xd4\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x90\x9c\x0fab#j\xc0\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xda\x01y\xfdc\xa01A\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x01W\xe3\xc9\x19\xe1\\@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x9e\xdf`\x1f0\xb3g@%m7\n\xbag\x80@+P\x0e\x8fi\'\x93@]\x14Y\xd8j`\xa3@8\x8e\xd1\n?\xf4\xb0@\x94\xdaQ&4F\xb4@#\x88on\x82V\xb1@\t\xd5!!!\xef\xa5@\x96a\x89\r:\xe6\x99@\x9f,\xf6|A6\x8d@\x06I8\xbd\r\x8b\x7f@R\xa2;\x84\xf1]l@\xb3\x1eCP\xc24\x82@\xacuFJ\x85\n\x94@\xba"\xedU\x85a\xa3@\x94\xe88\xfc\x11j\xb0@h s\x17\xe4{\xb3@F\xaf\xf0\xa4j\xcd\xb0@0k\t^\xda\xe0\xa5@\xa5\x90\xd4\xc6\r\xc4\x9a@\x04\x0b\xeb\xa3\xebs\x8f@\xc3\x9d\x85yh\xc9\x81@\x8a\xee\x83\xb1\xc0\x8ep@(U\x86s\xcc\xe5\x83@%=\xa9\x01\x08\xcd\x94@\x1c\xd9\xf8\r\xc2T\xa3@\xe3\x19\x15@K\xce\xaf@\x14\x93\xe6\x91\xbf\xbf\xb2@\x12\xcb \xdf\x16K\xb0@>\xda2?\xc3\xc4\xa5@\xdf\x05\x8eV\xdaz\x9b@\x84\xb3;\xab\x12\xbd\x90@\xc6N\xe3\x80\x02\xb7\x83@\xc7\x07B\xb2\x88\xeer@\x92t\xd2\x83\x87x\x85@D\xdf\x8d\xa6\xfcq\x95@vex\x8b\xce<\xa3@^\xe0\xf5\xbc\'\xd6\xae@\xda\x18\x05\xbev\x10\xb2@\xc9\xac\x00~s\x9e\xaf@\xe2\xd5\xbd\x85\xae\x9d\xa5@d\xaau\xe4.\x0f\x9c@\x17\xec\x1e^>\xa5\x91@2B8O\xfc\x89\x85@.=5;\xc6Eu@?\xe9\x9a\xb5\xf6\xeb\x86@\x02\x9d\x0fSb\xfc\x95@\xefI6\xdb\xe1\x1b\xa3@\xbe\x88\x85\xf6\xec\xea\xad@\xa2R\x05\xad\xdbl\xb1@\xd7\x0f\xbd\xcc\x01\xb3\xae@c\xa1\xa99\xedm\xa5@B\xadb/A\x85\x9c@\xdf\x1e\x12\x1b\xc1s\x92@a\x05\x99"\xa6?\x87@V\x96\xe0\x16\xb8\x8dw@,\xb0\xd4\x01\x18@\x88@\xa6\x8f$\x13\x13o\x96@\x9e\x93\xea\x0b\xd0\xf3\xa2@\xeas\xfd\xa7\xd2\x0b\xad@\xd2\xe5\xd6\xba\xdf\xd3\xb0@\xee\xe3\x05~0\xd3\xad@\x1e\xe3\xf3\xf9d7\xa5@[\xb8\x93\x8e\xe2\xe0\x9c@\x00q\xc8S/*\x93@ \x11I\x0c\x9d\xd6\x88@\xc0A\xf8\x9a:\xc1y@\x10\x84"\xcd\x9bu\x89@\x08\xe1\xba\x13\xb6\xcc\x96@?\x07M\xf6\x1a\xc6\xa2@\xa2r\x9f\xba\x178\xac@\xfa\xcf\xb5\xb9\x90D\xb0@D{\x16eX\xfe\xac@\xee]m\xf7\xa2\xfb\xa4@*\xb3O9~%\x9d@<*\xe91D\xca\x93@n\xe3\x0e\xf4zN\x8a@\x92\x02\xdb\xe3\x90\xdc{@}T\xbd\x1f\xac\x8d\x8a@f\xb92>\xb9\x17\x97@\xa7a\x01[\x00\x94\xa2@\xea\x15\xe0\x15\x04o\xab@\x9b\xb6\xcb\xf5+|\xaf@q\x88\xdes\xd63\xac@\xf6G\xa5\xbf\xeb\xbb\xa4@\xf4\x01\xa3\xdb\x1cV\x9d@\xe9|o^\xc7U\x94@\x18 ]8\x94\xa7\x8b@\xe9\xad\x96\xca+\xdd}@-\x1c\ne\xc1\x89\x8b@1m\x13\xbdOR\x97@\xadM\xc3\xc5\x85^\xa2@\xd7`\x90\xad\xe9\xaf\xaa@\x12\x7fk)[\x7f\xae@\x01X*\xb9\rs\xab@\xc7\xd5\xa8CHy\xa4@\xea\t\xcb\xa9ku\x9d@\xc2\xad\x16\x88{\xce\x94@0}rt\xc1\xe2\x8c@ly \x80s\xc1\x7f@j\xa5\xe5\xcb\x81k\x8c@\xef+Sgs~\x97@\xc0\x9cUQ\x82&\xa2@\xdea\x19\xe4$\xfa\xa9@Y\xee^\xa5S\x91\xad@\xe6\x8f\xe6yh\xbb\xaa@\x88\x03\xcd\x9a\x904\xa4@N7\xbc\x17\xc4\x85\x9d@\xf3?\xb1\xd4\x126\x95@\xc6\xf8p\x986\x01\x8e@\x87\xc7#[K\xc4\x80@\xf7\x93\xbc\x92\xa94\x8d@I\xe3\x02\xf6\xe7\x9d\x97@\xe4\xc8\xfd\xcd\xa6\xec\xa1@\xd6\xc1s\xa5\x1cM\xa9@\xd0e2\xc9\xdd\xb0\xac@\xb1&\xe9\x00Y\x0c\xaa@R0\x05\xcdt\xee\xa3@\xa5\x0e\x9f\x954\x89\x9d@\x7f\xc1\xe1\xf7\'\x8e\x95@\xfcn.Za\x04\x8f@\xf5\xe5x\xf4/\x99\x81@Cwf\x19\xfa\xe6\x8d@\xe0\x7f\xca\x07?\xb2\x97@\x07\xe4\xb1p\x84\xb1\xa1@\xf1\xbb\x9f&B\xa8\xa8@$#\xc2\xa1\xe1\xdc\xab@\xd9u\xa8\xc3Ye\xa9@.D\xcf\x0e\x84\xa7\xa3@\x13\x19w\xdb\x88\x81\x9d@Cz\x97^:\xd8\x95@\xf7A\xc5t\xd1\xed\x8f@ \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_range_accrual_option(
            expiry_date=datetime(2020, 8, 19),
            delivery_date=datetime(2020, 8, 20),
            cash=0.01,
            asset=0.0,
            lower_barrier=66380.00 * 0.95,
            upper_barrier=66380.00 * 1.05,
            obs_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_range_accrual_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_range_accrual_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_airbag_option_pricer(self):
        expected = b'\n\xf4\x06\t@\x0f\xa4\xfd5c\xdeA\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x86LzIy\xea*A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08@\x83\xc3W\x06\xd5\xcd\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08N\xd4\x94\x1f\xb2r\xc1A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x80\xae-@p\xf8\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xcf^\xb4\xba\xbc\x056\xc0\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x80\x152\x15(\x82b\xc1\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\xecN\xc4\x7f\x85vA\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08L?\xe9\xaf\x84\x12BA\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xdf\x91\xbd\x06\xb5\xfdmA\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08{\x9bd\xaat\xcc\xfb\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x080\xab!\x1b\x88\xca\x91\xc1\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x11\xff\x8a\xcd22SB\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x80]\x988ls\x7fA\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08O\xb6-\xa5.X\xf7\xc1\x97\x7f\xd1\x9c\x08\xe5\xff\xc1E\xd7s\t\x89\x17\xf2\xc1\xa4\x9c\x17A\xf6:\xdd\xc1\xf0\xa2\x8dt#\x0e\xb3Ar\x17\xbeE\x83%\xe1A=\xc1\xcd\xde\x9a\x98\xefA\x9b\xdc&\xb16n\xf7A\xe9U\x07\xde\xa9\xa4\xffA\xa6\x0f\xd6 \x122\x04B\xe4ns=\x92\xc5\x08B\xc5\xf9=\tz\xd6\xf6\xc1\xe6\x0cQs\xf4M\x00\xc2\xd8\xba\xf4=\xd4\x14\xf3\xc1B$\xd2\xe6\xcbI\xe0\xc1\xf4\xb0\x8a1D\x80\xa7Ay\x19\x1e7\xec\xff\xe0A*F\xf0=\xf3\xf7\xefAd\xbam\xdc\xd3\xba\xf7A\xb8O_\x11q\xf2\xffA\x94\xfe\xec=HR\x04Bo|\xa4\xbb\x07\xdd\x08B7\xe3\xe7p\xf9\x8e\xf6\xc1\x976\xc0\xdeF\x9e\x00\xc2|\xfb\x0c\'\x03\xf9\xf3\xc1\n\x96\xe1\xaa\xa6\xde\xe1\xc1\x7f\x94\xcd\x1d\x9d\x84\x90Al}\xce\xeb\x1b\xbc\xe0A\xb9)t\x8b~\x1b\xf0A2\xb1\x8b\x9c\x13\xfc\xf7A\xce;\xa8D\xa2\x1d\x00B\xb8c\xef\xb7\x9er\x04Br\x90\x86\x0c\x1d\xf6\x08Be\x9dT}\x16\x1e\x08\xc2w1J\xa1\xfa\xe4\x00\xc25\x9f\x81\x8aZ\xc6\xf4\xc1\xa6L\xc9\xda\x88Z\xe3\xc1D];("\xfa\x8c\xc1\x17\x02v\x19\xf2_\xe0A\xe4\x93\xdf\xdf1,\xf0A[0\x8f\x9b\x841\xf8A\x0cc\x12X\xd0>\x00B\x05\x03\xe9?9\x92\x04B7\xa4V\xe5%\x10\tB\x82\xcbO[]\xb9\x07\xc23P\x8b\x92Y#\x01\xc2\x0e7\xb1L\x0f\x7f\xf5\xc1\xba,Mg\xfe\xbc\xe4\xc1\x94\x0c;\x8fo\xb7\xa6\xc1\x98\xa6\xccv\xc3\xe1\xdfAi{Kx\xb5/\xf0A\xae\xd1\xd5v?[\xf8A!=\xfc\xb7<\\\x00B*\xf8e\xb7X\xb0\x04B\xa6=\xf6at*\tB\x0c\xdb\x8b\xb7\x07W\x07\xc2\x0e\xe5|^\x7fZ\x01\xc2\xf9\xfbK 7%\xf6\xc1\xbe\xc1\r\x01S\x06\xe6\xc1\xee\xbfW\x15\xb4\xf7\xb2\xc1\r\xf1J\x9f\x8b\xe7\xdeAha\\\x89\xcc\'\xf0AD"\xe9L\xc6y\xf8A\x7f\xe3`\xe2\x99u\x00B\xfa\xfb\xea\xf0b\xcc\x04B;\xfe\x93\x12eD\tB\xabq\xb7\x88\xcd\xf9\x06\xc2/\xda\xa1\xd2_\x8b\x01\xc2\xbeb\x82r\xc0\xba\xf6\xc1B\x04\x94xT7\xe7\xc1\x12\x04PRB^\xba\xc1u#\xa3\x9d\xa0\xd9\xddA\t\x89:\xdd;\x16\xf0A@\x03\xc7\xfb\xe4\x8d\xf8A\xbe\xc7X\xa8\xcb\x8a\x00BN)\x7fp\xe5\xe5\x04BF\xb4\xfb\xcbg]\tB:~\xcdr\x82\xa3\x06\xc2|r6R\xcc\xb6\x01\xc2\x1e3\xa9\x15oA\xf7\xc1\x02\xb8\xf2\x9e$Q\xe8\xc1\x90d\xb56O\xc0\xc0\xc1\x84\xf3\xa2\xd3\x13\xbf\xdcA\x9e\xd0\xf7\xddg\xf9\xefA^\xa81G\x95\x98\xf8A\x95yQ\x06\xde\x9b\x00Bz\x82}F\x94\xfc\x04B\xfd3\x14\xfb\x04u\tBCI\xbdAEU\x06\xc2\xbb\xda&\xd5x\xdd\x01\xc2\xb2\xf4*\xb6\xdb\xba\xf7\xc13o\xde\x19\x17U\xe9\xc1w\x8evb}*\xc4\xc1mi\xa6G\xb8\x9d\xdbANl\xb0\xcd\x86\xb9\xefA\xc9\x98\xc5\x02\xe8\x9a\xf8A\xae\xc0\x8c\x9e\xfb\xa8\x00B1P\xb6MF\x10\x05B\x13B\xd0\xdc\xdf\x8a\tB\x93\xe5f\x80\xaa\x0f\x06\xc2\xc3\x1d)\r\x00\x00\x02\xc2\xac\xb9\x91\x98u(\xf8\xc1\x071\xad\xa6\x98D\xea\xc1j\xe0P\xc5\xb7j\xc7\xc1E*Q\xdfAz\xdaA,\x02\xb4\xa2\x9fo\xefA:PL\xb5\xf3\x95\xf8A^\x02\x89\xd0d\xb2\x00B9R\xb2\xf3\xef \x05B\xb4\xc6u7\xb6\x9e\tBF\xe3f\xc2\xe0\xd2\x05\xc2\x8f\x19\x9a\x12\xe7\x1e\x02\xc2O\x13\x1e\xf1\x84\x8b\xf8\xc15(.\x95\x1d!\xeb\xc1gNSe\x8a\x7f\xca\xc1\xb2\xb7\xdfLiX\xd9Ap\xb8\xd3\xc5%\x1e\xefA\r\xa4K\x06\xc8\x8a\xf8A\xdf\x81\xe7\xfbg\xb8\x00B\x98\xc9\x1ds\x9d.\x05B\xfd\xae\xbf^^\xb0\tB\x9c\xd5\x18\x87\xcd\x9e\x05\xc2s\xb9\x85\x8c\xa0:\x02\xc2\x03\\\x82\x03.\xe5\xf8\xc1\x08\xeb\x08\xb5\x15\xec\xeb\xc1\xfa\xff\xa8\xc9\xa2h\xcd\xc1\x9b\xb0\xf1Z\x10;\xd8A\xf9s?\xad5\xc7\xeeA\xba|1Tez\xf8AU\xbb\x85&[\xbb\x00B6h\xef/m9\x05BU\x85\xd5%\xc4\xbf\tB \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_airbag_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_strike=66380.00,
            upper_strike=66380.00 * 1.05,
            lower_participation=1.0,
            upper_participation=1.0,
            knock_in_strike=66380.00,
            barrier_type='DOWN_IN',
            barrier_value=66380.00 * 0.8,
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_airbag_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_airbag_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_ping_pong_option_pricer(self):
        expected = b'\n\xf4\x06\t\xc5\x14(,\xfb\x1a\xb8@\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xc6\x81\xfd\x9cf:\xb0?\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x080\xf2\xe1xr\xc6\xa7\xc0\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xc0w\xef\xa2\x1d\nE@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00`\x9a{\x04z\xd3\xbf\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x084\x11\xcc\xa2\xeay\x0e\xbf\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xfe\x82\x18\xb0\x02\x9d9\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xc0\x87|\xae\xedp_\xc0\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08TSY!*\xc4\r@\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08?\xcc\xa9\xe0\xd4\xb28@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xe3\xc3\xbf\x99\xe5v\xe1@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08`k\x1e}\xbfZv@\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\xa5Q\xa1\xd5\xde=\xc1\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x80&\xcelIxh\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x88o\x85T`\xedh@\xfe\x03\xf7\x9a\xaaa\x83@\xfa7\x89\xd0&\x19\x98@\xb7\x85\x8ew\xbd\xde\xa8@\xca$?\'\xb5\x1d\xb1@\xd3\xb7\xeb\x05\xdb\x87\xaf@F\xe4\xaa\x19\x0e\xa7\xb2@q\xc0\xbc+97\xae@\xb7\xa7\xb9\xf6\x14Z\xa1@\x85\xa26\x906\xfb\x92@5u$J|\x18\x83@N\xa4k7\x95\tq@!KD4\x04\xba\x87@<4\x1f\x82i\xfc\x9b@P\xd0\xb7\x97\xb5m\xab@\x1b6\xc5\xe33\xb0\xb2@7z\xa7\xb0Zw\xb1@\xaeS\xa3!\x81O\xb4@\x0c\xda\x87?\x9e\xa8\xb0@s\xc8\x1e\x90\xd2V\xa4@\x1f*Q\xd3\'\xd7\x96@\xd7\xa33\x19\x1aq\x89@K\x7f\xf2\xf8\xbd=x@V\x11\xf7y.\x0b\x8d@\xe8\x99\x81\xf9\x15\x18\xa0@\xaf\xd3w\x8f\x16\xd8\xad@5PUb\x01\xff\xb3@\\js\xa8\xed3\xb3@\xb2{2\x01]\xd3\xb5@\x9aU\xd8\xc8\xdf]\xb2@\xa2\xcb\xde\x873\xc1\xa6@\'WG\x8a\xa4\xed\x9a@\x8e\x8e\'\x7f\xa0\xf5\x8f@\xb6\xffT.r\xac\x7f@\xcfIG\xab D\x91@\xb1\x99\xf31\xf71\xa2@\xb3\xc8:\x1f\x10#\xb0@H;,\xbb\xd4B\xb5@\xcd\x87Q\xe6:\xd3\xb4@N-\x9f\x92\xa7\'\xb7@t\x05\x9ez\xea\x89\xb3@=tz\xf3\x1ff\xa9@\x8cCp\x97\x89\x80\x9f@\x97^\x1b\'\x1f\'\x93@l\xe6Q\xf9M\xcf\x82@\x84(8\xbb\x06\x95\x94@\xdem\xfa\x03^\x91\xa4@\xb1&\xe6\x06\xf5E\xb1@\xe2\xec\x98L\x1f\x97\xb6@<\x8b\xd8A\xab0\xb6@\xe0\xde\x0b$\xf2{\xb8@\xa4\xdb\x12\x90\xa0\x04\xb5@\x90\x91\x91\xcd]\x12\xac@\xaf\x99\xf31\xf71\xa2@\x04\x10\xfa\xf0J\x95\x96@\xebS\x15Zbi\x86@\xa8\xac\x04?x \x98@\x87\x82\xce\x7f\\t\xa6@-\x1cn\x01Wn\xb2@\x8d\x90\x06}[\xb6\xb7@\xaa_\xa6w!\x83\xb7@\x02\xeb\x9cA\xb1\x95\xb9@\xcb\x82\xce\x7f\\t\xb6@A \xd6Vmu\xae@/\xf9~\x95\x0c\x8a\xa4@y\x1fK\x06k\x19\x9a@W\x8b\xf5N^q\x8b@\x8e\xfc"\xfe/(\x9b@\xdf\xdcdD\xb2S\xa8@\x1a\xed\xedwMp\xb3@44t\xad\x97\xd5\xb8@d\x90?\x19L\xc3\xb8@s\x82\xb2\xf0\x1e\xa8\xba@\x1f=E\xd3\xf5\x84\xb7@]6S\xabR\x96\xb0@\xf1\'\xaa\xf3\xe7\xae\xa6@I2\xea\xda\n\xee\x9d@\x1f&\xcb\xf5,\xee\x8e@FS\xdd;\xe7\xd0\x9e@)\xaa\xcf\x97\x19\x12\xaa@\xde\x0f\x88\xb8i{\xb4@na\xbf\x8fB\xc5\xb9@e\x1d\xa4&+\xf1\xb9@\xdd\xbbUT\x98\xa4\xbb@\xfa\xbdQ_\x06\xa6\xb8@o\x7f\x9cO\x1a\xf0\xb1@a\xcbP\xc0\x14\xdb\xa8@\xd9-I\xe9\x03\xda\xa0@ag\xecL}\xd6\x91@/\xe8\x86\xa0\xac\xdd\xa0@\x973\x1f\x82i\xfc\xab@\x82U\x83\x9d\xb1\x84\xb5@\x1f\xc8t9v\xa4\xba@vmP\xad\x01\xdf\xba@s\xd2\x97\x13\xe6\xa2\xbc@-(U\xc8\xb9\xd5\xb9@V!c\xa0\x16\xe7\xb2@,X\xeei\xe4\x15\xab@\x1b\x8f{,YZ\xa3@\x9fB\x8f\x9d\xe3\xd6\x94@|\x868\xce\x19\x91\xa2@\xf5s^d\xe2\x99\xad@\x18\x0eS\x11\x0bm\xb6@{t\xec+\x01\x80\xbb@\x84\x8eC\x0e\xde\xc1\xbb@\xd4\xad\xc8+k\x89\xbd@\x9a2?\x069\xc7\xba@.\xa2o,\'\x08\xb4@\xaf?\xb0\x9f(\x16\xad@f\x8b\x9f\xa5\xba#\xa5@L\x00\x8dL\xedD\x97@\xbc\xc6w\xb0\x92.\xa4@*\xd0\x9b\x08x\xa1\xaf@\xf1\xc4{\xa5$-\xb7@s\xd9\xfa\xf5\xf46\xbc@\x8b\xaf6o\xba\xa4\xbc@.Z@\x1e\xf6d\xbe@/\xeb\x0ez\x92\xaf\xbb@\x8c\xbaX\xcb\xb4.\xb5@\xf7\x86\x8b\x00\xa1T\xaf@[\xcd\x85gs\xe9\xa6@T\xa7\x81\xd8\x99\xc1\x99@\xfc5p\xb8\x05\xd7\xa5@\x9aB\xab,!\xa3\xb0@\x03\xe6n\x06\x01\x10\xb8@w\xcb41\xd7\x0e\xbd@\xac\xf1\xe3\x94\x82]\xbd@\x8a\x06\xb8\x10\x81@\xbf@/\x7f\xd6i\x80q\xbc@\xe8\xd2AjBU\xb6@)\xc5%\xfc\x80\xdf\xb0@\n\x11\x13\tl\xd7\xa8@ \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_ping_pong_option(
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=66380.00 * 0.95,
            upper_barrier_type='UP_IN',
            upper_barrier_value=66380.00 * 1.05,
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            payment_type='PAY_AT_MATURITY',
            cash=0.015,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_ping_pong_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_ping_pong_option_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_phoenix_auto_callable_note_pricer(self):
        expected = b'\n\xf4\x06\t\xc5\x97\xcc\xc2w\xde(A\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xe71#\xf0\x14\n\xf1?\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08`Vw\x80\xd9\x87\x13\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xffm"\xf2^\x17\x86@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x80\xb1\xec\xc0\xff?\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x9emd\xc2\xe5:\x8a?\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08~F\xdb\xd1p\x0b\xb6@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\xbb6\xb7pr\xb6\xc0\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08V/\xb9\xaf\x9f\xc3E@\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xff\x9d*\t\x0e\x0fr@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x18>\xab\xb0\x83\xd9CA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00;\xb2\xa4Vh\xd9@\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08>\xa68\xd0\x06F\xa3\xc1\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00\x9a\xb1\xc1\xe8\x93\xcf\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x084\xf6\xb9hg\xbe+A\x10\x04\x17\x86\xda\x9e,A7Q\xab\x9d\x84M-A\xfc\xa5`\xe2\x97\t*A2&A\xf9CU%A)r\x01\xc0\xf5s#A\x16\xf7\xf7\x97\xf8\xb7$A!\x84\x03$\x1c\x1d(AZ&X\xa1\x17\xdd+A5\x9f\xdb`\xcd\xf4-A\xce\x9fk{\xb8\x9e.A\xf4VZ\xde\xc8\xae+A\x9e\xfa\x0f\x10\x91\x8d,A\x89K\x10\x81\xd1=-A\x10\x92\xc0\xf3C\x82*A\x82\xc5\xe7~\x0cb&A\xc5m\xe1 <\xba$A\x12\x1932\xca\xab%A\x1c\xfa\xd5\x05\xdc\xa7(A@\x0b\xf6\x88\xc6\x01,A]\x90K\x9e\xdc\xe7-A\x9e\n~\xe6\xb0\x97.A\x16\xc5\x97\x882\x9f+AX\x93u\xd6Q|,A\xdb\x8c\x15P\xac--A\xb6\x9a\xb2\x86\x91\xc4*A.\xa4\xed\xe9\xe7C\'Aw]\r\xc8D\xd1%A\xf8&\xb54\xb3\xad&A\xef\x14mDjF)Al\n\x12T\x1f\x18,AR:\xf0\x15^\xdd-AHW\xb1\x8br\x87.AP\xaf\x86\xd3,\x8f+Af\xed\x15/Vk,At\x13\xcdP\x8b\x1d-A\xb0:\x19\xe9_\x14+A\xac\x87\xe5\x98\xcc\x13(A\xd8GX\x8d2\xdd&A|U\x16k\x1f\x92\'A\t\x9at\x86\xd3\xce)A]\xb9\x94\xc1\xe5;,A\xc6\xe4\xabmJ\xd3-A\xe4`\r\x87\xa2}.A\xdd\x07\x85K0\x7f+A\x84\x0b\x8b\x19\x0bZ,Ar%"\x14W\x0b-A\x8cH\x98G\xb9^+A6\xcb\xc0\x0b*\xcb(A,[\xbd\xda\xa5\xbc\'A\xfc\x1a_\x99\xf5i(Aj\xf6"5\x99R*Am\xa0\x98\x86\x97[,A\xaf:\x8aF\xe7\xd2-A\xcfeA\x0fws.A\xd4#\x15 7o+A6v\x8f\xb5\xb0H,A\xba\xdf]\xf5\x8d\xfa,A\xf5\xfb\x12Q\xe8\x9a+A\xf7m"aip)A\x8c\xc5g\xfc$~(A\x96N\xb8\x1c\x02()A\x91\xdd\x94\xbe\x9d\xb8*A\xa4\xa2Ep\x1e\x8a,A\xa1\xf2\x1f\x91Z\xd3-A\rZ\x15|\xb8q.A\xd4\xe1\xf1\xc4;_+A\x02\x89d\x15o7,AJc\xb8\xd3\xc4\xe9,A\xb0\xdf\xc5\xa2G\xcb+Aq\x03p-\xca\x05*A+W5\x7f45)Av18\xc9\xdf\xc9)A\xae\x1d\x97u8\x17+A=\x93\x1e\xfay\xbe,Af\x14\xcba\x81\xd0-A\xd5\t\xedq\x19i.A\x15\xd9\x9b\x06\tO+A\xd1d\x18\x92V&,Ai\x19\xc7D}\xd8,A\xee~\xc6\x1aD\xe9+A\xd216\xc3\xd2r*A\xb2\x85[tz\xe9)A\x9a\xa9\x04:>l*AV\xbe\x02\x89\'z+A?(\xa0\xf3\x90\xe1,Awu^\xbc\x7f\xd2-A\xb8~\xfa\x0b1h.A\x8e\xc2:A\xf6>+AA\xca\x87\xe0?\x15,A\x7fL\xbf2X\xc7,A\xfa\xc2"\xb5\x96\x0f,A\xd2\x10\x86\x112\xdf*Aa\r\xcb\xec\x84b*A1\xe1\xe2<+\xf0*A\xc6\xa5\x87M\x9b\xcb+A%\xb8\xf5k\t\x12-A\xc9q\x06\x1c\x8d\xe5-A\xdb\xe0\xc9\xc4\x90l.A%I\xb3\x97\xd8.+A\x0c\xb0\xd0\xe1\xd7\x03,A/\x93\xec\xdd\xec\xb5,A\xe7D\x84k\xd9.,A\'\x13\xe2\xa2%.+AV\x83\x8c~\x88\xd4*A\x91\xab\x8f\x0fcR+A\xc30\x971\xca\x1f,A\xe9\x7f\xaf\xad)0-A%\xb3\xfc\xb7\xcd\xf8-An\xbe\x16\xd0\xf3o.Ae\x98\x1fe\xc8\x1e+A\xc1\xf6)qj\xf2+Ad0\xc3Vg\xa4,Ad\np\xac(7,A\xd6\x04\xbfK\xd9\x89+A\x93\x99\xc4\x9a\x973+AL\xdf\xaa\xc0\x9b\xbb+A\'fg\xa1\x1ai,A\x84\x8a\x1b\xa9RK-A\x16?\xeb\xfa \x05.AD\x0cC;\xb1f.A\x83\xe9<)\x12\x0e+A\xea\x1a\xce*\xe9\xe0+A8\xdb11\x04\x93,A\x1a*\xeeO\xd3V,A\x94\xac\xe0\xf1\\\xc4+A\xe3\xdf\xd4\x03\x8e\x94+A\xbek\xd0\xf6\xd0\r,A1\x89\t\xd1\xf3\xa7,A\x07\xfbCg\xb8\\-A\x1c\x04\x9b\x80\x95\x0e.A\xdfx<\xe5\xbcl.A \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_phoenix_auto_callable_note(
            coupon_payoff_type='CALL',
            coupon_strike=66380.00,
            coupon_rate=0.12,
            start_date=datetime(2020, 2, 22),
            coupon_dates=[datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22),
                          datetime(2020, 7, 23), datetime(2020, 8, 22)],
            day_count='ACT_365_FIXED',
            knock_out_barrier_type='UP_OUT',
            knock_out_barrier_value=66380.00 * 1.1,
            knock_out_sched=[
                [datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22),
                 datetime(2020, 7, 23), datetime(2020, 8, 22)],
                [0.0] * 6,
                [1.0] * 6
            ],
            knock_in_barrier_type='DOWN_IN',
            knock_in_barrier_value=66380.00 * 0.9,
            knock_in_sched=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            long_short='SELL',
            knock_in_payoff_type='PUT',
            knock_in_payoff_strike=66380.00 * 0.85,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_phoenix_auto_callable_note_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_phoenix_auto_callable_note_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)

    def test_cm_snowball_auto_callable_note_pricer(self):
        expected = b'\n\xf4\x06\t\xed\x04G\x12\x86\xf9-A\x1a\xdf\x06\n\x94\x01\n\x05DELTA\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xc5\xe4x\xbd[\x1e @\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xa0\xe0uC\xd5\x8b\t\xc1\x12\x05TOTAL\x1a\x00\n\x9d\x01\n\x0eDELTA_EXPOSURE\x12R\n\tCOMMODITY\x12"\n\tDIVIDEND_\x12\x15\n\n\n\x08\x00\x00\x00\x00\x00\x00\x00\x00\x12\x05TOTAL\x1a\x00\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\xa2\xf2V\xc2\xe5\xb4@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00@\xf5en\xed4\xc0\x12\x05TOTAL\x1a\x00\n7\n\x05GAMMA\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\x8e:\x93\xfc\xc6\xb8U\xbf\x12\x00\x1a\x00\n@\n\x0eGAMMA_EXPOSURE\x12.\n\tCOMMODITY\x12!\n\rPRICE_CU.SHFE\x12\x10\n\n\n\x08\xff\x1f\x16\x08zA\x82\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\xc2\xb9\xdf\\w\x92@\x12\x00\x1a\x00\n;\n\x05VANNA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x9d\x80kf{\x00G@\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVANNA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xffS\x99\xcc\xf7\x15s@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x04VEGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\xfc\xa7)\xc4E\xa5\x14\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\rVEGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x80\xeb\xce\x19&m\xaa\xc0\x12\x05TOTAL\x1a\x05TOTAL\n;\n\x05VOLGA\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x8a\xfc\xf79\xaf\x1clA\x12\x05TOTAL\x1a\x05TOTAL\nD\n\x0eVOLGA_EXPOSURE\x122\n\tCOMMODITY\x12%\n\x07CU.SHFE\x12\x1a\n\n\n\x08\x00T\xec\x1f\x86\x07\x97@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08,\x9f\x0f\xc7`.(A{\xc6\x10=\xe1"*AQ\x15\x88\x9dP\xc7+A\xc5O\xed\x07\xb1\n-Ah\x1e\xcc\x1c\xde\xe8-A\x11z\xa0\x85\xc5r.A\n]\xd3.\x02\xba.A\x99$\xb5pa\xd4.A\x82\xfe`y\xf9\xd4.AD\x96M\xa1?\xd4.A\xa6\xc4\xe1\xed\xdb\xd3.A#~\xc9\xe10-(A\x82~\xad\xb7q\n*A\xa2Rsg4\xa7+A\xdf\x95~$\xee\xe5,A\xec\xaf\x82@\xec\xcc-Ap\xb42\xc4\xa5].A\xacX\x9d\xa1\xa2\xae.A\t*U\x82\x8c\xd0.A\x88\x04\xf7Ho\xd3.A\xf8\x87\xcc\xe1-\xd4.A\xfeb\xda\xb7\xe5\xd3.AQ\r\x1dH\xce)(A\xa6\x92\xdaI\xbb\xf1)At\xd6\x11\xa0\xdf\x85+A\xa3W\xa9\xf3\x1b\xc4,Af\xb4\x1dTa\xae-A\xbd3\x05\xdd\x04J.A\xe38y9\x13\xa3.A\xda\xce\xd1\x14\xbc\xc9.A\xe4"\xd8\xfe\x94\xd2.A5\x7f\xebh\x17\xd4.AjM\xa1\xd3\xe4\xd3.A\x9a\xed\x8dpm\'(A\xd4P\xe1\x91\x08\xdc)A\xb7\xd9\xf9iVg+A\xe1\xdfB@\xfa\xa5,A\xc0ftzv\x91-Ab\xa5\x90+\xaf4.A/0\x85\xf8\xed\x93.A8M\xc3x\xf5\xc2.A\xe2\x07)W\xa0\xd1.A3\x94\xb6\xdd\xcc\xd3.A\xcbT6\x9c\xee\xd3.A\xdbo\x92\x8dO$(A\x9d\xc6"\x9b}\xc8)A\xbe=\xdc\xf4dL+AQ\x10\x12\xb2\xfa\x82,A\xf6\x91\xf3z9r-A\xbc,\xdc\xcd\xf2\x1e.A\x1a\x97\xa5\xf2\xf0\x82.Ar-7j\xbf\xbb.A\xf9\xbdz\xe9\x1d\xcf.A\x10\x17\xa4]\x11\xd3.Aw\r\xb9\xa7\x1b\xd4.A\xad\x9e\x87X6!(A\xa3\x9a\x8dg0\xb6)A\x81\x00o~\xa2.+A\xd6\xc92\xa9Vg,A\xa2\xe1\x03\xdc)W-A\xd57\xb64\xf5\x05.A/l\x17;\xa4s.A+\xb6\x7fE\xfe\xb1.AP-.\xaf\xcc\xcc.AUq\xbf\xfeW\xd2.A\x04T.\x14\xf3\xd3.A@\xde\xb5\xe1p\x1e(A\r\xb7\x9d\x9a!\xa6)A\xad\xf4.8$\x17+A\xee\x89\xb3p\x10D,AX{\x18\xc1\xba<-A;\x12\xfa9\xf8\xed-A\x03\xdc=Z\xefc.A\xaf\x9b\xb5\xa7\xab\xa7.A\x9c\xc9Rl>\xc8.A\xa8\'@\x02\x84\xd0.Afm\xa7\xab\x9c\xd3.A\x8c]"a\x1d\x1a(AQ?M^\x11\x96)A]\xdc\xc9\xab\xd2\xfe*A\x0bX\xfe\xba\xe4(,A-\xa7\xdb\x8f\xd7\x1f-ArCI\x82\x82\xd3-A76\x1f"\x8eS.A\xbf\x8a\xa3r\xbb\x9b.A\x0c\xe8\xb1p\xe1\xc3.Ab\xf4\x85;\xd7\xce.A?P\xcc!\x01\xd3.A\x95\xb7\xeb}\\\x15(A\x8a!\xe1\x8a\xe6\x88)A\x1bq\x98\xf3\x01\xe9*A\x16\x03\xdf%\x90\x0f,A\x9c\x05\xcf\xd8&\x04-A:\x1e\xfa!C\xbc-A\xb1\x151\x1f<D.A\xd3(\xe9e\xed\x91.A\x19\xd4e\x86\xf8\xbc.A`\xaaL\x1d\xb4\xcc.AM\xc1\xe3\xafY\xd2.AA\xcb\xb5\x10i\x12(A\xc5wB\x87\x8f{)A_{\x96\xb4M\xd4*AP\x96j\x8a\x0e\xf4+A\xc1L\x14$\xb9\xe8,A\xdc\xa3\x1brT\xa5-Ay\xeei\xd7\xed1.A\xad\xc6\xec\xaa\xbf\x86.A\xac\x93\xe0.\xf2\xb6.A\x96\xfd\x17"\xb5\xc9.A\x1e\x8f-^\x93\xd1.A\xceHs2\x83\r(A\x1d\x0f\x05J<p)AAO\xb7h5\xbe*A\xf8\x16\xb4"G\xdb+A\xf5\xda\xd4^\xb3\xce,AB\x0b[J\xf4\x8e-A\xadq\x05(a\x1f.A\t\x97\xfc\xa8\x85y.A\xa0\x8eq\xc9\xab\xae.A\x1d\'\xcc\xf7\xf2\xc6.A\xf2i\n\x0c\x1c\xd0.A`\x9d\x1c\xb6\x8a\x07(A\xe5sA%\xb9`)A\x80\xeb\xc0Ql\xa9*A\x8c\x90_\xbd\xda\xc5+A\x87{H^\xe7\xb6,A\x95B\x8e\x7f5x-A\xc5\x1e,\x86\n\n.A\x84\xad\xc2oOn.A\x07{\xc2\x8a\x94\xa6.An\xdci\x81.\xc2.AQ\x98\x17\x10\xda\xce.A \x01*Z\nX\x00\x00\x00\x00\x00\xee\xe9@ffff\xe69\xeb@\xcc\xcc\xcc\xcc\xcc\x85\xec@2333\xb3\xd1\xed@\x98\x99\x99\x99\x99\x1d\xef@\xff\xff\xff\xff\xbf4\xf0@2333\xb3\xda\xf0@efff\xa6\x80\xf1@\x98\x99\x99\x99\x99&\xf2@\xcb\xcc\xcc\xcc\x8c\xcc\xf2@\xfe\xff\xff\xff\x7fr\xf3@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_snowball_auto_callable_note(
            coupon_rate=0.12,
            start_date=datetime(2020, 2, 21),
            coupon_dates=[datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22),
                          datetime(2020, 7, 23), datetime(2020, 8, 22)],
            day_count='ACT_365_FIXED',
            knock_out_barrier_type='UP_OUT',
            knock_out_barrier_value=66380.00 * 1.05,
            knock_out_sched=[
                [datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22),
                 datetime(2020, 7, 23), datetime(2020, 8, 22)],
                [0.0] * 6,
                [1.0] * 6
            ],
            knock_in_barrier_type='DOWN_IN',
            knock_in_barrier_value=66380.00 * 0.80,
            knock_in_sched=[
                [
                    datetime(2020, 2, 22), datetime(2020, 2, 23), datetime(2020, 2, 24),
                    datetime(2020, 2, 25), datetime(2020, 2, 26), datetime(2020, 2, 27),
                    datetime(2020, 2, 28), datetime(2020, 2, 29), datetime(2020, 3, 1),
                    datetime(2020, 3, 2), datetime(2020, 3, 3), datetime(2020, 3, 4),
                    datetime(2020, 3, 5), datetime(2020, 3, 6), datetime(2020, 3, 7),
                    datetime(2020, 3, 8), datetime(2020, 3, 9), datetime(2020, 3, 10),
                    datetime(2020, 3, 11), datetime(2020, 3, 12), datetime(2020, 3, 13),
                    datetime(2020, 3, 14), datetime(2020, 3, 15), datetime(2020, 3, 16),
                    datetime(2020, 3, 17), datetime(2020, 3, 18), datetime(2020, 3, 19),
                    datetime(2020, 3, 20), datetime(2020, 3, 21), datetime(2020, 3, 22),
                    datetime(2020, 3, 23), datetime(2020, 3, 24), datetime(2020, 3, 25),
                    datetime(2020, 3, 26), datetime(2020, 3, 27), datetime(2020, 3, 28),
                    datetime(2020, 3, 29), datetime(2020, 3, 30), datetime(2020, 3, 31),
                    datetime(2020, 4, 1), datetime(2020, 4, 2), datetime(2020, 4, 3),
                    datetime(2020, 4, 4), datetime(2020, 4, 5), datetime(2020, 4, 6),
                    datetime(2020, 4, 7), datetime(2020, 4, 8), datetime(2020, 4, 9),
                    datetime(2020, 4, 10), datetime(2020, 4, 11), datetime(2020, 4, 12),
                    datetime(2020, 4, 13), datetime(2020, 4, 14), datetime(2020, 4, 15),
                    datetime(2020, 4, 16), datetime(2020, 4, 17), datetime(2020, 4, 18),
                    datetime(2020, 4, 19), datetime(2020, 4, 20), datetime(2020, 4, 21),
                    datetime(2020, 4, 22), datetime(2020, 4, 23), datetime(2020, 4, 24),
                    datetime(2020, 4, 25), datetime(2020, 4, 26), datetime(2020, 4, 27),
                    datetime(2020, 4, 28), datetime(2020, 4, 29), datetime(2020, 4, 30),
                    datetime(2020, 5, 1), datetime(2020, 5, 2), datetime(2020, 5, 3),
                    datetime(2020, 5, 4), datetime(2020, 5, 5), datetime(2020, 5, 6),
                    datetime(2020, 5, 7), datetime(2020, 5, 8), datetime(2020, 5, 9),
                    datetime(2020, 5, 10), datetime(2020, 5, 11), datetime(2020, 5, 12),
                    datetime(2020, 5, 13), datetime(2020, 5, 14), datetime(2020, 5, 15),
                    datetime(2020, 5, 16), datetime(2020, 5, 17), datetime(2020, 5, 18),
                    datetime(2020, 5, 19), datetime(2020, 5, 20), datetime(2020, 5, 21),
                    datetime(2020, 5, 22), datetime(2020, 5, 23), datetime(2020, 5, 24),
                    datetime(2020, 5, 25), datetime(2020, 5, 26), datetime(2020, 5, 27),
                    datetime(2020, 5, 28), datetime(2020, 5, 29), datetime(2020, 5, 30),
                    datetime(2020, 5, 31), datetime(2020, 6, 1), datetime(2020, 6, 2),
                    datetime(2020, 6, 3), datetime(2020, 6, 4), datetime(2020, 6, 5),
                    datetime(2020, 6, 6), datetime(2020, 6, 7), datetime(2020, 6, 8),
                    datetime(2020, 6, 9), datetime(2020, 6, 10), datetime(2020, 6, 11),
                    datetime(2020, 6, 12), datetime(2020, 6, 13), datetime(2020, 6, 14),
                    datetime(2020, 6, 15), datetime(2020, 6, 16), datetime(2020, 6, 17),
                    datetime(2020, 6, 18), datetime(2020, 6, 19), datetime(2020, 6, 20),
                    datetime(2020, 6, 21), datetime(2020, 6, 22), datetime(2020, 6, 23),
                    datetime(2020, 6, 24), datetime(2020, 6, 25), datetime(2020, 6, 26),
                    datetime(2020, 6, 27), datetime(2020, 6, 28), datetime(2020, 6, 29),
                    datetime(2020, 6, 30), datetime(2020, 7, 1), datetime(2020, 7, 2),
                    datetime(2020, 7, 3), datetime(2020, 7, 4), datetime(2020, 7, 5),
                    datetime(2020, 7, 6), datetime(2020, 7, 7), datetime(2020, 7, 8),
                    datetime(2020, 7, 9), datetime(2020, 7, 10), datetime(2020, 7, 11),
                    datetime(2020, 7, 12), datetime(2020, 7, 13), datetime(2020, 7, 14),
                    datetime(2020, 7, 15), datetime(2020, 7, 16), datetime(2020, 7, 17),
                    datetime(2020, 7, 18), datetime(2020, 7, 19), datetime(2020, 7, 20),
                    datetime(2020, 7, 21), datetime(2020, 7, 22), datetime(2020, 7, 23),
                    datetime(2020, 7, 24), datetime(2020, 7, 25), datetime(2020, 7, 26),
                    datetime(2020, 7, 27), datetime(2020, 7, 28), datetime(2020, 7, 29),
                    datetime(2020, 7, 30), datetime(2020, 7, 31), datetime(2020, 8, 1),
                    datetime(2020, 8, 2), datetime(2020, 8, 3), datetime(2020, 8, 4),
                    datetime(2020, 8, 5), datetime(2020, 8, 6), datetime(2020, 8, 7),
                    datetime(2020, 8, 8), datetime(2020, 8, 9), datetime(2020, 8, 10),
                    datetime(2020, 8, 11), datetime(2020, 8, 12), datetime(2020, 8, 13),
                    datetime(2020, 8, 14), datetime(2020, 8, 15), datetime(2020, 8, 16),
                    datetime(2020, 8, 17), datetime(2020, 8, 18), datetime(2020, 8, 19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            long_short='SELL',
            knock_in_payoff_type='PUT',
            knock_in_payoff_strike=66380.00 * 1.0,
            knock_in_payoff_gearing=1.0,
            reference_price=66380.00,
            expiry=datetime(2020, 8, 22),
            delivery=datetime(2020, 8, 22),
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_snowball_auto_callable_note_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        # print('test_cm_snowball_auto_callable_note_pricer:', result.SerializeToString())

        self.assertEqual(result.SerializeToString(), expected)


'''
        
    
    def test_build_cm_yield_curve(self):
        expected = b'\n|\nx\n\x07\x08\xe4\x0f\x10\x02\x18\x15\x10\x02\x1a\x07\x08\xe4\x0f\x10\x02\x18\x1a\x1a\x07\x08\xe4\x0f\x10\x03\x18\x19\x1a\x07\x08\xe4\x0f\x10\x06\x18\x18\x1a\x07\x08\xe4\x0f\x10\t\x18\x17"\x055DAYS"\x0633DAYS"\x07124DAYS"\x07215DAYS*"\n d\x14\x7f\xb1\xdd\xdcK\xc0\x81\xff\xbd\x8e\xdf7"\xc0h\xa0\xac\\[x\n\xc0\'\xe4\n\xa5\xe8\xdb\x05\xc00\x018\x01\x10\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15'
        
        result = cm_yield_curve_builder(
            term_dates=[
                datetime(2020, 2, 26),
                datetime(2020, 3, 25),
                datetime(2020, 6, 24),
                datetime(2020, 9, 23)
            ],
            future_prices=[],
            call_price_matrix=[
                [45.800, 40.800, 35.800, 30.800, 25.800, 20.800, 15.860, 10.810, 6.090, 2.320, 0.530, 0.120, 0.050, 0.010, 0.010, 0.010],
                [45.990, 41.320, 36.410, 31.650, 26.230, 22.310, 17.660, 13.650, 10.190, 7.190, 4.980, 2.160, 1.100, 0.660, 0.460, 0.300],
                [49.900, 45.500, 40.930, 36.660, 32.610, 28.660, 25.260, 21.460, 18.540, 15.780, 13.250, 9.350, 6.590, 4.700, 3.300, 2.420],
                [53.280, 49.320, 45.240, 41.260, 37.360, 33.900, 30.450, 27.420, 24.330, 21.570, 19.300, 14.830, 12.980, 8.970, 7.020]
            ],
            put_price_matrix=[
                [0.020, 0.020, 0.020, 0.020, 0.040, 0.080, 0.150, 0.230, 0.650, 1.840, 5.000, 14.720, 24.510, 34.300, 44.370, 54.370],
                [0.270, 0.330, 0.400, 0.540, 0.810, 1.130, 1.720, 2.700, 4.000, 6.080, 8.750, 16.000, 24.960, 34.310, 44.370, 54.370],
                [2.010, 2.300, 2.810, 3.450, 4.330, 5.410, 6.680, 8.180, 10.040, 12.290, 14.710, 20.760, 27.730, 35.690, 44.570, 54.370],
                [3.650, 4.340, 5.160, 6.120, 7.240, 8.620, 10.150, 11.840, 13.810, 15.950, 18.410, 23.940, 30.380, 37.590, 45.670]
            ],
            strike_matrix=[
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40]
            ],
            spot=self.underlying_price,
            discount_curve=self.ir_curve
        )
        
        self.assertEqual(result.SerializeToString(), expected)
     
    def test_pm_vol_surface_builder(self):
        expected = b'\n \x08\x01\x10\x03\x18\x01 \x01(\x010\x028\x01I\xf1h\xe3\x88\xb5\xf8\xe4\xbeQ\xf1h\xe3\x88\xb5\xf8\xe4>\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xc3\xecB\xd9k!\x02@!\xc9"\xc8\xa43y\x0e@*\x8b\x01\n\x88\x01\xc3\xecB\xd9k!\x02@\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xc9"\xc8\xa43y\x0e@0\x039\xe0\xc0\x81\x03\x07\x0e\x8c?B\x8b\x01\n\x88\x01\x99(c`\xcb*\xe1?\x11Ii\xe7\xb9,\xe1?6\x85\x19\xea?\xa9\xde?Q\x06C\xd6a\x02\xdb?\x03\xac\x0f9\xcfb\xd7?)\xcb\xe6%1\x83\xd5?=\xe5\xde}\x1a\x82\xd3?Q\xfb\x1c4\x03\x1a\xd1?\x9c\xad\x972\x0fY\xcb?\xed\xd8\x96\xc4\xaa\xb0\xc7?\x82+\xc8\x80@\xe7\xc3?\x1a\xaa\xbbB\xabc\xc2?\x9e\xa7)0&\x19\xcd?&\xd7\xfd\xb1\xc1x\xd3?\xdd\x0b;\x97%\x9e\xd5?\x9e\x95Z\t\xa2\xd3\xda?m\x7f\x818\x04#\xe0?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00i\x0f/\x7f\xd80\x8d?\x8e\x03\xfd\x9e\xcf\x86\x7f\xbflo\xec\xf1`Q\x80?\x02\\x\xfe\x18]\x94?R"\n \xe0\xc0\x81\x03\x07\x0e\x8c?qI[\x0b\xa9\xa6\x07@\xc3\xecB\xd9k!\x02@\xc9"\xc8\xa43y\x0e@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x9f\x8b\xa1\xa6\xcb\x13\xf9?!\xc2\x16d{\xfb\xae\x17@*\x8b\x01\n\x88\x01\x9f\x8b\xa1\xa6\xcb\x13\xf9?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xc2\x16d{\xfb\xae\x17@0\x039Sr\xb1/,%\xb7?B\x8b\x01\n\x88\x01\x06\xec#\xfa{(\xe0?\xa2W$\xc6\xb7\xf9\xd2?\x02\xdb\x8c\xbd\x14\xb4\xd1?rbr\xf1N]\xd0?mW\x86b\xc3\xa7\xce?\xb8k\x1c\xd3.A\xcd?\x9f|S:\x1cE\xcb?\xf9\xce\x82\xba/\xfc\xc9?\xdc\xc3\xd6\xbd,3\xc9?\x19]h?i\x04\xc8?\xc2(o\x86\xbd\xd6\xc7?&\x05\x08qtL\xc8?=\xc8\xed\xf41\xd7\xc8?%\xd0o\xb5T{\xcb?ign\x9b\xfe\xcc\xce?\xfa\tM\xb9\x8c@\xd1?\xcc&\xd4\n!\xe9\xe1?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00\x17\xf3ak\xde\x99\xa3?\xb5*@\xc1\r:\xb3?\xd1@\xfb\xcc\xafz\x83?\xea<`\xd0\x0b\xe5\xb4?R"\n Sr\xb1/,%\xb7?\xc2\x16d{\xfb\xae\x07@\x9f\x8b\xa1\xa6\xcb\x13\xf9?\xc2\x16d{\xfb\xae\x17@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xa8\xa1\xf8\xdc\xa8\xe0\xf7?!\xa8\xa1\xf8\xdc\xa8\xe0\x17@*\x8b\x01\n\x88\x01\xa8\xa1\xf8\xdc\xa8\xe0\xf7?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xa8\xa1\xf8\xdc\xa8\xe0\x17@0\x039\xe1[Q<\x12\xbe\xd5?B\x8b\x01\n\x88\x01\xc6\xcd;\x86(P\xda?\x1a\x04\xebo\xe9q\xcf?\x16E\xb3\x0b\x07\xf9\xcd?\x1d\xf4\xb3I\xc6\x16\xcd?\xe6\xe5\x02\x87lH\xcc?\xff"\xd8\xb1\xde\xc5\xcb?alBR\xcaQ\xcb?\xec\x88\xcd\x16\xf8\xd5\xca?\xf5\xb3\x8d\xc7\xa1]\xca?3A\xaa\xca\xaf\x1e\xca?k\xb6\x1f\xb8K\x19\xca?6\x12\x87Iy\xfa\xc9?n\xb5\xd4\xe1\xad2\xca?\xaa\x8c\xcf\x17x\xc3\xca?3\xf6 \xe7\x04\x97\xcb?\xfd\xd6\xa2;\xfb?\xcc?\xbb\xa8\x0fV\x95\x98\xd8?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00\xf0qS\x89\xfe6\xb3?\xa3\x93\x8c\xfa{\x9d\xb6\xbf\xcfTb\x87\x85p\x8b\xbf\xd2<\x03Gf\xf5\xc7?R"\n \xe1[Q<\x12\xbe\xd5?\xa8\xa1\xf8\xdc\xa8\xe0\x07@\xa8\xa1\xf8\xdc\xa8\xe0\xf7?\xa8\xa1\xf8\xdc\xa8\xe0\x17@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x15\xca\x94$\xb6\x10\xf8?!\x15\xca\x94$\xb6\x10\x18@*\x8b\x01\n\x88\x01\x15\xca\x94$\xb6\x10\xf8?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\x15\xca\x94$\xb6\x10\x18@0\x039\x97-[\xb6l\xd9\xe2?B\x8b\x01\n\x88\x01\xb5\xfe\x91\xf8\x15\xa5\xd2?\x92\x9ao\xc1I\x8b\xcd?\x84\xa6R\xb6%\x06\xcd?\xfa\xa9qx\x95\x8e\xcc?6\x0eO\xe2) \xcc?\x90\x0e\xe7`h\xbc\xcb?#{\xc2\xf4\xb3\x87\xcb?g\xbe\xb7jZG\xcb?\xa7F@\xb7\xa9\xfd\xca?@P\xbf\xf7@\xd6\xca?\x01\xaa\xff\xf4V\xa3\xca?\xd4\xc2F\xc4\x89\xba\xca?\xd8z3\xd4\xffs\xca?\xfe\x01\xf7\'L\xe4\xcc?\xe9a\xe1p\x8a0\xcb?\xc2/\xf6]\n\xbc\xcb?o\xea\x84_1\xa7\xd0?J*\n(H{\xbd\x8eJ\xac\x99?\xc6\x058\x97X\xc0\x9d?\xe6\x17O\x83\x16\x0f\xd3\xbfo\xa8\x8e\xe1\x1f\x9b\xa0\xbf~(\x99\xc4\x16\xa8\x93?R"\n \x97-[\xb6l\xd9\xe2?\x15\xca\x94$\xb6\x10\x08@\x15\xca\x94$\xb6\x10\xf8?\x15\xca\x94$\xb6\x10\x18@X\x01"\x07\x08\xe4\x0f\x10\x02\x18\x1a"\x07\x08\xe4\x0f\x10\x03\x18\x19"\x07\x08\xe4\x0f\x10\x06\x18\x18"\x07\x08\xe4\x0f\x10\t\x18\x17*\t50ETF.SSE'
        result = self.vol_surf
        self.assertEqual(result.SerializeToString(), expected)

    def test_pm_european_option_pricer(self):
        expected = b'\n\xe4\x06\t\xcd.\x8b\xdf\\g\x07A\x1a\xcf\x06\n\x96\x01\n\x05DELTA\x12T\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\x88\x1d6\x82\x07\x13!A\x12\x00\x1a\x00\x12+\n\x12DIVIDEND_50ETF.SSE\x12\x15\n\n\n\x08\x9dD\xb0X\x07A)\xc1\x12\x05TOTAL\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x02\x93\x8d\x16^^&A\x12\x05TOTAL\x1a\x00\n\x9f\x01\n\x0eDELTA_EXPOSURE\x12T\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\xef\xec\xa9\x07q)\xd0@\x12\x00\x1a\x00\x12+\n\x12DIVIDEND_50ETF.SSE\x12\x15\n\n\n\x08n\x9fD\xc9&\xb0T\xc0\x12\x05TOTAL\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x1fZ\xeaO\nSR@\x12\x05TOTAL\x1a\x00\n0\n\x05GAMMA\x12\'\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\xef|\xac6\x88\'+A\x12\x00\x1a\x00\n9\n\x0eGAMMA_EXPOSURE\x12\'\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\x03\x16\xf5\x9dgT\x88@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xea\xd9\x05\xb5\xea\x95\x05A\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xa5EjZj\xb0\xe4\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08#5e\xb8\x1e\x11)\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\t\x9e\xd6\xff\xea\xf6(A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xba\x07\xcb\xff_\xf4\xbf@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08I\xcb\xf7\xc2\'\xda\xcc@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xbc\xb76\n\xbd\xa2\xf7?\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08F\x14Q\x97\xa0Z\xa9@\xd6\xe66~\xa4\xe6\xc2@\x15V\x98EtF\xd7@]e\xbb\xca\x00j\xe8@d#\x1e\x9a\xd8g\xf6@(\xd8\xa0\xa7Cj\x02A\xd0\xa8\x07\xbd\xda\xa9\x0bA\x93T\xb6\xaf^R\x13A>.\xad*sx\x19Aee\xe2XX\t Al\x8c\xdb\xca\x10{#AX1\xed\x8dJw\xb1@3\xc0,p\x8f\xd5\xc7@\xa2B\xd4\xf0Wj\xdb@\xbc\xa1\x15\xa3UQ\xeb@\x0b\xd1<\xfc\t%\xf8@G\xaaRB:R\x03Ae\x8b\xd6\xd4\x89\x7f\x0cA\x99Pe\xba\x96\xaa\x13A\xf5\xa4\xbe\x1f\x88\xba\x19AN\xa68&\t  A\x8e\xa7\xcf\xfa{\x89#A\xc5M]lN,\xb7@\xfeF<\xdf\xbeM\xcd@K\nz\xbf\xc9\xc6\xdf@a5\xc3\x00\x1bG\xee@\xe9\xa3\'\xf9s\xe3\xf9@6\xfd\x01yS:\x04A2N\xf2\xac\x9cW\rAy\xdf\x8c\x95\xc2\x05\x14A\xe0\xabi.\x08\x01\x1aA\xb2\x04V\xcf?9 Ag\xd5\xd7\x03N\x9a#A\'\x03\xca\xa7\xea\xcc\xbd@\xd2\x96H M\xa3\xd1@\x18\xd8\x13:\xe2*\xe2@\xd8\x04\x9e\xef\xaf\xa4\xf0@$r\x95\xb7\xe4\xa2\xfb@8B\xd8\xf3\x87"\x05A\x1c"\x15(\xbc1\x0eA\xa0\xd7C\xa0\x85c\x14A\xf35\xc7\x1c\x87K\x1aAY\xa0\x10\'\xd5T Ac\xb6JK|\xad#A\xd9\xa7;%\t\xab\xc2@\xa2\xc7\xfa\xca\xaf\xdb\xd4@\xf5\xf0\xba\xf2\xf8\x88\xe4@\xa5\xefR\x19B+\xf2@\x1e\x85\x06W3c\xfd@\xfcMg\x9d\xd1\n\x06A<\xf7\xeb8\xa0\r\x0fAl\x86\xab\x92\x90\xc3\x14A\x16\xab5`\xa2\x99\x1aA\xaa\xd4_\xe6\xa0r A%?m\xf9\xf4\xc2#A\xc6s\x81\xbf\x05\xe1\xc6@\xbc\xc9\xad\x9d\xb8K\xd8@}\x93%\x8dM\xfb\xe6@\t\x914\xc7\x94\xb6\xf3@\x05\xb3w\xf7=$\xff@u<\xc7Z+\xf3\x06A\x94\xa4\xb9\xca\x0c\xeb\x0fA\xde\xdf\xcdc\x9f%\x15Ab\xcf^\xfd\x00\xeb\x1aA@\x9b\xe5+{\x92 A\xac\x07^O\xa1\xda#A\x89\xfb\xa5V\x9c\x84\xcb@\x17zFpC\xef\xdb@D:\xfb\xa5\xcb\x7f\xe9@%\xed\xe3\x81\x13F\xf5@\xf3\xaa\xfe\x1f\xf4r\x00A\x1a\x03\x1c\xd6\x90\xdb\x07At\x1d\x00\xb1\xe7d\x10A\xbb\x1b\xe6\x81w\x89\x15A\xd6\x89\xa4\x05S?\x1bA\x82\x87|q=\xb4 A\xe2\x04?}g\xf4#A^nB\xb9\xaeH\xd0@{z\x9c\x18e\xc2\xdf@\xcb<x\xaa\x9d\x14\xec@\xec\xab]\xc0?\xd9\xf6@\x92\x93O \rT\x01A^\xde=V\xfe\xc3\x08A\xe6\x13\x03\xa3\xde\xd4\x10A[\xc4GT\xe6\xee\x15A\xd9\xf3)\xe4P\x96\x1bA\x86\x8e\x93\x1f\xc3\xd7 A\x92\xfb\xc3\x07,\x10$A\xf6E5<<\x01\xd3@\xf3\xa8\xa0a\xb9\xe0\xe1@\xf7\xa2e\'%\xb8\xee@}"~\x10\xado\xf8@\xa0\xabc\xcb_5\x02A\xect\xe7\x9fp\xac\tA\x88X\x8c\x08YE\x11A\x0e\xe0\xa6\xfc\xbfU\x16A\xf3.\xc5\x98\xba\xef\x1bA\xe2\x12h\xdc\xe9\xfc A\xa0\x8af\xd0\xd2-$A\xbe\xe9\x95\nv\xe9\xd5@\x154^O\x82\xf4\xe3@j\xb49\xecy\xb4\xf0@\xfa\xf3\xed\x01\xfe\x08\xfa@\xcc,ua\xe3\x16\x03A,C\xe4\xdd\xe4\x94\nA\xe8\xb9RGG\xb6\x11A\x1c\xbd\xd0M\xde\xbd\x16Az\xaa\x82\xf2VK\x1cA=\xb7\x1d\xad\x91#!AeA7\xd3?M$A\xdd\xd4g\x1e\xd5\xfe\xd8@\x17 \xc9\xbd\xfa\x1a\xe6@X\x94\xa1\xc0\xe2\x12\xf2@\x80;p\xa9\xe1\xa4\xfb@\x10\xe4\x82Q\x90\xf8\x03A\x7f\xf2u\x8eX}\x0bA\x19lj\xea\x9b\'\x12AL\x1a\xeb\xee\x1f\'\x17A!v|\xd3\xf2\xa8\x1cAejp\xf8\x9cK!A\xd1D\x0b\xabWn$AY\x96\x85!\xd9>\xdc@\x93\x8biB\xb0R\xe8@\xfbk]D\xbdv\xf3@\xc8$7\x9c\x11C\xfd@\x8e7\x81\x04`\xda\x04A\x98\xe2\xa7t\xc9e\x0cA\xac\xcd\x02FK\x99\x12A\x0b{\x8d\xa3g\x91\x17AL\x86\xed\x82`\x08\x1dAdU\x10v\xf0t!Ah\x08\xf3\xe9\xff\x90$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_european_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike=66380.00,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='FUTURE',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = cm_european_option_pricer(
            instrument=inst,
            pricing_date=self.as_of_date,
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        self.assertEqual(result.SerializeToString(), expected)
    
    
'''
if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestCmAnalytics)
    unittest.TextTestRunner(verbosity=2).run(suite)
