import unittest
from datetime import datetime

from caplib.market import *
from caplib.analytics import *
from caplib.eqanalytics import *

class TestEqAnalytics(unittest.TestCase):

    def setUp(self):

        # Create calendar
        cal_cnbe = 'CAL_CNBE'      
        hol_serial_numbers =[44654, 44954]
        sbd_serial_numbers = [44655]
        # Convert list of serial numbers to datetime objects
        holidays = [datetime.fromordinal(sn) for sn in hol_serial_numbers]
        specials = [datetime.fromordinal(sn) for sn in sbd_serial_numbers]
        create_calendar(cal_cnbe, holidays, specials)
        
        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and ANALYTICAL method 
        self.bsm_analytical_pricing_settings = create_pricing_settings(
            'CNY', False, 
            create_model_settings('BLACK_SCHOLES_MERTON'), 
            'ANALYTICAL', 
            create_pde_settings(), 
            create_monte_carlo_settings()
            )

        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and PDE method
        self.bsm_pde_pricing_settings = create_pricing_settings(
            'CNY', False, 
            create_model_settings('BLACK_SCHOLES_MERTON'), 
            'PDE', 
            create_pde_settings(201, 401, -5, 5, 'MMT_NUM_STDEVS', 0.001, 'ADAPTIVE_GRID', 'CUBIC_SPLINE_INTERP'), 
            create_monte_carlo_settings()
            )

        # Create Pricing Settings with BLACK_SCHOLES_MERTON model and MONTE_CARLO method
        self.bsm_mc_pricing_settings = create_pricing_settings(
            'CNY', False, 
            create_model_settings('BLACK_SCHOLES_MERTON'), 
            'MONTE_CARLO', 
            create_pde_settings(), 
            create_monte_carlo_settings(8096, 'SOBOL_NUMBER', 1023, 'BROWNIAN_BRIDGE_METHOD', 'INVERSE_CUMULATIVE_METHOD',False, 1)
            )

        # Create Pricing Settings with Duprie Local Vol model and PDE method 
        self.duprie_pde_pricing_settings = create_pricing_settings(
            'CNY', False, 
            create_model_settings('DUPIRE_LOCAL_VOL_MODEL',[201,401,4, 0.001]), 
            'PDE', 
            create_pde_settings(201, 401, -5, 5, 'MMT_NUM_STDEVS', 0.001, 'ADAPTIVE_GRID', 'CUBIC_SPLINE_INTERP'), 
            create_monte_carlo_settings()
            )

        # Create Pricing Settings with Duprie Local Vol model and MONTE_CARLO method
        self.duprie_mc_pricing_settings = create_pricing_settings(
            'CNY', False, 
            create_model_settings('DUPIRE_LOCAL_VOL_MODEL',[201,401,4, 0.001]), 
            'MONTE_CARLO', 
            create_pde_settings(), 
            create_monte_carlo_settings(8096, 'SOBOL_NUMBER', 1023, 'BROWNIAN_BRIDGE_METHOD', 'INVERSE_CUMULATIVE_METHOD', False, 201)
            )
        
        # Create Risk Settings
        self.risk_settings = create_eq_risk_settings(
            create_ir_curve_risk_settings(
                delta=True, gamma=False, curvature=False, 
                shift=1.0e-4, curvature_shift=5.0e-1, 
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK', 
                scaling_factor=1.0e-4, threading_mode='SINGLE_THREADING_MODE'),
            create_price_risk_settings(
                delta=True, gamma=True, curvature=False, 
                shift=1.0e-2, curvature_shift=5.0e-1, 
                method='CENTRAL_DIFFERENCE_METHOD', 
                scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'), 
            create_vol_risk_settings(
                vega=True, volga=True, 
                shift=1.0e-2, 
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK', 
                scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'),
            create_price_vol_risk_settings(
                vanna=True, 
                price_shift=1.0e-2, vol_shift=1.0e-2, 
                method='CENTRAL_DIFFERENCE_METHOD', granularity='TOTAL_RISK', 
                price_scaling_factor=1.0e-2, vol_scaling_factor=1.0e-2, threading_mode='SINGLE_THREADING_MODE'), 
            create_dividend_curve_risk_settings(
                delta=True, gamma=False, 
                shift=1.0e-4, 
                method="CENTRAL_DIFFERENCE_METHOD", granularity="TOTAL_RISK", 
                scaling_factor=1.0e-4, threading_mode="SINGLE_THREADING_MODE"), 
            create_theta_risk_settings(
                theta=True, shift=1, scaling_factor=1./365.)
            )

        # Create Scenario Analysis Settings
        self.scenario_analysis_settings = create_scn_analysis_settings(
            scn_analysis_type = 'PRICE_VOL_SCN_ANALYSIS', 
            min_underlying_price=-20e-2, 
            max_underlying_price=20e-2, 
            num_price_scns = 11,
            price_scn_gen_type = 1,
            min_vol = -5.e-2, 
            max_vol = 5.e-2,
            num_vol_scns = 12, 
            vol_scn_gen_type=0,
            threading_mode='SINGLE_THREADING_MODE'
            )

        # currency
        self.currency = 'CNY'
        # underlying
        self.underlying = '50ETF.SSE'

        # Create Market Data
        self.as_of_date = datetime(2020, 2, 21)
        
        # Create IR Yield Curve
        self.ir_curve = create_ir_yield_curve(
            as_of_date = self.as_of_date,
            currency=self.currency,
            term_dates=[
                datetime(2020, 2, 26), 
                datetime(2020, 3, 2), 
                datetime(2020, 3, 9), 
                datetime(2020, 3, 24), 
                datetime(2020, 5, 24), 
                datetime(2020, 8, 24), 
                datetime(2020, 11, 24), 
                datetime(2021, 2, 24), 
                datetime(2022, 2, 24), 
                datetime(2023, 2, 24), 
                datetime(2024, 2, 24), 
                datetime(2025, 2, 24), 
                datetime(2027, 2, 24), 
                datetime(2030, 2, 24)
            ],
            zero_rates=[
                0.0135486283791684, 0.0197762034605164, 0.0197686053073393, 0.0224838821372655, 0.0241740300538751, 0.0256822601972516, 0.0265096948143765, 0.0271330931714993, 0.0274314991366822, 0.0284834397783798, 0.0297276346662025, 0.0308410887945891, 0.032692683803743, 0.034410206396147
            ],
            curve_name='CNY_SHIBOR_3M'
        )

        # equity spot price
        self.underlying_price = 2.958

        # dividend curve
        self.dividend_curve = create_dividend_curve(
            as_of_date = self.as_of_date,
            pillar_dates=[
                datetime(2020, 2, 26), 
                datetime(2020, 3, 25), 
                datetime(2020, 6, 24), 
                datetime(2020, 9, 23)
            ],
            pillar_values=[0.05361055758784821, 0.01338941242891319, -0.001765240983827533, -0.002594662370949712],
            dividend_type='CONTINUOUS_DIVIDEND',
            interp_method='LINEAR_INTERP',
            extrap_method='FLAT_EXTRAP',
            day_count='ACTUAL_365_FIXED',
            yield_start_date=self.as_of_date,
            pillar_names=None,
            curve_name='DIVIDEND_CURVE_50ETF'
        )

        # Create Option Quote Matrix
        self.eq_option_quote_matrix = create_eq_option_quote_matrix(
            exercise_type='EUROPEAN', 
            underlying_type="SPOT_UNDERLYING_TYPE", 
            as_of_date = self.as_of_date,
            term_dates = [
                datetime(2020, 2, 26), 
                datetime(2020, 3, 25), 
                datetime(2020, 6, 24), 
                datetime(2020, 9, 23)
            ], 
            payoff_types = [
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'], 
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'], 
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL'], 
                ['PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'PUT', 'CALL', 'CALL', 'CALL', 'CALL', 'CALL']
            ],
            option_prices = [
                [0.0002, 0.0002, 0.0002, 0.0002, 0.0004, 0.0008, 0.0015, 0.0023, 0.0065, 0.0184, 0.0053, 0.0012, 0.0005, 0.0001, 0.0001], 
                [0.0027, 0.0033, 0.004, 0.0054, 0.0081, 0.0113, 0.0172, 0.027, 0.04, 0.0608, 0.0498, 0.0216, 0.011, 0.0066, 0.0046], 
                [0.0201, 0.023, 0.0281, 0.0345, 0.0433, 0.0541, 0.0668, 0.0818, 0.1004, 0.1229, 0.1325, 0.0935, 0.0659, 0.047, 0.033], 
                [0.0365, 0.0434, 0.0516, 0.0612, 0.0724, 0.0862, 0.1015, 0.1184, 0.1381, 0.1595, 0.193, 0.1483, 0.1298, 0.0897, 0.0702]
            ],
            option_strikes = [
                [2.5, 2.55, 2.6, 2.65, 2.7, 2.75, 2.8, 2.85, 2.9, 2.95, 3.0, 3.1, 3.2, 3.3, 3.4], 
                [2.5, 2.55, 2.6, 2.65, 2.7, 2.75, 2.8, 2.85, 2.9, 2.95, 3.0, 3.1, 3.2, 3.3, 3.4], 
                [2.5, 2.55, 2.6, 2.65, 2.7, 2.75, 2.8, 2.85, 2.9, 2.95, 3.0, 3.1, 3.2, 3.3, 3.4], 
                [2.5, 2.55, 2.6, 2.65, 2.7, 2.75, 2.8, 2.85, 2.9, 2.95, 3.0, 3.1, 3.2, 3.3, 3.4]
            ],
            underlying = self.underlying
        )

        # Build Volatility Surface
        self.vol_surf = eq_vol_surface_builder(
            as_of_date=self.as_of_date, 
            smile_method='SVI_SMILE_METHOD', 
            wing_strike_type='DELTA',
            lower=-1e-5,
            upper=1e-5, 
            option_quote_matrix=self.eq_option_quote_matrix, 
            underlying_prices=[self.underlying_price],
            discount_curve=self.ir_curve, 
            dividend_curve=self.dividend_curve,
            pricing_settings=self.bsm_analytical_pricing_settings,
            building_settings = [1, 0.5], 
            underlying=self.underlying
         )

        quanto_discount_curve = create_flat_ir_yield_curve(self.as_of_date, 'USD', 0.0)
        quanto_fx_vol_curve = create_flat_vol_curve(self.as_of_date, 0.0)
        quanto_correlation = 0.0
        self.mkt_data_set = create_eq_mkt_data_set(
            as_of_date=self.as_of_date, 
            discount_curve=self.ir_curve,
            underlying_price=self.underlying_price, 
            vol_surf=self.vol_surf,
            dividend_curve=self.dividend_curve, 
            quanto_discount_curve=quanto_discount_curve,
            quanto_fx_vol_curve=quanto_fx_vol_curve, 
            quanto_correlation=quanto_correlation,
            underlying=self.underlying)

    def test_build_eq_index_dividend_curve(self):
        expected = b'\n|\nx\n\x07\x08\xe4\x0f\x10\x02\x18\x15\x10\x02\x1a\x07\x08\xe4\x0f\x10\x02\x18\x1a\x1a\x07\x08\xe4\x0f\x10\x03\x18\x19\x1a\x07\x08\xe4\x0f\x10\x06\x18\x18\x1a\x07\x08\xe4\x0f\x10\t\x18\x17"\x055DAYS"\x0633DAYS"\x07124DAYS"\x07215DAYS*"\n d\x14\x7f\xb1\xdd\xdcK\xc0\x81\xff\xbd\x8e\xdf7"\xc0h\xa0\xac\\[x\n\xc0\'\xe4\n\xa5\xe8\xdb\x05\xc00\x018\x01\x10\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15'
        
        result = build_eq_index_dividend_curve(
            term_dates=[
                datetime(2020, 2, 26),
                datetime(2020, 3, 25),
                datetime(2020, 6, 24),
                datetime(2020, 9, 23)
            ],
            future_prices=[],
            call_price_matrix=[
                [45.800, 40.800, 35.800, 30.800, 25.800, 20.800, 15.860, 10.810, 6.090, 2.320, 0.530, 0.120, 0.050, 0.010, 0.010, 0.010],
                [45.990, 41.320, 36.410, 31.650, 26.230, 22.310, 17.660, 13.650, 10.190, 7.190, 4.980, 2.160, 1.100, 0.660, 0.460, 0.300],
                [49.900, 45.500, 40.930, 36.660, 32.610, 28.660, 25.260, 21.460, 18.540, 15.780, 13.250, 9.350, 6.590, 4.700, 3.300, 2.420],
                [53.280, 49.320, 45.240, 41.260, 37.360, 33.900, 30.450, 27.420, 24.330, 21.570, 19.300, 14.830, 12.980, 8.970, 7.020]
            ],
            put_price_matrix=[
                [0.020, 0.020, 0.020, 0.020, 0.040, 0.080, 0.150, 0.230, 0.650, 1.840, 5.000, 14.720, 24.510, 34.300, 44.370, 54.370],
                [0.270, 0.330, 0.400, 0.540, 0.810, 1.130, 1.720, 2.700, 4.000, 6.080, 8.750, 16.000, 24.960, 34.310, 44.370, 54.370],
                [2.010, 2.300, 2.810, 3.450, 4.330, 5.410, 6.680, 8.180, 10.040, 12.290, 14.710, 20.760, 27.730, 35.690, 44.570, 54.370],
                [3.650, 4.340, 5.160, 6.120, 7.240, 8.620, 10.150, 11.840, 13.810, 15.950, 18.410, 23.940, 30.380, 37.590, 45.670]
            ],
            strike_matrix=[
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40, 3.50],
                [2.50, 2.55, 2.60, 2.65, 2.70, 2.75, 2.80, 2.85, 2.90, 2.95, 3.00, 3.10, 3.20, 3.30, 3.40]
            ],
            spot=self.underlying_price,
            discount_curve=self.ir_curve
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_create_proxy_option_quote_matrix(self):
        expected = b'\x10\x01\x18\x01 \x01:\x07\x08\xe4\x0f\x10\x02\x18\x15B\x86\x03\n\x04\x08\x05\x10\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x1a\x1a\x14\x08\x01\x11\x99(c`\xcb*\xe1?\x19\x82 u\x02-\xc4\xa1@\x1a\x14\x08\x01\x11\x11Ii\xe7\xb9,\xe1?\x19j\x9fa\xd4#\x99\xa3@\x1a\x14\x08\x01\x116\x85\x19\xea?\xa9\xde?\x19\xb8\xa7\x11\x96{\xfd\xa3@\x1a\x14\x08\x01\x11Q\x06C\xd6a\x02\xdb?\x19\x08\xb0\xc1W\xd3a\xa4@\x1a\x14\x08\x01\x11\x03\xac\x0f9\xcfb\xd7?\x19V\xb8q\x19+\xc6\xa4@\x1a\x14\x08\x01\x11)\xcb\xe6%1\x83\xd5?\x19\xa6\xc0!\xdb\x82*\xa5@\x1a\x14\x08\x01\x11=\xe5\xde}\x1a\x82\xd3?\x19\xf4\xc8\xd1\x9c\xda\x8e\xa5@\x1a\x14\x08\x01\x11Q\xfb\x1c4\x03\x1a\xd1?\x19C\xd1\x81^2\xf3\xa5@\x1a\x14\x08\x01\x11\x9c\xad\x972\x0fY\xcb?\x19\x93\xd91 \x8aW\xa6@\x1a\x14\x08\x01\x11\xed\xd8\x96\xc4\xaa\xb0\xc7?\x19\xe1\xe1\xe1\xe1\xe1\xbb\xa6@\x1a\x14\x08\x01\x11\x82+\xc8\x80@\xe7\xc3?\x190\xea\x91\xa39 \xa7@\x1a\x12\x11\x1a\xaa\xbbB\xabc\xc2?\x19\x80\xf2Ae\x91\x84\xa7@\x1a\x12\x11\x9e\xa7)0&\x19\xcd?\x19\x1d\x03\xa2\xe8@M\xa8@\x1a\x12\x11&\xd7\xfd\xb1\xc1x\xd3?\x19\xbb\x13\x02l\xf0\x15\xa9@\x1a\x12\x11\xdd\x0b;\x97%\x9e\xd5?\x19Y$b\xef\x9f\xde\xa9@\x1a\x12\x11\x9e\x95Z\t\xa2\xd3\xda?\x19\xf64\xc2rO\xa7\xaa@\x1a\x12\x11m\x7f\x818\x04#\xe0?\x19\xcc\x91\x1e\x0cz\xdc\xad@"\x02\x10\x01*\x07\x08\xed\x0e\x10\x01\x18\x01B\x86\x03\n\x04\x08!\x10\x01\x12\x07\x08\xe4\x0f\x10\x03\x18\x19\x1a\x14\x08\x01\x11\x06\xec#\xfa{(\xe0?\x19\x0f9+\xa1\xd2\x92\x98@\x1a\x14\x08\x01\x11\xa2W$\xc6\xb7\xf9\xd2?\x19j\x9fa\xd4#\x99\xa3@\x1a\x14\x08\x01\x11\x02\xdb\x8c\xbd\x14\xb4\xd1?\x19\xb8\xa7\x11\x96{\xfd\xa3@\x1a\x14\x08\x01\x11rbr\xf1N]\xd0?\x19\x08\xb0\xc1W\xd3a\xa4@\x1a\x14\x08\x01\x11mW\x86b\xc3\xa7\xce?\x19V\xb8q\x19+\xc6\xa4@\x1a\x14\x08\x01\x11\xb8k\x1c\xd3.A\xcd?\x19\xa6\xc0!\xdb\x82*\xa5@\x1a\x14\x08\x01\x11\x9f|S:\x1cE\xcb?\x19\xf4\xc8\xd1\x9c\xda\x8e\xa5@\x1a\x14\x08\x01\x11\xf9\xce\x82\xba/\xfc\xc9?\x19C\xd1\x81^2\xf3\xa5@\x1a\x14\x08\x01\x11\xdc\xc3\xd6\xbd,3\xc9?\x19\x93\xd91 \x8aW\xa6@\x1a\x14\x08\x01\x11\x19]h?i\x04\xc8?\x19\xe1\xe1\xe1\xe1\xe1\xbb\xa6@\x1a\x14\x08\x01\x11\xc2(o\x86\xbd\xd6\xc7?\x190\xea\x91\xa39 \xa7@\x1a\x12\x11&\x05\x08qtL\xc8?\x19\x80\xf2Ae\x91\x84\xa7@\x1a\x12\x11=\xc8\xed\xf41\xd7\xc8?\x19\x1d\x03\xa2\xe8@M\xa8@\x1a\x12\x11%\xd0o\xb5T{\xcb?\x19\xbb\x13\x02l\xf0\x15\xa9@\x1a\x12\x11ign\x9b\xfe\xcc\xce?\x19Y$b\xef\x9f\xde\xa9@\x1a\x12\x11\xfa\tM\xb9\x8c@\xd1?\x19\xf64\xc2rO\xa7\xaa@\x1a\x12\x11\xcc&\xd4\n!\xe9\xe1?\x19\xfb\xbb,\x8d-5\xb7@"\x02\x10\x01*\x07\x08\xed\x0e\x10\x01\x18\x01B\x86\x03\n\x04\x08|\x10\x01\x12\x07\x08\xe4\x0f\x10\x06\x18\x18\x1a\x14\x08\x01\x11\xc6\xcd;\x86(P\xda?\x19\xc6\xe1<q\xdbe\x97@\x1a\x14\x08\x01\x11\x1a\x04\xebo\xe9q\xcf?\x19j\x9fa\xd4#\x99\xa3@\x1a\x14\x08\x01\x11\x16E\xb3\x0b\x07\xf9\xcd?\x19\xb8\xa7\x11\x96{\xfd\xa3@\x1a\x14\x08\x01\x11\x1d\xf4\xb3I\xc6\x16\xcd?\x19\x08\xb0\xc1W\xd3a\xa4@\x1a\x14\x08\x01\x11\xe6\xe5\x02\x87lH\xcc?\x19V\xb8q\x19+\xc6\xa4@\x1a\x14\x08\x01\x11\xff"\xd8\xb1\xde\xc5\xcb?\x19\xa6\xc0!\xdb\x82*\xa5@\x1a\x14\x08\x01\x11alBR\xcaQ\xcb?\x19\xf4\xc8\xd1\x9c\xda\x8e\xa5@\x1a\x14\x08\x01\x11\xec\x88\xcd\x16\xf8\xd5\xca?\x19C\xd1\x81^2\xf3\xa5@\x1a\x14\x08\x01\x11\xf5\xb3\x8d\xc7\xa1]\xca?\x19\x93\xd91 \x8aW\xa6@\x1a\x14\x08\x01\x113A\xaa\xca\xaf\x1e\xca?\x19\xe1\xe1\xe1\xe1\xe1\xbb\xa6@\x1a\x14\x08\x01\x11k\xb6\x1f\xb8K\x19\xca?\x190\xea\x91\xa39 \xa7@\x1a\x12\x116\x12\x87Iy\xfa\xc9?\x19\x80\xf2Ae\x91\x84\xa7@\x1a\x12\x11n\xb5\xd4\xe1\xad2\xca?\x19\x1d\x03\xa2\xe8@M\xa8@\x1a\x12\x11\xaa\x8c\xcf\x17x\xc3\xca?\x19\xbb\x13\x02l\xf0\x15\xa9@\x1a\x12\x113\xf6 \xe7\x04\x97\xcb?\x19Y$b\xef\x9f\xde\xa9@\x1a\x12\x11\xfd\xd6\xa2;\xfb?\xcc?\x19\xf64\xc2rO\xa7\xaa@\x1a\x12\x11\xbb\xa8\x0fV\x95\x98\xd8?\x19\xc6\xe1<q\xdbe\xb7@"\x02\x10\x01*\x07\x08\xed\x0e\x10\x01\x18\x01B\x87\x03\n\x05\x08\xd7\x01\x10\x01\x12\x07\x08\xe4\x0f\x10\t\x18\x17\x1a\x14\x08\x01\x11\xb5\xfe\x91\xf8\x15\xa5\xd2?\x19\x9dZW\x97\xf1\x94\x97@\x1a\x14\x08\x01\x11\x92\x9ao\xc1I\x8b\xcd?\x19j\x9fa\xd4#\x99\xa3@\x1a\x14\x08\x01\x11\x84\xa6R\xb6%\x06\xcd?\x19\xb8\xa7\x11\x96{\xfd\xa3@\x1a\x14\x08\x01\x11\xfa\xa9qx\x95\x8e\xcc?\x19\x08\xb0\xc1W\xd3a\xa4@\x1a\x14\x08\x01\x116\x0eO\xe2) \xcc?\x19V\xb8q\x19+\xc6\xa4@\x1a\x14\x08\x01\x11\x90\x0e\xe7`h\xbc\xcb?\x19\xa6\xc0!\xdb\x82*\xa5@\x1a\x14\x08\x01\x11#{\xc2\xf4\xb3\x87\xcb?\x19\xf4\xc8\xd1\x9c\xda\x8e\xa5@\x1a\x14\x08\x01\x11g\xbe\xb7jZG\xcb?\x19C\xd1\x81^2\xf3\xa5@\x1a\x14\x08\x01\x11\xa7F@\xb7\xa9\xfd\xca?\x19\x93\xd91 \x8aW\xa6@\x1a\x14\x08\x01\x11@P\xbf\xf7@\xd6\xca?\x19\xe1\xe1\xe1\xe1\xe1\xbb\xa6@\x1a\x14\x08\x01\x11\x01\xaa\xff\xf4V\xa3\xca?\x190\xea\x91\xa39 \xa7@\x1a\x12\x11\xd4\xc2F\xc4\x89\xba\xca?\x19\x80\xf2Ae\x91\x84\xa7@\x1a\x12\x11\xd8z3\xd4\xffs\xca?\x19\x1d\x03\xa2\xe8@M\xa8@\x1a\x12\x11\xfe\x01\xf7\'L\xe4\xcc?\x19\xbb\x13\x02l\xf0\x15\xa9@\x1a\x12\x11\xe9a\xe1p\x8a0\xcb?\x19Y$b\xef\x9f\xde\xa9@\x1a\x12\x11\xc2/\xf6]\n\xbc\xcb?\x19\xf64\xc2rO\xa7\xaa@\x1a\x12\x11o\xea\x84_1\xa7\xd0?\x19\x9dZW\x97\xf1\x94\xb7@"\x02\x10\x01*\x07\x08\xed\x0e\x10\x01\x18\x01J\rSZ50INDEX.SSE'

        result = create_proxy_option_quote_matrix(
            underlying='SZ50Index.SSE',
            ref_vol_surface=self.vol_surf,
            ref_underlying_price=self.underlying_price,
            underlying_price=2968.14
            )
        
        self.assertEqual(result.SerializeToString(), expected)
        
    def test_eq_vol_surface_builder(self):
        expected = b'\n \x08\x01\x10\x03\x18\x01 \x01(\x010\x028\x01I\xf1h\xe3\x88\xb5\xf8\xe4\xbeQ\xf1h\xe3\x88\xb5\xf8\xe4>\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xc3\xecB\xd9k!\x02@!\xc9"\xc8\xa43y\x0e@*\x8b\x01\n\x88\x01\xc3\xecB\xd9k!\x02@\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xc9"\xc8\xa43y\x0e@0\x039\xe0\xc0\x81\x03\x07\x0e\x8c?B\x8b\x01\n\x88\x01\x99(c`\xcb*\xe1?\x11Ii\xe7\xb9,\xe1?6\x85\x19\xea?\xa9\xde?Q\x06C\xd6a\x02\xdb?\x03\xac\x0f9\xcfb\xd7?)\xcb\xe6%1\x83\xd5?=\xe5\xde}\x1a\x82\xd3?Q\xfb\x1c4\x03\x1a\xd1?\x9c\xad\x972\x0fY\xcb?\xed\xd8\x96\xc4\xaa\xb0\xc7?\x82+\xc8\x80@\xe7\xc3?\x1a\xaa\xbbB\xabc\xc2?\x9e\xa7)0&\x19\xcd?&\xd7\xfd\xb1\xc1x\xd3?\xdd\x0b;\x97%\x9e\xd5?\x9e\x95Z\t\xa2\xd3\xda?m\x7f\x818\x04#\xe0?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00i\x0f/\x7f\xd80\x8d?\x8e\x03\xfd\x9e\xcf\x86\x7f\xbflo\xec\xf1`Q\x80?\x02\\x\xfe\x18]\x94?R"\n \xe0\xc0\x81\x03\x07\x0e\x8c?qI[\x0b\xa9\xa6\x07@\xc3\xecB\xd9k!\x02@\xc9"\xc8\xa43y\x0e@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x9f\x8b\xa1\xa6\xcb\x13\xf9?!\xc2\x16d{\xfb\xae\x17@*\x8b\x01\n\x88\x01\x9f\x8b\xa1\xa6\xcb\x13\xf9?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xc2\x16d{\xfb\xae\x17@0\x039Sr\xb1/,%\xb7?B\x8b\x01\n\x88\x01\x06\xec#\xfa{(\xe0?\xa2W$\xc6\xb7\xf9\xd2?\x02\xdb\x8c\xbd\x14\xb4\xd1?rbr\xf1N]\xd0?mW\x86b\xc3\xa7\xce?\xb8k\x1c\xd3.A\xcd?\x9f|S:\x1cE\xcb?\xf9\xce\x82\xba/\xfc\xc9?\xdc\xc3\xd6\xbd,3\xc9?\x19]h?i\x04\xc8?\xc2(o\x86\xbd\xd6\xc7?&\x05\x08qtL\xc8?=\xc8\xed\xf41\xd7\xc8?%\xd0o\xb5T{\xcb?ign\x9b\xfe\xcc\xce?\xfa\tM\xb9\x8c@\xd1?\xcc&\xd4\n!\xe9\xe1?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00\x17\xf3ak\xde\x99\xa3?\xb5*@\xc1\r:\xb3?\xd1@\xfb\xcc\xafz\x83?\xea<`\xd0\x0b\xe5\xb4?R"\n Sr\xb1/,%\xb7?\xc2\x16d{\xfb\xae\x07@\x9f\x8b\xa1\xa6\xcb\x13\xf9?\xc2\x16d{\xfb\xae\x17@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\xa8\xa1\xf8\xdc\xa8\xe0\xf7?!\xa8\xa1\xf8\xdc\xa8\xe0\x17@*\x8b\x01\n\x88\x01\xa8\xa1\xf8\xdc\xa8\xe0\xf7?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\xa8\xa1\xf8\xdc\xa8\xe0\x17@0\x039\xe1[Q<\x12\xbe\xd5?B\x8b\x01\n\x88\x01\xc6\xcd;\x86(P\xda?\x1a\x04\xebo\xe9q\xcf?\x16E\xb3\x0b\x07\xf9\xcd?\x1d\xf4\xb3I\xc6\x16\xcd?\xe6\xe5\x02\x87lH\xcc?\xff"\xd8\xb1\xde\xc5\xcb?alBR\xcaQ\xcb?\xec\x88\xcd\x16\xf8\xd5\xca?\xf5\xb3\x8d\xc7\xa1]\xca?3A\xaa\xca\xaf\x1e\xca?k\xb6\x1f\xb8K\x19\xca?6\x12\x87Iy\xfa\xc9?n\xb5\xd4\xe1\xad2\xca?\xaa\x8c\xcf\x17x\xc3\xca?3\xf6 \xe7\x04\x97\xcb?\xfd\xd6\xa2;\xfb?\xcc?\xbb\xa8\x0fV\x95\x98\xd8?J*\n(\x00\x00\x00\x00\x00\x00\x00\x00\xf0qS\x89\xfe6\xb3?\xa3\x93\x8c\xfa{\x9d\xb6\xbf\xcfTb\x87\x85p\x8b\xbf\xd2<\x03Gf\xf5\xc7?R"\n \xe1[Q<\x12\xbe\xd5?\xa8\xa1\xf8\xdc\xa8\xe0\x07@\xa8\xa1\xf8\xdc\xa8\xe0\xf7?\xa8\xa1\xf8\xdc\xa8\xe0\x17@X\x01\x1a\x96\x03\x08\x01\x12\x07\x08\xe4\x0f\x10\x02\x18\x15\x19\x15\xca\x94$\xb6\x10\xf8?!\x15\xca\x94$\xb6\x10\x18@*\x8b\x01\n\x88\x01\x15\xca\x94$\xb6\x10\xf8?\x00\x00\x00\x00\x00\x00\x04@ffffff\x04@\xcd\xcc\xcc\xcc\xcc\xcc\x04@333333\x05@\x9a\x99\x99\x99\x99\x99\x05@\x00\x00\x00\x00\x00\x00\x06@ffffff\x06@\xcd\xcc\xcc\xcc\xcc\xcc\x06@333333\x07@\x9a\x99\x99\x99\x99\x99\x07@\x00\x00\x00\x00\x00\x00\x08@\xcd\xcc\xcc\xcc\xcc\xcc\x08@\x9a\x99\x99\x99\x99\x99\t@ffffff\n@333333\x0b@\x15\xca\x94$\xb6\x10\x18@0\x039\x97-[\xb6l\xd9\xe2?B\x8b\x01\n\x88\x01\xb5\xfe\x91\xf8\x15\xa5\xd2?\x92\x9ao\xc1I\x8b\xcd?\x84\xa6R\xb6%\x06\xcd?\xfa\xa9qx\x95\x8e\xcc?6\x0eO\xe2) \xcc?\x90\x0e\xe7`h\xbc\xcb?#{\xc2\xf4\xb3\x87\xcb?g\xbe\xb7jZG\xcb?\xa7F@\xb7\xa9\xfd\xca?@P\xbf\xf7@\xd6\xca?\x01\xaa\xff\xf4V\xa3\xca?\xd4\xc2F\xc4\x89\xba\xca?\xd8z3\xd4\xffs\xca?\xfe\x01\xf7\'L\xe4\xcc?\xe9a\xe1p\x8a0\xcb?\xc2/\xf6]\n\xbc\xcb?o\xea\x84_1\xa7\xd0?J*\n(H{\xbd\x8eJ\xac\x99?\xc6\x058\x97X\xc0\x9d?\xe6\x17O\x83\x16\x0f\xd3\xbfo\xa8\x8e\xe1\x1f\x9b\xa0\xbf~(\x99\xc4\x16\xa8\x93?R"\n \x97-[\xb6l\xd9\xe2?\x15\xca\x94$\xb6\x10\x08@\x15\xca\x94$\xb6\x10\xf8?\x15\xca\x94$\xb6\x10\x18@X\x01"\x07\x08\xe4\x0f\x10\x02\x18\x1a"\x07\x08\xe4\x0f\x10\x03\x18\x19"\x07\x08\xe4\x0f\x10\x06\x18\x18"\x07\x08\xe4\x0f\x10\t\x18\x17*\t50ETF.SSE'
        result = self.vol_surf
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_european_option_pricer(self):
        expected = b'\n\xe4\x06\t\xcd.\x8b\xdf\\g\x07A\x1a\xcf\x06\n\x96\x01\n\x05DELTA\x12T\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\x88\x1d6\x82\x07\x13!A\x12\x00\x1a\x00\x12+\n\x12DIVIDEND_50ETF.SSE\x12\x15\n\n\n\x08\x9dD\xb0X\x07A)\xc1\x12\x05TOTAL\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x02\x93\x8d\x16^^&A\x12\x05TOTAL\x1a\x00\n\x9f\x01\n\x0eDELTA_EXPOSURE\x12T\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\xef\xec\xa9\x07q)\xd0@\x12\x00\x1a\x00\x12+\n\x12DIVIDEND_50ETF.SSE\x12\x15\n\n\n\x08n\x9fD\xc9&\xb0T\xc0\x12\x05TOTAL\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x1fZ\xeaO\nSR@\x12\x05TOTAL\x1a\x00\n0\n\x05GAMMA\x12\'\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\xef|\xac6\x88\'+A\x12\x00\x1a\x00\n9\n\x0eGAMMA_EXPOSURE\x12\'\n\x06EQUITY\x12\x1d\n\t50ETF.SSE\x12\x10\n\n\n\x08\x03\x16\xf5\x9dgT\x88@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xea\xd9\x05\xb5\xea\x95\x05A\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xa5EjZj\xb0\xe4\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08#5e\xb8\x1e\x11)\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\t\x9e\xd6\xff\xea\xf6(A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xba\x07\xcb\xff_\xf4\xbf@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08I\xcb\xf7\xc2\'\xda\xcc@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xbc\xb76\n\xbd\xa2\xf7?\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08F\x14Q\x97\xa0Z\xa9@\xd6\xe66~\xa4\xe6\xc2@\x15V\x98EtF\xd7@]e\xbb\xca\x00j\xe8@d#\x1e\x9a\xd8g\xf6@(\xd8\xa0\xa7Cj\x02A\xd0\xa8\x07\xbd\xda\xa9\x0bA\x93T\xb6\xaf^R\x13A>.\xad*sx\x19Aee\xe2XX\t Al\x8c\xdb\xca\x10{#AX1\xed\x8dJw\xb1@3\xc0,p\x8f\xd5\xc7@\xa2B\xd4\xf0Wj\xdb@\xbc\xa1\x15\xa3UQ\xeb@\x0b\xd1<\xfc\t%\xf8@G\xaaRB:R\x03Ae\x8b\xd6\xd4\x89\x7f\x0cA\x99Pe\xba\x96\xaa\x13A\xf5\xa4\xbe\x1f\x88\xba\x19AN\xa68&\t  A\x8e\xa7\xcf\xfa{\x89#A\xc5M]lN,\xb7@\xfeF<\xdf\xbeM\xcd@K\nz\xbf\xc9\xc6\xdf@a5\xc3\x00\x1bG\xee@\xe9\xa3\'\xf9s\xe3\xf9@6\xfd\x01yS:\x04A2N\xf2\xac\x9cW\rAy\xdf\x8c\x95\xc2\x05\x14A\xe0\xabi.\x08\x01\x1aA\xb2\x04V\xcf?9 Ag\xd5\xd7\x03N\x9a#A\'\x03\xca\xa7\xea\xcc\xbd@\xd2\x96H M\xa3\xd1@\x18\xd8\x13:\xe2*\xe2@\xd8\x04\x9e\xef\xaf\xa4\xf0@$r\x95\xb7\xe4\xa2\xfb@8B\xd8\xf3\x87"\x05A\x1c"\x15(\xbc1\x0eA\xa0\xd7C\xa0\x85c\x14A\xf35\xc7\x1c\x87K\x1aAY\xa0\x10\'\xd5T Ac\xb6JK|\xad#A\xd9\xa7;%\t\xab\xc2@\xa2\xc7\xfa\xca\xaf\xdb\xd4@\xf5\xf0\xba\xf2\xf8\x88\xe4@\xa5\xefR\x19B+\xf2@\x1e\x85\x06W3c\xfd@\xfcMg\x9d\xd1\n\x06A<\xf7\xeb8\xa0\r\x0fAl\x86\xab\x92\x90\xc3\x14A\x16\xab5`\xa2\x99\x1aA\xaa\xd4_\xe6\xa0r A%?m\xf9\xf4\xc2#A\xc6s\x81\xbf\x05\xe1\xc6@\xbc\xc9\xad\x9d\xb8K\xd8@}\x93%\x8dM\xfb\xe6@\t\x914\xc7\x94\xb6\xf3@\x05\xb3w\xf7=$\xff@u<\xc7Z+\xf3\x06A\x94\xa4\xb9\xca\x0c\xeb\x0fA\xde\xdf\xcdc\x9f%\x15Ab\xcf^\xfd\x00\xeb\x1aA@\x9b\xe5+{\x92 A\xac\x07^O\xa1\xda#A\x89\xfb\xa5V\x9c\x84\xcb@\x17zFpC\xef\xdb@D:\xfb\xa5\xcb\x7f\xe9@%\xed\xe3\x81\x13F\xf5@\xf3\xaa\xfe\x1f\xf4r\x00A\x1a\x03\x1c\xd6\x90\xdb\x07At\x1d\x00\xb1\xe7d\x10A\xbb\x1b\xe6\x81w\x89\x15A\xd6\x89\xa4\x05S?\x1bA\x82\x87|q=\xb4 A\xe2\x04?}g\xf4#A^nB\xb9\xaeH\xd0@{z\x9c\x18e\xc2\xdf@\xcb<x\xaa\x9d\x14\xec@\xec\xab]\xc0?\xd9\xf6@\x92\x93O \rT\x01A^\xde=V\xfe\xc3\x08A\xe6\x13\x03\xa3\xde\xd4\x10A[\xc4GT\xe6\xee\x15A\xd9\xf3)\xe4P\x96\x1bA\x86\x8e\x93\x1f\xc3\xd7 A\x92\xfb\xc3\x07,\x10$A\xf6E5<<\x01\xd3@\xf3\xa8\xa0a\xb9\xe0\xe1@\xf7\xa2e\'%\xb8\xee@}"~\x10\xado\xf8@\xa0\xabc\xcb_5\x02A\xect\xe7\x9fp\xac\tA\x88X\x8c\x08YE\x11A\x0e\xe0\xa6\xfc\xbfU\x16A\xf3.\xc5\x98\xba\xef\x1bA\xe2\x12h\xdc\xe9\xfc A\xa0\x8af\xd0\xd2-$A\xbe\xe9\x95\nv\xe9\xd5@\x154^O\x82\xf4\xe3@j\xb49\xecy\xb4\xf0@\xfa\xf3\xed\x01\xfe\x08\xfa@\xcc,ua\xe3\x16\x03A,C\xe4\xdd\xe4\x94\nA\xe8\xb9RGG\xb6\x11A\x1c\xbd\xd0M\xde\xbd\x16Az\xaa\x82\xf2VK\x1cA=\xb7\x1d\xad\x91#!AeA7\xd3?M$A\xdd\xd4g\x1e\xd5\xfe\xd8@\x17 \xc9\xbd\xfa\x1a\xe6@X\x94\xa1\xc0\xe2\x12\xf2@\x80;p\xa9\xe1\xa4\xfb@\x10\xe4\x82Q\x90\xf8\x03A\x7f\xf2u\x8eX}\x0bA\x19lj\xea\x9b\'\x12AL\x1a\xeb\xee\x1f\'\x17A!v|\xd3\xf2\xa8\x1cAejp\xf8\x9cK!A\xd1D\x0b\xabWn$AY\x96\x85!\xd9>\xdc@\x93\x8biB\xb0R\xe8@\xfbk]D\xbdv\xf3@\xc8$7\x9c\x11C\xfd@\x8e7\x81\x04`\xda\x04A\x98\xe2\xa7t\xc9e\x0cA\xac\xcd\x02FK\x99\x12A\x0b{\x8d\xa3g\x91\x17AL\x86\xed\x82`\x08\x1dAdU\x10v\xf0t!Ah\x08\xf3\xe9\xff\x90$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_european_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike=2.958,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_european_option_pricer(
            instrument=inst,
            pricing_date=self.as_of_date,
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        self.assertEqual(result.SerializeToString(), expected)
    
    def test_eq_american_option_pricer(self):
        expected = b'\n\x92\x07\t\x1f\xe9\x85\x15*c\x07A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\xda\xb5\xa7\xc16>)\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08{}F\xcb=M!A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08*g\'\xff\x11\\&A\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\xf4\x98z\xd8\xadT\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xec\x08\xed\xef\x8a`\xd0@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x945\x8c(QR@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xef1\xbfO\xd4\xe8+A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xff\xc2D,\x98\x01\x89@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00`P\xd8\xbc\x06E\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xf7\x102\xe7 \x0e\xe4\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00 \xccb~L(\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08}]0\x8d!\x0f)A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08PP\xae\x12\xaf\t\xc0@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x90\x1f|\x00\xde\x8f\x15\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00 \x81\r\xe0\xa9A\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08=\x8c\x8e\xb8HM\xa9@h\x84\xa7=\x0f\xde\xc2@[,5Sf=\xd7@\x93\xd1\xd2@\xeca\xe8@\xc6S\xe9>\x93a\xf6@\x0b\x8e\x9e\xda\xece\x02ALc\xee\xf5`\xa4\x0bA\xa9,8\x8d\'O\x13A\x9d\x99v/\xdft\x19A\xf9F \xf5m\x07 A"\x94\xbf7\x11y#ANF"\xdf\xe4n\xb1@\xfe\x1c\xc3f\x9f\xcb\xc7@\xd7\xf0,\xca}`\xdb@\xaaX\x8c*\xecH\xeb@[R\x9a7\xb3\x1e\xf8@\xcd\xbc\x8b\x1d\xebM\x03A*\x9e\x89\xea%z\x0cA\xe5G:\x80n\xa7\x13A\x1e\xc7r\x8d\x03\xb7\x19A\xa9\r\x9et%\x1e A\xd5\xc3t\x85\x81\x87#A\xe2;\xa9\xf7\x11"\xb7@8Y\xd8]wB\xcd@1\t\xbe\xc2/\xbc\xdf@\xd6\xb3\xd8\x98d>\xee@\xb5\xe1c\xf4\r\xdd\xf9@\xee~\x89g\x0b6\x04A\xca\x19\x0b\xc5LR\rA\x96\xef\xac?\xa8\x02\x14A\x8f\x92\x12i\x92\xfd\x19A\x8c\x9c\x19\xcfb7 A\xa3~:\xecX\x98#Aq\xa0-\xa8\xc2\xc0\xbd@A\x0c\xf5\xf2\x00\x9d\xd1@\xe0\x12\xb0V;%\xe2@f\x1d\\\xcf1\xa0\xf0@\x83o\xc6Lq\x9c\xfb@h\xf6\x90vF\x1e\x05A\xe7\xf9/\xa5~,\x0eA\xe7c\xdd?x`\x14A\xc08\x13\x81\x1fH\x1aAb\x18\xbd\xc6\xfeR A%\x03A\xbc\x8c\xab#A\xb2-j\xa5\xf8\xa3\xc2@tB\xe6\xad\xbf\xd4\xd4@u\xfa\xa6\xef\xfd\x82\xe4@\xfa\x17\xc42\xa4&\xf2@\x85B\\\x1f\xb4\\\xfd@\xe7\xc5\xf4F\x96\x06\x06AQ\x9b\x14\xb1s\x08\x0fA3.SM\x8f\xc0\x14A\xcb\x93BIH\x96\x1aA\x1b\xfcF\x08\xd1p A\xb7\xc9\xdd\x0b\x0b\xc1#A\xb9\x85\xf7&\xf5\xd8\xc6@@\x1aY\xdb)D\xd8@\x05`\t\xe2\x03\xf5\xe6@\x14\xc8\x0b\xe8\xd9\xb1\xf3@\xafqzV\xb4\x1d\xff@\x8dee\xcc\xf5\xee\x06A\xe97\x8b\xff\xef\xe5\x0fA\xf7\xaa6r\xa9"\x15AG\xd0\x9c\xc8\xb3\xe7\x1aA$]\xb0\xa9\xb1\x90 A\xb5\xd0a\r\xbd\xd8#A\xf1\xeb\xe7\xa9\x8a{\xcb@U\xd6\xad\xd6\x1b\xe7\xdb@10\xdav8y\xe9@\'\xa0\xed*>A\xf5@\xa1*=\xb6\xaao\x00A\xc2\x81"\xbd`\xd7\x07A\xddX\x13\x9d`b\x10A\xba^e.\x8c\x86\x15AY\xcf[\x15\x12<\x1bAwY\xf5\x1ez\xb2 Ao\x0c\xfd\xe4\x88\xf2#A\xe5:1\xec\xa5C\xd0@WP\xe2\xc0\xaa\xb9\xdf@\xf0\x98K\xc6\xc5\r\xec@\x92\x0eZ/R\xd4\xf6@\xf9\xfc\xe7\xa5\xbfP\x01Ao>%j\xd3\xbf\x08AC\x0c\xefc^\xd2\x10A\xddcL\xf9\x04\xec\x15A\xcaD\t\xa1\x1b\x93\x1bA\\\xaa\xcc\xcc\x05\xd6 A8\x8f\xf1\rS\x0e$A\xb0\xfaV\xe6\xb4\xfb\xd2@\xac4\xc7\xf3\x15\xdc\xe1@\x90\xff\x9a\t\r\xb1\xee@x:\x9dH\xa9j\xf8@b\x85\r\xb9\x0e2\x02A#\xb3\xb1\xa0J\xa8\tA.\xb0\x1b/\xdfB\x11A\x1e\x82:\x03\xe8R\x16A\xcb\x06\xc4r\x90\xec\x1bA\x1b\x13]W2\xfb A\xbe\thb\xff+$A\x8f\x9c\x0fEr\xe3\xd5@\xde\xe0\xa0\xbd\x9b\xef\xe3@!\xc4\x16\xd6\xcf\xb0\xf0@S-\x1b\xd3\xe5\x03\xfa@\xe6\xb4^"\x8f\x13\x03A\xa78\xd2\x92\xc3\x90\nA\x89\xd1\x0ep\xd3\xb3\x11A\x93\xfc.,\x0f\xbb\x16A\x84OLa7H\x1cA\x96(\xec\xc2\xdf!!A\x13;\xe6\xd8qK$Ay\xcc\xa1\x85W\xf8\xd8@\xab:\x13\x1c\xd4\x15\xe6@\tv\xcd\x92\x1c\x0f\xf2@X*\x98\xb6\xb6\x9f\xfb@\xb8sAE9\xf5\x03A\x9e\xe1\xf8\xc4;y\x0bA\xe0\xc1\n\xbc-%\x12A\xb4w4\'Y$\x17AK4\x8aV\xdd\xa5\x1cA8M>v\xf0I!A,2+\x08\x8fl$A\x94i\xda\xb3\xe47\xdc@\xfd\x1a\x93\x92LM\xe8@W\xe6\xbf\xca\xdcr\xf3@\x92\x80\xad`\xd5=\xfd@\xac,\xac\x80\x06\xd7\x04A\x00~\x82\xff\xb0a\x0cAZ_\x1bp\xe2\x96\x12A\xf5\xfb\x97\xc2\xa8\x8e\x17Ar k\xa1T\x05\x1dA\xe0+\xb3)Is!A\xd8s_\x7f<\x8f$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_american_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            strike=2.958,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_american_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)
    
    def test_eq_asian_option_pricer(self):
        expected = b'\n\x92\x07\t\xf8\x9f\xbd\xef\xcf!\xf9@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\xb8n\xe4\x0f\xa0\x81\x18\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08..\x1c\xc5\xc83 A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x9az\xdc]jh\x15A\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00p\x8c\x92Z\x13D\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08k\xf2|\xda@\xac\xce@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00tzn\x8e\x89A@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xb4\x99\x15r\x9e\x11?A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08A$\xe6\x16@\xd6\x9b@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xe8\xd2\xd6\x9a\xd8\x92\xc1\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08v\xe4\x99\xadSD\xf2\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xa8i\x9a\xd2!6\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xf6X\xaa\x16\xa8\xdc\x1cA\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xa8W%\x7f\xbdx\xb2@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xe8\xbb`\xc0\x1d3"\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xa0y\xfa~\xd1M\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x00\x00\x00\x00\x00\x00\x00\x00\x08J\x84\xf1\t\x8b_@$\xcai\xe1\xcc\x04\x97@s\x12\x01\x0fh3\xc0@\x8b\xda\xefO\xdd\xc7\xdd@8\x83;>\x0b]\xf3@\xe5\xdbk\x1f\x98@\x03A\xea\r\xa4|o\x8d\x0fAC\x8c\xe9\x12\x17\x9c\x16A0\x8dU%\xf3\xb3\x1dAg-;|\x7fn"A\x10\x83\x07x\x8b\xbf\x17@t\x91\xf5\x9b]`n@2q\x1e\xadru\xa0@7\xdfM|:\x06\xc4@\r\xfd#\x8f\x03\xa8\xe0@\xa8]\x18gPi\xf4@)\xa8\xd4\xf8\xf0\xa9\x03A\'O\x87\xe0c\xc6\x0fA\xd2\xf6\xe3$\xbf\xa6\x16A\xb2\xc2G\xb3\xaa\xb6\x1dA\xdd\x8c\x8e\xee\x97n"A\xd6\xd0%\xb0Q\xf43@\x88\r\xc7\x8b\xaf9z@\xear\x80\xa5\xa8\x87\xa6@" ,\n\xe4(\xc8@\xf5\x95\xae\xc3]t\xe2@/\xe9\x9f\x07\xb7u\xf5@\x99\xb9\x1byZ\x16\x04AU\x9d\xbf\xbb-\x02\x10A\x8b\xc5\xf9\xb4\x1d\xb4\x16A\x10N\xb5\xfe\x8f\xba\x1dA\xe4\xf7\xc0\x13\xddn"A\x0f\xd9aC\x17cI@\xde\xe8\x9a}/+\x85@\xa4\r\xcd3\xa9\xad\xad@P5\xcdh\x1f\x99\xcc@\x9c\x8f\xf1u\xc7G\xe4@\xfe_\xf5m9\x82\xf6@p\xef\x8c\xb2\x82\x85\x04A`\x8c\x08=\xf1#\x10A\x19F>\xc8\xf8\xc3\x16A*\xff\xb9G&\xc0\x1dAWI\xb3X{o"A\x7f\x1cN\xe9\xd5\xf2Y@c\'(9\xb4x\x8f@\xfc\x89\xbf\x81\xf2\xfb\xb2@5\x8a\x99\x88\xbd\xa4\xd0@_\xaf_\xb8Q!\xe6@\xb9P\xa6\x07\xd2\x8e\xf7@\xc1_\x93r\x9b\xf7\x04A\xd9\x86p\x17\xfdG\x10A\xc9\x00S`\x97\xd6\x16A\x888\'\xbe\xff\xc6\x1dA\x7f\xcb=\xa0vp"A\xce\r\xbc9\x14\xa1g@\xf3\xe9\x88x\x14H\x96@\xaaF=\n\xad\xc0\xb7@\xa2\xc7s\xc5\x8b\x1d\xd3@f3\x05\xe1\xc3\xff\xe7@_,\xa9\xd0z\x9b\xf8@n\x05\xeb\x99\xe5k\x05A2\xa0\xffr\xa5n\x10A\x0bQ\xfc\xb5c\xeb\x16A}\xcc\xf6\xb9\xb5\xcf\x1dA}Fa\xd8\xecq"A\x0c\x8dA>_Gs@\xa1\xfc\r\x1d\xb2\xcb\x9d@\xa8\xc5\r\xa4z\n\xbd@\x8eq\xa2\xa6h\xb3\xd5@\xebQ \xe26\xe2\xe9@\xdd\nB\xaa#\xa8\xf9@\xb0\x10\xc93\xa6\xe1\x05A\xef\x80O\xc4a\x97\x10A\x88\x94&d\xd3\x02\x17AP\x08u#Y\xda\x1dA\xe0V\xa5\xed\xdes"A\x8e\'\x84E\xe1\x16~@\x13\x91\\@\x7fl\xa3@\xe7H\xd9ftp\xc1@\xcaR\x9d\xdc\x8fa\xd8@\x8b\xd9h\x81`\xc8\xeb@\xff\xb8D\x0f\xca\xb4\xfa@f\xfdX&\xbdX\x06A\xd1\xcbm\xb3\xd2\xc1\x10A\x89e\xf8\xb3\x89\x1c\x17A\x07\xaeg\xa2D\xe7\x1dA\xd1\x02\xfaZ\x8bv"A\xf0\xb6\x0c\xe8vT\x86@h\x8f\x92\xda,\xdf\xa8@\xec\xfeBY\xd7\x9e\xc4@\x8c\x1a\xbf,\xb9&\xdb@\x83\xddU\xf5\xf8\xb1\xed@9v\xc1]\x8d\xc1\xfb@\r\xf0V7%\xd1\x06A\xec"\x82\xfd\x01\xee\x10A\x0b|+\x91C8\x17A\xf5\xef(f\xf9\xf5\x1dA\xd2\xc0j6\xc1y"A\xf5\x95K\xe6\x9aA\x8f@\x8b\x85K,\xf1\x15\xaf@O\tdk\x8c\x01\xc8@\xb7[\xb5\xc6u\x02\xde@)#E\x91\xc1\x9e\xef@\xa7;\xadTS\xce\xfc@<+\x11\x8d\xd8J\x07AuZ\xad\x9e\xf6\x1b\x11A\xef@\x95i\xe6U\x17A\x04\xdc\x8a%\x9b\x06\x1eA\x87\xb2d#\x9e}"A]\xf5CE\xba\x07\x95@\xa76z\x19\x1d\x01\xb3@\xc9\x9c\xd1JZ\x9b\xcb@9\x98\xa0\xa5\x18y\xe0@m\xaf\x1a\x17I\xc7\xf0@"\x1fxo\n\xdb\xfd@\x82\xde\xe2\xfd\xc8\xc5\x07A\x95.\xad(aK\x11Aq\x1b\x86\xd0]u\x17A\x11\xdd\x96J\x7f\x19\x1eA\x884\r\xe3@\x82"A\x0c\xda\xf4\xe2\x8f\x90\x9b@.\xc1\xa5\xb2r\xe3\xb6@\xe93\x96\x0f\xdej\xcf@\xb7%\xe0\x1f\xab\xfa\xe1@_\x93\xf62\x8a\xc0\xf1@\x97(I?\xc7\xe7\xfe@*b\xa2`\xb6A\x08A\xd8#\xb2\xca;|\x11A\xe5\x865\xfa\xc2\x96\x17A6BYF\x04.\x1eA\x071\x12\x0e\xce\x87"A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_asian_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike_type='FIXED_STRIKE',
            strike=2.958,
            avg_method='ARITHMETIC_AVERAGE_METHOD',
            obs_type='DISCRETE_OBSERVATION_TYPE',
            fixing_schedule= [
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],                
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_asian_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)
    
    def test_eq_digital_option_pricer(self):
        expected = b'\n\x92\x07\t\xe7\xd9?\xa1\xe2Z9A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\xa8\xbb\xe1)\xa5QR\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08D\x96\x8c\xeb`\xf6HA\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xfc.MG\xdcYNA\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\xa0\xe8\xbf\x83\x03~\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08H}{s\xe0\xa0\xf7@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00xI\xd7\x11\xddx@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xfd4\x028l\x95\xe8\xc0\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x02\x80x2\xc1\x06F\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\xe0\xfe\x80Y\xe9d\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08@2\'\xd3\x13\ri\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x80Y4i\x10Z\xae\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x10\xbc\x87\xab\x91\xd5\x01\xc1\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xd2[\xa8\xed\xd3\x96\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xfc\x90\x9aL\xdd9eA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00xM\xf8kc\x91@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x06\xdb&q\x87\x02\xf3@9t\x00\x8dM\x98\x08A\xfeD\xabl0 \x1aA\x91\xc0\xc9\x87\x13\x83\'A\x1b\xe3\x7f\x12so2A\x14\x1a\x07(l\xd49A\xac\xaf\xecv4\x8a@A\xda\xe3q\xea\xff\xc2CA>r#\x8f\rmFA\xa02\xd5#\x0c\x8aHA\xbc\xa2\xfb\x13\x1c4JA\x13z9\x84\xf8\x01\xf8@]s\xec-\xfd\x94\x0cA\x84\x83\xb3Wp\x8a\x1cA\xd1.\xa9\x14\xbd\x94(A,\xda\xa1c\x92\xb72A\x7ffx\x82\xd1\xc39A\xff\x91\xdf\x8d\xf4Y@A\xaa\x85\xcd\xaf\xeazCAE\xa4\xae\x8a\xcd\x1fFA(\xd9\xca\x89\x9aEHA\xab\x91&<.\xffIA\x86\xddZi\xdf]\xfd@\x98\xa3\xe3\x95\x08H\x10A\x92R\\\xf9\xda\xd5\x1eAU\xe2\xdb\x14D\x91)A*_"\xbf\xd7\xf92A\xd7\xb7\xff\x1an\xb69A\xf7\x90#\xd9\xeb.@A\xb8\xf5\xa8[\xba8CA\xd5e^?`\xd6EA\x8b\xc1\x83\x87\xbe\x01HA\x10\xe1C\xf8\x08\xc8IA\xf7>\xec\xc6\xce\x7f\x01A\x95\x85\x83U\x01@\x12A,\x91\xabU\x7f\x81 A\xac\xfe\x1e\x0c\x0b{*A\x88\xf8(Y\x1973Ai\xb7\x01\x1d\xc9\xab9Ak/\xeb7n\x08@A\x1a\xc7\xe8\xa6\xf0\xfbBAk\x82\xbfe\xe2\x90EA\xe9)%l&\xbfGA\xf2\x03\xf3\xcb\x91\x8fIAW\xc2>Y}i\x04A~!nL\xf1.\x14A\x98\x8d\xdd\xfcy\x89!A\xbb_\x0fc\'T+A\xdf\x84t\xb8\x06p3AL\x81\x91\x12\x80\xa39A\xae\xa9\x08>\xd5\xcb?A\xbe1T\xe4\x14\xc4BA\xd3\xff\x84\xebQOEAZ\xbf\xf5\x7fO~GA\xac\x9b\x01\xb1\x89VIA\x0cc\x8b}<c\x07AD\xb8&\xe1h\x12\x16A\x1cT\\\xd3\x91\x83"Aj\xdf\x04\x14j\x1e,Aio\xe9\xdc0\xa53A>\xef\x960B\x9d9A\xa5\xae\x0e\x8e\xcc\x8d?A\x12\x8f\xd3\x01\xb7\x90BAS\xf6#O\x98\x11EA5\x04iS\x90?GAh\xfa\xae\x0f\x8e\x1dIA\xeco`\xb0\xabe\nAPl\xe6\xd4\xc5\xe8\x17A\xe7\xb5tT\x8fp#A\xca\xae\x8bdh\xdb,A/\x1d,\xaa\x10\xd73A\x8e\xab\xa3{\xcc\x989A}\x0e\x1aj\xf1U?A\xd1\x89\x05\xc2oaBA*\x85ta\x92\xd7DA\xd0V\xfc\xaf \x03GA\xd1\x91\xab9\x1c\xe5HA\xb5\xe3\x1a\xe3\xb2j\rA\xf2\xc1\x1d\xb9\x03\xb1\x19A\xfd\xa5\xcf\x1eBQ$A\xc5\x97/\x8f\x84\x8c-A\xd4N\x87\xd1\x0b\x064A\xa0\x12%\xb7\xe6\x959A\xb5\x9b\xb5E\x91#?A4d\xe2w\xe15BA\xbc\xbc$U\x15\xa1DA\x93\x1a\xc0\x89!\xc9FA\xf5\xc7\x99\xf7\x95\xadHA\xb5Br^\xb06\x10A\x1a9\x96\xff\x94j\x1bAB.\xb0\xeax&%ARw1w\xf32.AC\xb2\xba<x24AM\xfb)\xa8a\x949A\x1b\xcc\x0c\x82\x12\xf6>Ao\xf7\xef%\xb7\rBA/\xa5P7\xf2mDA\xba\xcfdV\xa2\x91FA\xc3f\xabNEwHA\x82\xfdd\xf6\xe1\xb4\x11A#\xed\xcf\xa0D\x15\x1dA\xf5R\xc0\xe3\xfb\xf0%A\xaag@ \xc4\xcf.An\xb7f\x15\x9f\\4A$\xb5\xfe\xf0\x14\x949A\x86\xbc\xeb=\xf1\xcc>A\xbdBY$\xa4\xe8AA%I\x0e\x85\xf8=DAZ\xfd\x80\xae\xa5\\FA\x08\xc9k\xfd`BHA\xc0y\x10Sc.\x13A\xbf(\x1f\x95\x1e\xb1\x1eA\xa986\x8a\x89\xb1&A\r\xb4k\x93\xe4c/A\xca>\xcc\x00\xbf\x844AcG.G\xde\x949A\xf0\x82\x1dz\xba\xa7>AiX\xd3\xbcc\xc6AA\x0e\xfb\xf1\xda\xf7\x10DAI\xd1v\xa1$*FAa\xfaM\xe1\x0f\x0fHA/q]t\x04\xa2\x14A\xd01\x81\xd2.\x1f A\x14\xcdr\x12\xd6h\'A*b5\xc9%\xf0/A\te\x93\x10\x0e\xab4A\xf0\x17\xc8R\xa0\x969A}\xd6\x1eu\n\x86>A\xd3\xb6\xae\xf3\xb7\xa6AA\xf7x\xe3 \xc1\xe6CA\xd2G\x0e\xbd\x11\xfaEA(\x0e\xa3\x05l\xddGA \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_digital_option(
            payoff_type='CALL',
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            strike=2.958,
            cash = 1.0,
            asset= 0.0,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )
        
        result = eq_digital_option_pricer(
            instrument=inst,
            pricing_date=self.as_of_date,
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        self.assertEqual(result.SerializeToString(), expected)
        
    def test_eq_single_barrier_option_pricer(self):
        expected = b'\n\x92\x07\t\x9d\x91\xfch\tg\x06A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08`\xc4\x85\x9c\x93 )\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08f\xff(\xf8Z\x01"A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xf8\xa1o\xd1\x96Y&A\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\xc0\xf4\x16\x91\x95T\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08dE\xc8\t\x08\x0b\xd1@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xf0\x9a5 OR@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08s\xee\x99\xaa\x10.4A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x80\x19\x84c\xab\x14\x92@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00`\x8c\x1b\xb3\x9cG\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xee\xe9\x8e^\x16x\x03\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xa4`N\xb4\x96G\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08_d\xb9;\xba\',A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xb8T\xbe\x11\xf2\x04\xc2@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08$\x83\xca\x98\x84\x16>\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xc8\xfe\x17\xe7\xa5h\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08N\xec\xaa\xc7\xd8\x8b\x96@\x99\xd4x>\xa5\x7f\xb4@\xa7\xf0\x13\x87v\xdc\xcd@M\x84\xce\x95\xba\xfc\xe1@\xa2+\r\xec&n\xf2@\xca\xc5;\xa5>x\x00Ay\'y$2C\nAmE\xc8\xb9\x9a\x12\x13A>.\xad*sx\x19Aee\xe2XX\t Al\x8c\xdb\xca\x10{#A\x98\xe0\xedW$\xc8\xa1@\xf0\nS\xcaw\xab\xbc@a\xa5\x1ehE\xf9\xd2@\xe8\x10\xceb<6\xe5@L\x9f\x9es\xa1\x89\xf4@\xe8~tR1\x9c\x01A\xb5\x96\xfav\xaeI\x0bA\x00uV\x93\xd6s\x13A\xf5\xa4\xbe\x1f\x88\xba\x19AN\xa68&\t  A\x8e\xa7\xcf\xfa{\x89#Ag\x045\x93dQ\xaa@"\xc4\xb0\xa85(\xc3@\xa5\xb3\x9b\x94\x88_\xd7@]f^\xa0\xfb\x84\xe8@>\xab6hj\x9e\xf6@\x11\'\xb3@\xc4\xb7\x02AC\x8a+\xe1\\J\x0cA\xb4\x86iGp\xd6\x13A\xe0\xabi.\x08\x01\x1aA\xb2\x04V\xcf?9 Ag\xd5\xd7\x03N\x9a#AO\xe2iLV\x7f\xb2@9\xfd?\r\xb3\xac\xc8@oCj\xb7;\x13\xdc@\xbdx\x10\xa3\xb4\xe2\xeb@\xb1\xda:\x17E\xac\xf8@\xdb\xaf\xc0=<\xcc\x03Az\xc0`\xc8~F\rA}\xc4/\xce^:\x14A\xf35\xc7\x1c\x87K\x1aAY\xa0\x10\'\xd5T Ac\xb6JK|\xad#A5T8\x04w\xed\xb8@V\xc3\x8b\x97\xf5\xd5\xce@D\xf9k\xbb\x0f\x84\xe0@(\xbb\xcbP\xc0J\xef@3!\xb4\x0fX\xb3\xfa@\xa2\xcbZ\x14\xaf\xda\x04A\xe2\x9d\x82\t\x0c?\x0eA\x9eY\xf0\x15\x93\x9f\x14A\x16\xab5`\xa2\x99\x1aA\xaa\xd4_\xe6\xa0r A%?m\xf9\xf4\xc2#A\xbd\xcf\x9e\x8b\x998\xc0@)\xb4 \x88\xcb\xca\xd2@\xaa\x01nt\xd2\x19\xe3@\x0b\xef\xcf\xe3\xdb\\\xf1@28\x99\xde\x01\xb4\xfc@a1\xd5\xca\x07\xe4\x05A\xb0=]\xa2\xc44\x0fA<\xc9:\xd6\xfa\x05\x15Ab\xcf^\xfd\x00\xeb\x1aA@\x9b\xe5+{\x92 A\xac\x07^O\xa1\xda#A\xc29\xc9\x94\x91\x81\xc4@\x872\x1f\xda\x8dn\xd6@\x0bACQp\xc6\xe5@\xd2\x00-\x83\x93\x16\xf3@:\xf4I5\xbd\xae\xfe@;\x99c\xf7\x0b\xe9\x06A\x9f\x9d?\xfb\x1e\x14\x10A\x17\xbfn]\x82m\x15A\xd6\x89\xa4\x05S?\x1bA\x82\x87|q=\xb4 A\xe2\x04?}g\xf4#AG\x9a\x1c\x0c\x06L\xc9@\xb7|jpPO\xda@WVD\xc51\x86\xe8@i=\xf3\x98\xa7\xd1\xf4@t\xfd\x93\xbc\x07R\x00A\xbb^p\'a\xea\x07A\xbcv\xd2d\xf6\x8c\x10A\xc8\xd5\xce\xb2\x15\xd6\x15A\xd9\xf3)\xe4P\x96\x1bA\x86\x8e\x93\x1f\xc3\xd7 A\x92\xfb\xc3\x07,\x10$AY\xd9t\'&\x91\xce@\x15;.a\xa0f\xde@\x06\xebY\xa7\x04V\xeb@B\xfd\xcdg{\x8d\xf6@u2\xa3\x1b?J\x01A\xbbF\x02\xe0\x91\xe8\x08A\x05\xde\xc8b\x16\x05\x11A=\xb1RH\xa1?\x16A\xf3.\xc5\x98\xba\xef\x1bA\xe2\x12h\xdc\xe9\xfc A\xa0\x8af\xd0\xd2-$A\x8b\x9b|\xa2\xbe$\xd2@\x90\x83\x93\\PW\xe1@\x1c6\xf5\xcfb3\xee@\x17\xa5\xf6\xc7\xa2I\xf8@\xdb\x7f\xb7\x89D@\x02A\xd1*\x13\x07\x12\xe4\tA!\xe3\xa7\x15\xa3|\x11A\xf5\xd2)c\x12\xaa\x16Az\xaa\x82\xf2VK\x1cA=\xb7\x1d\xad\x91#!AeA7\xd3?M$A\x04\xa4\xe2\xa2\xae6\xd5@X\xe4\xc5`\x08\x91\xe3@\xc2W\x8b=\x1d\x8e\xf0@]\xdd\xba\xa1\xd4\x05\xfa@r!\x18JS4\x03A\xfa\xf5\x0b\xacB\xdd\nA\xb3aP\x13\xb9\xf3\x11A\xeb\x8c\x9bTW\x15\x17A!v|\xd3\xf2\xa8\x1cAejp\xf8\x9cK!A\xd1D\x0b\xabWn$A\x89\xef\x1f\xb2\x94z\xd8@*5:\xd3#\xde\xe5@%g\xa1\xbcl\x07\xf2@[\xab\xff\xef\xe0\xc1\xfb@g\x9b\xcc\\\xa1&\x04A\xe9\xa3*8u\xd4\x0bA\xf0\xbf\xca\roj\x12Af\x049\x94_\x81\x17AL\x86\xed\x82`\x08\x1dAdU\x10v\xf0t!Ah\x08\xf3\xe9\xff\x90$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_single_barrier_option(
            payoff_type='CALL',
            strike=2.958,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            barrier_type='UP_IN',
            barrier_value=3.2538,    
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[],[],[]],
            payment_type='PAY_AT_MATURITY',
            cash_rebate=0.0,
            asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_single_barrier_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_double_barrier_option_pricer(self):
        expected = b'\n\x92\x07\t\xc4\xbf@\x81$g\x07A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08j\x0f\x8a$\xebd)\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xf0\x7f\x1a\x0e\x95t!A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xb2\xd0\x85R.~&A\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\x14\xcem\x8d\xcdT\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08l\x14u\xfd\xc7\x85\xd0@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xa4\xa1\x11\x1amR@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08^m\xd9\xf4*!2A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x7f\x03\xbc\xca_>\x90@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x90\xad\xed\xd0\x04E\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xfb8\xc4\x91\x87\xd2\xdb\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xc0\xf9\'\xd1\xda \xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x12h\xd0\x95\xf1\x0c)A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08h\x0f\xcd\xb1H\x08\xc0@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x80\xe8\xd8#"\xc6\x16\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x00a\x95\x0b\xa8B\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08F\x14Q\x97\xa0Z\xa9@\xd6\xe66~\xa4\xe6\xc2@\x15V\x98EtF\xd7@]e\xbb\xca\x00j\xe8@iDu\xf5%f\xf6@\xf6\xf6X\xa1\xc0g\x02A}\xf7\x01\xa8 \xa9\x0bA\x93T\xb6\xaf^R\x13A>.\xad*sx\x19Aee\xe2XX\t Al\x8c\xdb\xca\x10{#AX1\xed\x8dJw\xb1@3\xc0,p\x8f\xd5\xc7@\xa2B\xd4\xf0Wj\xdb@\xbc\xa1\x15\xa3UQ\xeb@\xbd\xef\xc3\x1e\'$\xf8@?\xaa\xd8M\xfeP\x03A\x1a\x0e\xffC.\x7f\x0cA\x99Pe\xba\x96\xaa\x13A\xf5\xa4\xbe\x1f\x88\xba\x19AN\xa68&\t  A\x8e\xa7\xcf\xfa{\x89#A\xc5M]lN,\xb7@\xfeF<\xdf\xbeM\xcd@K\nz\xbf\xc9\xc6\xdf@a5\xc3\x00\x1bG\xee@\xae\x02\x0c<\xe9\xe2\xf9@\xfd\xe2\xa9\xd1\xba9\x04A:\xb5\x82\xf0pW\rAy\xdf\x8c\x95\xc2\x05\x14A\xe0\xabi.\x08\x01\x1aA\xb2\x04V\xcf?9 Ag\xd5\xd7\x03N\x9a#A\'\x03\xca\xa7\xea\xcc\xbd@\xd2\x96H M\xa3\xd1@\x18\xd8\x13:\xe2*\xe2@\xd8\x04\x9e\xef\xaf\xa4\xf0@d\xd3\xf8hi\xa2\xfb@\xcfe\x01\x02:"\x05Ae\x04G\x0f\xa71\x0eA\xa0\xd7C\xa0\x85c\x14A\xf35\xc7\x1c\x87K\x1aAY\xa0\x10\'\xd5T Ac\xb6JK|\xad#A\xd9\xa7;%\t\xab\xc2@\xa2\xc7\xfa\xca\xaf\xdb\xd4@\xf5\xf0\xba\xf2\xf8\x88\xe4@\xa5\xefR\x19B+\xf2@xa\x83\xcd\x96b\xfd@\x1bR\xee\xcc\x9e\n\x06A.\x15kv\x94\r\x0fAl\x86\xab\x92\x90\xc3\x14A\x16\xab5`\xa2\x99\x1aA\xaa\xd4_\xe6\xa0r A%?m\xf9\xf4\xc2#A\xc6s\x81\xbf\x05\xe1\xc6@\xbc\xc9\xad\x9d\xb8K\xd8@}\x93%\x8dM\xfb\xe6@\t\x914\xc7\x94\xb6\xf3@\xe0\xa2\'9[#\xff@\xa9\xd6o(\xf9\xf2\x06AC_N?\x03\xeb\x0fA\xde\xdf\xcdc\x9f%\x15Ab\xcf^\xfd\x00\xeb\x1aA@\x9b\xe5+{\x92 A\xac\x07^O\xa1\xda#A\x89\xfb\xa5V\x9c\x84\xcb@\x17zFpC\xef\xdb@D:\xfb\xa5\xcb\x7f\xe9@%\xed\xe3\x81\x13F\xf5@\xbdmL(Pr\x00A\xcd\xaf(\x03O\xdb\x07A\x98\xfd\xf8\xfa\xe1d\x10A\xbb\x1b\xe6\x81w\x89\x15A\xd6\x89\xa4\x05S?\x1bA\x82\x87|q=\xb4 A\xe2\x04?}g\xf4#A^nB\xb9\xaeH\xd0@{z\x9c\x18e\xc2\xdf@\xcb<x\xaa\x9d\x14\xec@\xec\xab]\xc0?\xd9\xf6@;\x00\xf9\x18)S\x01A\x05\xad\xc1\x91\xa1\xc3\x08A\x17\x04\xc5\xa7\xd6\xd4\x10A[\xc4GT\xe6\xee\x15A\xd9\xf3)\xe4P\x96\x1bA\x86\x8e\x93\x1f\xc3\xd7 A\x92\xfb\xc3\x07,\x10$A\xf6E5<<\x01\xd3@\xf3\xa8\xa0a\xb9\xe0\xe1@\xf7\xa2e\'%\xb8\xee@}"~\x10\xado\xf8@\x1e\x17\xdc404\x02A9\xb9z1\xf0\xab\tA\xce\x06\xb2\xccME\x11A\x0e\xe0\xa6\xfc\xbfU\x16A\xf3.\xc5\x98\xba\xef\x1bA\xe2\x12h\xdc\xe9\xfc A\xa0\x8af\xd0\xd2-$A\xbe\xe9\x95\nv\xe9\xd5@\x154^O\x82\xf4\xe3@j\xb49\xecy\xb4\xf0@\xfa\xf3\xed\x01\xfe\x08\xfa@\xe3\xa3@\xf5^\x15\x03A\x99\x0f\x9d\xcf9\x94\nA\'\x90\x8a\x048\xb6\x11A\x1c\xbd\xd0M\xde\xbd\x16Az\xaa\x82\xf2VK\x1cA=\xb7\x1d\xad\x91#!AeA7\xd3?M$A\xdd\xd4g\x1e\xd5\xfe\xd8@\x17 \xc9\xbd\xfa\x1a\xe6@X\x94\xa1\xc0\xe2\x12\xf2@\x80;p\xa9\xe1\xa4\xfb@q\x18\xcbC\xb0\xf6\x03AuL\n{}|\x0bA\xc8\xff\xd9\x04\x88\'\x12AL\x1a\xeb\xee\x1f\'\x17A!v|\xd3\xf2\xa8\x1cAejp\xf8\x9cK!A\xd1D\x0b\xabWn$AY\x96\x85!\xd9>\xdc@\x93\x8biB\xb0R\xe8@\xfbk]D\xbdv\xf3@\xc8$7\x9c\x11C\xfd@\x00\xad\xc0" \xd8\x04Ay\xd4\x86\x88\xbad\x0cA\xe1\x98\xffJ2\x99\x12A\x0b{\x8d\xa3g\x91\x17AL\x86\xed\x82`\x08\x1dAdU\x10v\xf0t!Ah\x08\xf3\xe9\xff\x90$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_barrier_option(
            payoff_type='CALL',
            strike=2.958,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=2.810,    
            upper_barrier_type='UP_IN',
            upper_barrier_value=3.106,    
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[],[],[]],
            payment_type='PAY_AT_MATURITY',
            lower_cash_rebate=0.0,
            lower_asset_rebate=0.0,
            upper_cash_rebate=0.0,
            upper_asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_double_barrier_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_one_touch_option_pricer(self):
        expected = b'\n\x92\x07\tj\xaaj\xa0\xa7a&A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x10M\x91\x85\x9c0)\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08hd\xda\xd8\x82\x11:A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xf0\xce\x8e@\xa9.\x1cA\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00 \x0c\xd5\xb3\xa2T\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xd8\x0e\xa3\xf1\xe0\xac\xe8@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xe0\xab\x1a@\x16G@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08$\x13(\xa4\xb0H*A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xff\xb3\xef^\xbe\x8c\x87@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x80\xd4r\xbc\x7fI\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x11\x91\x12\x0e\\\xf2X\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x80a!^\xb19\x9e\xc0\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08>W\xb8EYr.A\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xe0\x0e$\x18b|\xc3@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xc4\xe8\x8c\xc7Xqa\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x10<1\x06\x94\x8c\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\xe5\x18\xb2~\x88\x86\xcf@\xc7\xa9U\xf0\xa5\xfe\xe7@\xfd\x12o\x07\xf7\x84\xfd@\x12Pmf\xdaC\x0eAz\xb8"\xd2\xce\x90\x1aA\xc1\xc8\xe9w\xe8r$A\x8cj\xf1\xad\x186,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AUifC\xa2\x08\xd6@p\xd7\r\xd1\xaaE\xee@\x05\x7f\x03\xe9|2\x01A\xef\x89\x9fDf\x9b\x10A\x1e\xcc1mC\xec\x1bA\xcd\x8d\xa1C<\xe0$A-\xb7\\]aK,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A39\xa0\xd5\xb9a\xdd@r[\xe0\xf8\x9d~\xf2@\x16\x84Z\x05\xdd\xa1\x03AK\xcdF\x01u\x02\x12A\xba\xd2!\xd8\x9f,\x1dA\xfc\x0f\x97\x05\x96C%A\xb9\xd9\r\x9a\xc3^,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A%?6@j\xda\xe2@N\x0c\xc2\xfb\x1d\x04\xf6@\xb6\xad\xe3\xbcW\n\x06A|\x94i\x13\rW\x13A\xdbgh\x15\x96T\x1eA\xd6\x9e\x11\xcfA\x9e%A<\x80m!}p,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\n\xb8\xde\x86\x87q\xe7@f\x0c\x0e\xe6W\xa6\xf9@R\xe9\x1f\xafOg\x08A\xd0\x84\xb5F\x8f\x99\x14A\xf5\x07\xcb\xdc\x92f\x1fA\xbd\xae\x83\xebT\xf1%A\xc9M\xfe\xc0\xc1\x80,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x14v\x1f}\xaee\xec@\x06\xc4\xe5\xbe\x14Z\xfd@\xd6~\xe7\x82{\xb5\nA\x11\xa8\x01\x90\xa0\xca\x15A\x00\x95\x8bF`2 A&\x02\x8c\x9d\xb8=&A\xf5\x82\x9c=\xbd\x8f,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xb3>\x9aV/\xd3\xf0@\xa7\xcb6S\xf2\x8a\x00AF\xa1Zi\x9d\xf2\x0cA&\xa6b\xce\r\xeb\x16A\xf8\xa82%\x86\xa8 A\xa9\xcd\xa7u2\x84&A(\xa8\x10\xd0\x94\x9d,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xdd*\xdd\xcd\xed\x91\xf3@\xcf\x82+\xa1\x00i\x02A2\xdb}\x82F\x1d\x0fA\xd7\xc5"\x91\xb8\xfb\x17A\x00h\xaa\xf5\x95\x16!A\x88\xafq\xdaj\xc5&A]\xcd\xe9Ph\xaa,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x98\x8a\x1ef\xd0g\xf6@l\x01Z\xac\x10D\x04A\xe1S\x82\xd6R\x9a\x10A\xd4\x99$2\x8a\xfd\x18A\xc4\x05P\xfcQ}!A\x7fx\xb91\xf2\x01\'A\xd2\x00\xb5&S\xb6,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xf8V*+ON\xf9@;\xe4\xd7\x9a\xa3\x19\x06A\x8e\xc3+20\x9c\x11A\x18\xf8VYk\xf1\x19A\x1c\x878\xaff\xdd!A\xab\x9e\xa8\xfcD:\'A\x9a\x96\xba\x04m\xc1,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AD\xb8\x1e7\xa7?\xfc@\x0cj]\xc2\xc7\xe7\x07A\xc7\x06\x04\x07:\x94\x12A\x9b\x08\x13\x8e>\xd8\x1aA\x9e`\x82Qm7"A\x9e_*$\xcfn\'A\xceO\xda\x84\xca\xcb,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AC6B{\xd86\xff@\x06\x18\x1b\xd5\x01\xad\tA\x18\x88\xc4\x12\x8f\x82\x13A\xc3\xc0\x9b\xe2\xdc\xb2\x1bA\xcd\xf7\x9aI\xee\x8b"A}{\x8f\xa3\xee\x9f\'A\x08\xd8\x86\xa4}\xd5,A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_one_touch_option(            
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            barrier_type='UP_IN',
            barrier_value=3.106,    
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[],[],[]],
            payment_type='PAY_AT_MATURITY',
            cash=1.0,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_one_touch_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_double_touch_option_pricer(self):
        expected = b'\n\x92\x07\t\xd8\r\xddp\x10\x8d.A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x10[y\xca!M\x1e\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x0bR\xc22\xd1gB@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x00\x08\x95/T6@\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00 \xe8|\xa4\xd2H\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\x00\x10\x86\x04l\xf1?\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x00\x00\x10\xb3Jb?\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xcc\x0cW\x92]\xd2\xb4@\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\x00\x80\xf1\xe0\xa7\x12@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\x00\xc0\xbf\xf5\xcf\xc8\xbf\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x7fg\xbd4\x01\'\xb0\xc0\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x00\xac^\xf5\x91\xf3\xbf\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x7f\xa4g\xb9\xff\xb0@\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xe0\x8a\xff4\xc2E@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x80v\xac\\\x96\xbb\x17\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x00=\x04\x1fqC\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xdc\xc2\x086J\x85.A\xa7e\xa1\xba\x16u.AXPM4T\x86.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x896\xe9\xf7k\x89.A\xba\xf1P\xeeu\x81.A|\x85q3\xe5\x89.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xd8\xc9\x89\xf5\x82\x8b.A\xc5\xd9\x83]\xc1\x87.A\xdb\xd4\xa6b\xb8\x8b.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xa8\xfa\x14j\x86\x8c.AR\xfc\x06\xbc\xd2\x8a.A\xf2Q\xdd)\x9d\x8c.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A/1>\x14\xff\x8c.A\n)\xdb\x81A\x8c.A\x95)\xc8n\x08\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x88\'G\xe64\x8d.A\xcc}\xdc\xad\xe5\x8c.A\x12\xa0w\x9c8\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x88\t\xd6\xecK\x8d.APz\xf6 ,\x8d.A\t\xd5\x8dXM\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x9dY\xf5_U\x8d.A\x87\xfa\xf5\x1dI\x8d.A\x99\xa64\xe6U\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\x93\x16m\x18Y\x8d.A\xd0\xb1t\x8eT\x8d.A;\xe8/HY\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AS\x90;\x80Z\x8d.A\xa9!\xf1\xe2X\x8d.A\xdb\r\x9b\x90Z\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.AT\xab\xa7\x02[\x8d.A\xaa\xbesuZ\x8d.A\xf9\xd3\x0f\x08[\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xf9p\x050[\x8d.A\xfe$\xae\x01[\x8d.A~\xa5\xbd1[\x8d.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A\xaa0\xc9\x8c\xe2!.A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_touch_option(            
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=2.810,    
            upper_barrier_type='UP_IN',
            upper_barrier_value=3.106,    
            barrier_obs_type='CONTINUOUS_OBSERVATION_TYPE',
            obs_schedule=[[],[],[]],
            payment_type='PAY_AT_MATURITY',
            cash=1.0,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_double_touch_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )

        self.assertEqual(result.SerializeToString(), expected)
    
    def test_eq_single_shark_fin_option_pricer(self):
        expected = b'\n\x92\x07\t\xfdG4\xb6\x8bIw@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\xa0\x90\xd4\xcb\xc7G`\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xcdsU\xa8\xeb\xdf\x9e\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08@s\x9e\xea:\x10N\xc0\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\x80\xaa\xd2}\xac\x8a\xbf\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08d\xf3Q\xf7\x889M\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x80\x8eq\xc0\xa0x\xbf\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xe4\x86W{\xbdp\xdc@\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08p\x13?\x03^{9@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x003\x1c\x01$X\n@\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xa2\xed\\\\\'9\xd6@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xd0R%\xe2\xf5\xec\x1a@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xe1$M\xdc) \xb2\xc0\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x0c\xd3\x10\xc8h3G\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x0853Dd\xdaa\xf1@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x80\xed\x99\xa9\xa3z\x1c@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08O/]+R\xd9k@V\xe7{\x18\xd4\xef}@\x14\x83\x01\x9a5\xf2\x88@\xbc\xa8}p\xd2?\x90@td\x1cQ-Z\x90@\xa6\xca\xe9r\xbc4\x87@\x92\xbcO\xb9\x96\xa6l@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x11\x1fzP\xd2\xf3o@\xa4\x00\xe9_\x7f6\x7f@\x92\xeb\xec\xcbC/\x88@1Y\x19\xf1\xae\xe5\x8d@\xce\x91\xb7\x13( \x8d@\x99l =\xd42\x84@\xb7X\x07e\x81<i@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x08\xbc\xef\x92\\\xb9q@\xa0\xba6\x86\xab\xe9\x7f@2\xc7-\x97l=\x87@\xe0\xe6$\xd7Sy\x8b@\xba\xa8 \xfc\x08\xff\x89@\xe3\xc2\\\x0c\x99\xb0\x81@.\xc8\x03q\xd4Qf@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\'5\xcd\xea:%s@\xfc\xf2\x8b\x84T\x11\x80@\xeb\xdbJz\x020\x86@7\xb4\xcd-2I\x89@\x88\r\x07\xf3\xb4K\x87@\xf7\x93+\xc3E)\x7f@\xee5,\x9c8\xe1c@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\t\xd1\xb6\x8d\x89>t@\xdb\xadQY\xfd\xf9\x7f@\xdf\x88(<0\x15\x85@\xa3\xeb6\xbd;>\x87@u\x04{\xd0\x03\xf5\x84@2\xfc\x87\xf0\x8c\x96{@\x04\x01\xc0`\x90\xe2a@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xa5\xe9\x82K\xb6\nu@\xc0\\\xf0\xa5\xba\x85\x7f@\xf8\xc7\x12\xf1\x0c\xf7\x83@k\xfb\xb6\xa3wb\x85@\x96\x87[\x91\x99\xec\x82@+X\xcd:\xd6\x96x@\x82\xed\x12\xd8\xd0 `@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x9c\x1c\x1c\xa9\x91u@6\xe8`\x8a\xd0\xd8~@\xc4\x85s\t\xbb\xe4\x82@|\xb5JQ\xe3\xb2\x83@m8+,\xd9.\x81@%\xbe\xbb\xa8l\xf7u@\xccu1\xb4eY]@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xfc\xday\xd5^\xdcu@\x1dJf^\xfb\x02~@\xf8\xe1z\xba6\xd2\x81@e}\xf6a`4\x82@\x9fM\xfam\xb2@\x7f@`\x1b3\xaf\xd0\xb2s@3\x1f\x94\xd5\x83\xbeZ@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x1c\xd1\x19\x8a\xf6\xf3u@]@\xcc\xea6\x1d}@>\x95\xef\xaa\xfb\xca\x80@\x17)\x0c\x01\x00\xd2\x80@\xc4\xe1\xa2x\xdf\x83|@\x82m(\x8a\xa2\xc2q@k\xe2\xfe\xe0\xc2{X@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\\!.8\x1f\xe1u@z<\x97\x0c\xd9\x18|@\x86F\x9cs\x90\xa1\x7f@\x84\xb1(Ml"\x7f@x<\x9c\xdb;&z@\xafyr\x88q\x06p@\xe4\x06\x86\x10\x15\x96V@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00H\xb7r\x0e\xb6\xb4u@C\x9a\xde&F\n{@J}\x0f\xc1R\xd6}@\x93\xba\xed)\xe0\xea|@E\x10\xc0IR\x01x@"\xcc\xd1\x9e\x85\x0bm@f\xb6\x1ce\xdc\xd8T@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x18\x97\xda%\xcccu@1\xd9\xc4\xde\xd4\x02z@25\xc6\x85_\x19|@8E_\xf0-\xdbz@\x06\xee$_`$v@\x83p\x1a\n\xefNj@\xa8\x8b\xd4\\``S@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_single_shark_fin_option(
            payoff_type='CALL',
            strike=2.958,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            gearing = 1.0,
            performance_type='RELATIVE_PERFORM_TYPE',
            barrier_type='UP_OUT',
            barrier_value=3.10590,    
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            payment_type='PAY_AT_MATURITY',
            cash_rebate=0.0,
            asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_single_shark_fin_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_double_shark_fin_option_pricer(self):
        expected = b'\n\x92\x07\t\x12\xf6#\xd9=\x13\xcd@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08@\xc8\xed\xb3\xef\x02\xa8@\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08_\x87G&\x8fj\xb6\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xe8v\xc2\xde\xcdc\xc4\xc0\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\x80\xd8\xfb\x91\xab\xd3?\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xc0\x91\xbf\xfc\xe37e\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xd0\x12G\x10\xb4\xf0\xbf\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xac\xc4\xf7\xdf\x06O\x0e\xc1\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00>\xba\x8a\xe6\'k\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\xc0\xf9`\xa2\x83wk@\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x86\xffI\xd8\xe3\xee\x01A\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00B\x8f\xc2N\xbaE@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x0c\x87\x81?0k\x12\xc1\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xdc\xac \xf5p\x93\xa7\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x087A\xd0\x97\xda\xbfRA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08?b\xe5\xa1\x14\xb8~@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xeb\xf6\xd5\xcc4V\xcc@\xe3\xd9\xbd\x92\xd3x\xdf@Dv\xbd<\x11\xef\xe1@\xda\x99\x18I\xb1&\xdc@\xfb\x02\xb16\xeav\xc8@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xcd\x82\xab\x16\xc3c\xc8@"\x8dgv\xb8\x04\xdb@\x8a/\x8f\xe8r\xed\xde@\xf1\xa54\xd6\x16u\xd8@T\xe9\xca3\xa4z\xc5@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xea\x07\x99\x12\x9c\xdf\xc4@iT\x91\x02\xb7\x0e\xd7@\xb0f,Bt\x80\xda@k\x02\xe0\xff\x11\x0f\xd5@\xcb\xf0\xee2\xee\xac\xc2@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xb6\x94\xbb\x9a\x81\xb7\xc1@\t3\x8ad\xf7\x8c\xd3@\xb8\xd5\xb8P\xc0\x84\xd6@\xa9[P\xec-\x06\xd2@\x10\x13V\xb5\\%\xc0@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x8a\x07M[\x82\xf9\xbd@\x96T\x02\xdd\xbcr\xd0@\x9b\xb8\x00\xc6\xf0\xf8\xd2@\x03\xad\xb0\xe4!\x96\xce@1m\x82.\xb1\x9b\xbb@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe7i\x87+\xaf:\xb9@9\x7fp\xec\xe3\x89\xcb@\xc2\xee\xa6L_\xd0\xcf@\xee5\xd2\xea:\xbc\xc9@\xaelx\xb2\xe9~\xb7@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd1W\x89\xda\x18\xb5@r\xf2\xe6\x9cf\xee\xc6@\xf2\xd5\xfbEG\x82\xca@\x95w\x01#\xdf\x8c\xc5@\xec\xf7\xa8q\x15\xce\xb3@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe4\x18X\xebj\x8a\xb1@\xe6\xc9\x7fn1\xf6\xc2@\xca\xe8\xbc\xe9\x19\xdf\xc5@JJYLd\xe6\xc1@\xaaV;$\xe6\x8e\xb0@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe8\xfa\x19%\x96\xeb\xac@V3\x13\x90\x9d+\xbf@\x12\xe7\xd6\x83\xea\xeb\xc1@RR\x0f\xdeX\x95\xbd@R\x0f\x924\xc1\x99\xab@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x12I\xd2\xdc\x1f\xcd\xa7@,J\xe2o\x82s\xb9@\xc5Mn\x13\x04;\xbd@\x9d\xa1\x06-\xed8\xb8@\xa9\xf4$@]\xcd\xa6@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xaa\x1e!\xd6\xe2v\xa3@ki\x90\x98X\xad\xb4@XV@\x82C\xb2\xb7@\'\x8a\xd3\xd1\xbb\xbc\xb3@\x1b\x0e\xa1&\xdd\xc5\xa2@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00S\x92\t\xf8\xf4\xab\x9f@\xb0\x95\xec\x86\x14\xba\xb0@\xbb\x03\x05\xfbw\x14\xb3@\x95\x1b\x16\x9d\xc9\x03\xb0@e)\x90\xfaG\x9c\x9e@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_double_shark_fin_option(
            lower_strike=2.958,
            upper_strike=2.958,            
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_participation = 1.0,
            upper_participation = 1.0,
            performance_type='ABSOLUTE_PERFORM_TYPE',
            lower_barrier=2.662200,    
            upper_barrier=3.253800,                
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],
            payment_type='PAY_AT_MATURITY',
            lower_cash_rebate=0.0,
            lower_asset_rebate=0.0,
            upper_cash_rebate=0.0,
            upper_asset_rebate=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_double_shark_fin_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_range_accrual_option_pricer(self):
        expected = b'\n\x92\x07\t\xe6s\x88\xfd\x08\xc5\xbc@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08@\x16FH\xd2\'\x82@\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08!3s@Hf\x83\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08(\x1dVt8\x89\xb0\xc0\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\x00i\x98\xfd\xbe\xad?\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\x1c\x17U\xe2\\2\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xa0\xce`\xb5\x17\xdb\xbf\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xa7\xec\xc8\xdb\xc70\xe7\xc0\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00{\xe0G6\xc7D\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00.`A\xef\xfd#@\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08#\x1e\xf2\x84\xe7\xff\xc2@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00B\x02J\x17\x05\x07@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xbe\xe4\xa5\xf5!\xbe\xcd\xc0\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xe0\xa6~~\x0b\tc\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x84\xf2\xda\x9f\xa7b\xf8@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x88\xa9\x98\xfb\xf9#@\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\xa5\xd1\xbaf\x0e\xb5\x85@O8A0o\x9b\x9a@\xf64`\x01\x0b\xe4\xab@\xc8 \xf3/jD\xb8@pq\x8f\xff\xe9\x1b\xbe@R\xba\xb8\n\x13^\xbf@1:\xef\x93\x9b\xeb\xbc@\xf1\xaan\x1c\xd9\xf2\xb6@F\x0bO\xf0(y\xac@\xfa\x81\x15#\xc8\xf5\x9f@\x07\xa9\xf6\xf1K-\x91@\x81\xc4:\x8b\x92?\x88@\xc9\x94\xbf\xa4\x1d\x02\x9c@\xa9BI\x92\xaa\x16\xac@/=\x8d7\xb9\xc1\xb7@\xb2MxSXU\xbd@\x12d\xef\xb0r\x95\xbe@f\r\x95j\xa8H\xbc@+\xad\xc2\\;\xa5\xb6@\xcf\xe7xC\xae\xea\xac@\xe79\xc6"\xba\xdf\xa0@\xb8WV\x84u\xf5\x92@\xc2\xfb\xd2p\x86\xb5\x8a@\xaf?\xb8\xf4\xd4F\x9d@e\xa3\x95\\H9\xac@\x88\r>\xbblC\xb7@\xcb\xf75S\xfa\x94\xbc@\x13F}\x1bd\xd1\xbd@\xd2[\xabv+\xa9\xbb@X\xdb\xcb(\xf7V\xb6@O\xac\xf3E\xb0F\xad@\x01\x1d\x1973\xaf\xa1@\'\x08`\xd1\xc2\xa8\x94@\xefhs\xd8Y\x12\x8d@)y\xac\xf7\x14k\x9e@\xbd\x9bF\x0e\xa3M\xac@\xb2\xd8\x80 -\xc9\xb6@\xab\x1a\xc9\xb2\x19\xdb\xbb@I\xcfs\x89\x9e\x12\xbd@a\xff\xc2\x13f\r\xbb@\xdd?\xac{\x13\x08\xb6@1:\xad\x8a.\x8f\xad@\x95\x92W\xe7+j\xa2@\xb7b\x87\xd0\xeeD\x96@\xe2\x14\xb1\xe8\xbfR\x8f@\xb3\x16J\xdb\x93p\x9f@\x9d\xd89\x84ZU\xac@\xcb\xe1B\x97\xbbR\xb6@\xa4D)\x04\xd6\'\xbb@QR\'\xbc\x97Y\xbc@\x13\xbe\xed\xcb\x8bu\xba@dY\xde\x93\xb1\xb8\xb5@*\xce:v\r\xc6\xad@\x9dK\r\x00\xa5\x11\xa3@\x0b\xbe\xcf\xb1\x85\xc8\x97@6\x9b\xc8\x11?\xba\x90@\xed.v\x01\x95,\xa0@&\x9ctj\xeeQ\xac@\xb6\x87\xfe;\xe9\xdf\xb5@\x13\x00\xf8+0{\xba@\xdc\x93:\xe6\x93\xa6\xbb@\xd2\xf2W\xe6\xc1\xe1\xb9@\xbd\xcb\xe8E\x01i\xb5@\xbc(,\xbe\x15\xed\xad@raT@\xb8\xa6\xa3@@\xc2P\xcb\xbf2\x99@\xc6\xdfR;\x1f\xbb\x91@\xe4%\xc2\x99`\x93\xa0@\tF\xf3\xc1\xbdD\xac@bN\xd9A\x91p\xb5@)Y\xe2\x11\x13\xd5\xb9@\x94\x05&2\xb2\xf9\xba@,b\xf1< R\xb9@D\xd3\xa3\xe48\x19\xb5@\xdd\xfd2\n\xf3\x05\xae@\x99x\xd3J\x8e*\xa4@\xb8=\xd9^a\x83\x9a@5\x87X\xd1\xb2\xab\x92@s\xd1\x89u\xa3\xed\xa0@\xf1Fj;\x07/\xac@\t\xc8\xbe\x05\x95\x04\xb5@-\xdc"+Z5\xb9@RqAk\xf6R\xba@\x176V\xbe\xb2\xc6\xb8@ES\xbe\xf3\x8f\xc9\xb4@\x12\xf1\x11\x9d2\x12\xae@:M"\x15V\x9e\xa4@\xd7O\xce\x86\x9d\xba\x9b@\x0c\xce\x86\xe1\xf6\x8b\x93@\xe4\x1a\xb7\xe8P<\xa1@\xd5?\xf8F\xea\x11\xac@\xf7\xf9\x81\x98\xd9\x9b\xb4@\n\x05Pa\xd6\x9b\xb8@K\xeb[mP\xb2\xb9@\x8c\xa5\xa1\x1b{?\xb8@\x05\xbe\xd7\xc9;z\xb4@d\x92{\xc1B\x13\xae@4\xf2m\xe0=\x03\xa5@V\xed]]\xfd\xd8\x9c@\xba\x1fX\x07#\\\x94@^\x12\xd6\xf3T\x80\xa1@\x18w<\xb9h\xee\xab@\xd0\xc0\xa1%F6\xb4@&z\xfb\xb5Q\x08\xb8@\x89":\xd8\xa1\x17\xb9@\xd9\xb48\x96r\xbc\xb7@\x1f=Z\x94m+\xb4@\xae\x13\xbc\xbdr\n\xae@\xe6\xbab\xbfmZ\xa5@\xbc2\xc9\x1cJ\xdf\x9d@\'[\x0f^\x9b\x1c\x95@\x1c\x1av\x7f\x91\xba\xa1@\x89\xd0\x10\xbeh\xc5\xab@m\x0f/\x00\xc3\xd3\xb3@\x98j\x81\x01\x92z\xb7@\xbe\x07GT\xc2\x82\xb8@\xe7\xd9E\x89\x8b=\xb7@\x80!\x9bKQ\xdd\xb3@\xd2\x9d\xe1\xb2\xf3\xf8\xad@\xb2B~y\x03\xa5\xa5@d>@\x82{\xce\x9e@\xc4\xb5\'\xff\xe4\xcd\x95@\xc4j\xc4\x88\xdc\xeb\xa1@p\rS\n\xb7\x97\xab@\x17\xde\xec\x159t\xb3@\xaf\'\xf9\xecZ\xf2\xb6@\x8a=e\xe4\x82\xf3\xb7@\xa3\xd5\x03\xee\xb2\xc2\xb6@\x98Z\x1fS\r\x90\xb3@@\xa8\xa0\xb9\xd9\xdf\xad@\xd3T\x02\xa3\x0f\xe4\xa5@r\x1c\xb4\x80\xa8\xa7\x9f@ \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_range_accrual_option(
            expiry_date=datetime(2020, 8, 19),
            delivery_date=datetime(2020, 8, 20),
            cash=0.01,
            asset=0.0,
            lower_barrier=2.662200,    
            upper_barrier=3.253800, 
            obs_schedule=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],   
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_range_accrual_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_analytical_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
                
        self.assertEqual(result.SerializeToString(), expected)
    
    def test_eq_airbag_option_pricer(self):
        expected = b'\n\x92\x07\tKbE\xe7\x1c\x01\xff@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08@\xc6\x16\x8dY\xa02\xc1\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x8b\x02l\xa9\xf5\x9f*A\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08OH\xf95Z\xac1A\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00"&\xdev\x84^\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x8as\xd7\xf8\xb63\xd9@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x1eX\xee\xb2\xf4\\@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x90\x7fB3\xe9\x06\x16\xc1\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\x9f!\xf3S\xbcs\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00p\x00\xa0&\x939\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x89Y\xfd\xbc/\xa0BA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08 O\x1b\x96\x1e\x91\x86@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x083}A\xfa\xe0\xe7\r\xc1\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xb0\x12\x86/\xc3#\xa3\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xcf\x12\xc0\x1e1\xfdO\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00~\x12\x03\x964z\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x03\xe07\xda\xb5}\x15\xc1\xb1\xa2\xe4 \x91\xaa\x14\xc1Z\xee\xbb\xce\x12,\x05\xc1\xfc\xad\xe6\xfd\xe5\x90\xe8\xc0;\x97\xfb\x8b\xeb5\xe6@\xed\xf7\xbc\xdft\x05\x00A&AZ\xed\x12\xb8\nA\xac<\x88\xf9c$\x13A\xe3W\xb7\x18]f\x19A#\xbe\xe2\x84\x1f\x05 AY\x87/\x93bx#A8\xe2\xf0\x08\xb2|\x15\xc1\xf3\xe2dD\xfbE\x15\xc1\x03\xee\x8cx\xd7\xb1\x06\xc1b(\xd9"u\x0c\xed\xc0_\x0c7\xfd\x91f\xe4@\x83]4\x1eV\x1f\x00AT\x92\x84\x90y\x1f\x0bA\xe2\xd9)\xf1.b\x13A\xe1x\x0c&\xdf\x9c\x19A\xa6\xe9\xa6`x\x19 A\x8by\xf5\x8e\xeb\x85#Ayj\x04\x98~\xde \xc1\x87\x94\xda\xe9\xd7\xcb\x15\xc1\x9a\x9d\xd2\xb6\xd5\x0f\x08\xc1\x04\xbcP`p\xad\xf0\xc0\xd5\x93\xf4*&U\xe2@\xa4q\xac\x9e\xd1\x1d\x00A\xf8W\xf8\xfe\xbep\x0bA.\x0c\x07D\x00\x9a\x13A\xed\xca\xa8\x8aM\xd2\x19An"\xbd\x1a\xd6. A\xdaV\xa2\x8e\x1e\x95#A\xb8|&\x9du\xb4 \xc1\xbc\xc9\xa2y\xeb@\x16\xc1\\$\x9e\xe5\xfeK\t\xc1L\x1f\xbbY\x97\xbc\xf2\xc0\x90e\xaf\xcf\xfe\x10\xe0@\xd5\xf2]c(\x06\x00A\x80\x80\xb7(\x93\xab\x0bA\x0e\xb6\xeec\xa8\xca\x13A<\x81\x16=$\x05\x1aA\xe6\t[w\x9fD A}\xac\xd3n\x9b\xa5#A/|\xceZ\xb4~ \xc1\xf0\x87\xd4\xf1\x93\xa7\x16\xc1n\xbe\xe2\x8c$i\n\xc1\xc5\xab\xe1\xbb\x80\xb4\xf4\xc0\x9bm\x13\xc4\xa0p\xdb@\xd8\xfd\xfd\xa4\xbb\xb7\xff@\xb9}\xd3\x1eL\xd2\x0bA\xdd\x1c"c\xe0\xf3\x13A\xe5\xbe\xc8\x17[4\x1aA\xec\x12M95Z A\xc6\xc8\x9c}\xf3\xb6#A\xa7s\xaeW\xe6A \xc1l\xeey\xf5\xd5\x01\x17\xc1\xe3Y\x95_\x1cj\x0b\xc1\xe0[\xe1\xf17\x8a\xf6\xc0\x84\x0e\x93-\xb2\xa7\xd6@\xb0\xfc\xc6d\xcdB\xff@O[\x92TQ\xe6\x0bAy\xc0\x01\x8d.\x15\x14A-6\t\xa4\xea^\x1aA]\xac\xc5[\x11o A]TU\xde\xb6\xc8#A\x88\x994\x9e2\x02 \xc1\xc02\x97l\x9bR\x17\xc1E\x06\xf0\xb7\x14T\x0c\xc1\x9a\xa9\xb4\xcb\xa0A\xf8\xc0\x0b\xb11\xed\x1b\xd1\xd1@U@\xc0m\xf3\xb7\xfe@\x94Q\x89_\x02\xe9\x0bA\xd0\xae\xa28"/\x14A\x04\x9d\xd6v{\x84\x1aA\xc3\xf0\xd2\xd7\xcb\x82 A\t>\x9a\xa4n\xda#A\x80\xb1>Z0\x86\x1f\xc1\t \xbf(\xde\x98\x17\xc1\x1c\xab\x198\x98$\r\xc1\xa5\xa2L\xfd\xfb\xda\xf9\xc0\xea\x1c\x9b\x07\\1\xca@\xf0\xd18a0\x1b\xfe@\xbc\xf4\xc6\x9a!\xde\x0bA\xc1\x1b\x06\xc4\xaaA\x14AK\x0b\xf2\xf7}\xa4\x1aA\xdcm\xc4*\xf8\x94 A\x8a\xb1\x84`\xb7\xeb#A\n-\xa8\xd0\xa0\x0e\x1f\xc1\xd6e\x19:)\xd7\x17\xc1\xdd\x85J\xfe\x9e\xe0\r\xc1\xf7n\t\x0f\x1f[\xfb\xc0P\x0bS?\xcc\xfc\xc0@\x89\xa8\xda\x9eqo\xfd@|\x06\xdc\xc23\xc7\x0bA\xfc\xd4\x12\x07\xddM\x14A\x07\xd7.k5\xbf\x1aA\x16<\x83\xfdn\xa5 AV\xd3t\x8eB\xfc#AV\x9cYt\x9e\xa1\x1e\xc1\xafN\xbe\x8d\x8b\x0e\x18\xc1RMsHS\x8a\x0e\xc1f\x8cUF;\xbb\xfc\xc0\x99!\x91<\x16.\xb0@\xab\x1d\x1dM0\xbe\xfc@\x982\xb8[c\xa5\x0bA\xc4Cd>\x1eT\x14A\xce\xdd\x10\x08`\xd4\x1aA\xe4)]\xe7\xfd\xb3 A\xaa\x81\xff\x8e\xb9\x0b$A\xb09\xf2\x03\xb2A\x1e\xc1\xb9\x1f\x834\xec?\x18\xc1,\x0e\xa7\xb8\xa9#\x0f\xc1<5+\x81\xf2\x00\xfe\xc0Mk\x19\x8f&\x92q\xc0\x99\x9aV\x85\xd1\x08\xfc@?\t\x95\xd2v|\x0bA\xdc.V\xcd\xc1T\x14A\x8d;&v\xad\xe4\x1aA\xd6Q9{|\xc0 A\x8a(\xca\xa4\xfb\x19$A\x06\x1dE\x84E\xf0\x1d\xc18\x02\xd4\x90\x10l\x18\xc1{\x12\xf0\x84c\xae\x0f\xc1\xc2\xc7BZ\xea-\xff\xc0\xf9\xef\xda\x9d\xaff\xb1\xc0\x0eV\x9a#\x95Q\xfb@f\x98\x96\xc9AM\x0bA\xf8FP\xfe*Q\x14A\xe5\x05\xca\xd8B\xf0\x1aA\x8c\x16\xae\x0c\x12\xcb Am\xf6\xa5\xa7\xdb&$A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_airbag_option(
            payoff_type='CALL',                    
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_strike=2.958,
            upper_strike=3.54960,    
            lower_participation = 1.0,
            upper_participation = 1.0,
            knock_in_strike = 2.958,
            barrier_type='DOWN_IN',
            barrier_value=2.36640,                
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],            
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_airbag_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_pde_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )

        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_ping_pong_option_pricer(self):
        expected = b'\n\x92\x07\t\x15B\x01\xd6\xc4\xf0\xb6@\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\xce\xdcg\xbb\xce\xdd\xe1\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08NJ\xb9\xd1L\x82\x9c\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08g\x8c\x08\x9d\xc9s\xe0@\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\xb8\xd2\xed\xb9E\r\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xc0A>gG\xfcJ\xc0\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\\\xe1\xbc\x97\xf4\n@\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08j\xdcM\xe8\xa3$\x03A\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xa0uY\xf1\xda&a@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x000{\x06\xa8\t%\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08m\xfd\x95\x03<\xc3\xf4@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xc0\x14a\xc8\xeb\'9@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xf0?\x08Bsy\xe3@\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x90p\x85T`\xedx@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08:\xd5AjBU\x06\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xb4\xa34\x94K2\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08\x88o\x85T`\xedh@48\xa5_d\xe5\x83@\x9e\xd4\x93\x1eT\xa4\x99@\x05\x82\xb2\xf0\x1e\xa8\xaa@\x86\xe9f\x1c\xdd\x12\xb2@@\x08\xb4\x1b\xefA\xac@j\xbc\xe2\x9dA\x16\xb0@\xe2\x0b\x1ej\xac^\xa8@\x82}\xbdQ_/\x9a@\x0f\xd8\xe1\xdd\xd3\xf4\x89@L\xbcq\xe3\xd5pw@\x04\xfdZ\x1c\xab\xc0r@|\xba<<wb\x89@uE\xa5>\xe8\x8e\x9d@\x94\x8ag\x87?\x8b\xad@+(\xff\x1e\x16\x88\xb3@\xd7Gg/\xf0\x0e\xb0@a\xa0V\x14\x06\xc6\xb1@\x1c\xa3\xa5Q\xfb\x8a\xab@m\x87\x8b\x00\xa1T\x9f@3\xce/\xbe\xcf\x9b\x90@\xb6\xca\xe1\xfeOK\x80@\x942\x06jH\xbay@z\xd2B\xe8\xb7\xc9\x8f@?\xa4k7\x95\t\xa1@CHe6I\xf9\xaf@\xba\x16$7i\x1c\xb5@\x9e\x1f\x83$\xe6\xb1\xb1@e\xf7\x9e\x19\xdcT\xb3@ZO\x8f|g\x80\xae@:\xc5^\x98q\xec\xa1@\x05\xc7w\xb0\x92.\x94@\xa2c\x82\xfe\xbf\xb9\x85@\xff\xf7\xf3D\n.\x80@\xce\xcfH\xd6\xf0\xdd\x92@rI\xb9\xe3\x01^\xa3@q\x8db\x148\x18\xb1@\x889\xbew\x85\'\xb6@\xf8J`\xc3A\x86\xb3@\x1f\xc3b\x8d\x03\xeb\xb4@\xd7\xdb.\x1f\xde\xd0\xb0@RqH\xc3\xdd\xe1\xa4@\x83\xf7\xbb&\x8f\x95\x98@\xb27mA\xe9L\x8c@\x10\xc0\xdb1\x93\x8d\x83@\x97j\x1e}\xbfZ\x96@/I+\x1c\xe3w\xa5@E=(\xc6BD\xb2@\x8b\x97i_jJ\xb7@p\x9eZ\t\x98\xc4\xb4@\xc9\x01\xfb\x8f<`\xb6@\xd62w$\xb4_\xb2@\x89\xd2z\x063b\xa7@#\xf7\x9f\x97Q\xc9\x9c@\xaa\xd1}}O\xec\x90@\xa1\xac\x04?x \x88@\xb0\x90x\xb5<\xd0\x99@\xeeH\x9dT\xc4\x91\xa7@\x99U\x11e\xd0j\xb3@\xcf"\'\x8d\tP\xb8@\x9at\xcf\x1eN?\xb6@\xa8\x0f3\x8d;\xa2\xb7@\x98\x1f\xf5\\\xc7\xcb\xb3@]\xecC!\xf1\xbd\xa9@\tp\xbdr\xdb\x85\xa0@\r%\xea\xfb\x86D\x94@\r\xe4\xe43t(\x8d@\xce\r\xa9\xba\xae\xba\x9c@\xaa\x19Vg\xab\xa0\xa9@\x07\xbem\xeeCr\xb4@\xcd\x80\xd2t\xeer\xb9@\xc0\x1b\x8b\x0e\n\xaf\xb7@K\xb3\xa0\xbdw\xc1\xb8@+\xd1a\xee\x11 \xb5@3"\x0b\xfe\xcb\x83\xac@\x82\x13d?\x08\xb2\xa2@\xb2\nw\x8al6\x98@\xa0s\x0b2[\xd6\x90@G\xd2B\xe8\xb7\xc9\x9f@\x1d\xd2^w\xf5\x95\xab@\xce\x82\x95\xe3kg\xb5@\x8et\xb3\x8f\x10s\xba@*\xc1\x9f\x1e\x86\xf6\xb8@H\x11L\xa5\\\xe4\xb9@\xea\x01\xfb\x8f<`\xb6@)\r\x1b\xf3\x8f\xd4\xae@\xb7\xe7j\x11o\x11\xa5@e\xa5L1;\xb3\x9b@\x97^\x1b\'\x1f\'\x93@$\xab\x07\xb6\x94\xaa\xa1@\xdf\x88\xc0\xa7\xffb\xad@\x11\xb0\xe0\xc5\x16W\xb6@K9\xdb\x848h\xbb@y{\x16r\x1f\x07\xba@\x94\xfa{\x1e\xf0\xff\xba@\xe90\xedQ\'x\xb7@\x1f\tAe\x98\xb3\xb0@\x1e-\x9f\x92\xa7\'\xa7@t\xe5\xfdK\x95j\x9f@zP\xc7\x9a\xe2\x18\x96@\x18\x8f{,YZ\xa3@\x03X\xd2\xda\xa6I\xaf@#\xd1\xd3&\xf39\xb7@\xd3\xc2\xf1\xd2\x97E\xbc@r\xaa\x084\n\x1f\xbb@\x87\xfa\xb4\xba\xe0\x0c\xbc@r&uL\x89\xa0\xb8@$\xc5^\x98q\xec\xb1@6\x16\x08\xa8+P\xa9@f\xa9`\xd6T\x82\xa1@\x8dU.r\x83\xab\x98@\x04D6}#\xff\xa4@\xe2o\xbdr\xdb\x85\xb0@\xad+1OX\x0c\xb8@\xfb3X\x1eZ\t\xbd@\xb65\xc6a\xa9$\xbc@\xb7\xc9\x8dQ\x97\xe6\xbc@\x11=\xb7\x0b\xd7\x9e\xb9@{\x0c\x01]\xf9\x1d\xb3@z\x8c\x9c.\x9e\x99\xab@~\x05\x9ez\xea\x89\xa3@\xe4\xe5\x19\xdb\xd26\x9b@\xe9\x9a~\x82\xf9\x8d\xa6@\x8dx\x00\xd1\x1aO\xb1@\x85\xe2Y\xe3q\xcc\xb8@\x1dv\x05D"\xc2\xbd@\xf0b\x11DT\x14\xbd@K\xe0\xcf\x10\xe5\xe4\xbd@\xafS\xf9\xca$\x9d\xba@\xfe_\xfb\xa2O\\\xb4@D\xa1p\xaa\x9c|\xad@\xda\x8e\xedd:t\xa5@ \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_ping_pong_option(
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            lower_barrier_type='DOWN_IN',
            lower_barrier_value=2.810,    
            upper_barrier_type='UP_IN',
            upper_barrier_value=3.106,    
            barrier_obs_type='DISCRETE_OBSERVATION_TYPE',
            obs_schedule=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],  
            payment_type='PAY_AT_MATURITY',
            cash=0.015,
            asset=0.0,
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_ping_pong_option_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_phoenix_auto_callable_note_pricer(self):
        expected = b'\n\x92\x07\t\t\x9b\xe9\x9c\xed}(A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x90Q\x86\xed\xd2\xa4\x1eA\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xf9\xbet\xc9\x02\xd6\xdd@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\xf0\x91+\x19\x0e\xcc(\xc1\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00 %\xd2z\x1aI@\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\x08\xe1\xe8\xd5=\x8c@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\xe0\x91\xa3SPT\xc0\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xf9\xb7\xb6\xd1\x16\x0eBA\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\xcc\x15\xb9G-\xa0@\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x00\xfe\xc0\xac\x88X\xa7\xc0\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08MP\x91X\xbe\x88IA\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\x96\x1c\xc6\xe5\xef\x8e@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xa9=\x7f/4bEA\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08 \x82\x13\x1e\xfb^\xdb@\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08#?>\x04w\xe4s\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xa6\xf9K\xbfK\xa0\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08`\xe4\xcd\xa4\xa6\x0c,A}\x9d\xcfY\xa4\xe1,A\x15\x12\xd6z\xa0\x82-A:.\x19"\xe0\xbf)A\xf8\xe20\xb86o$A \x9cG\x10\x90\xc8"A5Z\xfb\x1d\xd2H$A\x0fu\x9e\xafN (Aw\x0c\x9d\x0c\x80\t,A\xa6\xb3\xd4\xb8\xaa\x17.A;\xf8\'\xa2~\xaa.A.\xb1\x80\xb0P\xfc+A\x1c\xefn\xf4O\xd0,A\xd0\x96\xf1\x85\x1at-A\x1f\x1bdB\x834*Ac\x95\xdd\x0f\xc4\x91%A_9\xea\x8b\xd0\x0e$A@-\xd8\xb0\xc8_%A\x0fu\x1dTy\xa9(ADh+\x95\xa7\x17,A\x1dB\x0b(\'\x00.A8+\xe7\x06\xcd\x9e.A\xee\x06\xea\xda\xb2\xeb+A\xbf\x99f\x92J\xbf,Ag\x1f\xe9\xb1\xe2d-A\x15\xa7\xaf~\xc3\x9e*A\xc2~\x82c\xf2\x98&A\x1e\x8dV\xb5\xb28%A\xe0\x05\x9eH\x96G&A\xe7V2\xdf\xdc")A\x89\n\xf0@\xcd2,A\xdc\x81\xb0\xe2K\xef-A\xc64\xf6+\xe7\x96.A\xad\xe0\xa9We\xdb+A\xb0\xc4\xbe\x18\xf0\xad,A\xb4gI\xb0bU-A\xc8M\xf9\xf6\xe9\xe8*AK\x16=\xa4\xe8y\'A\x03\x83~\x98\xd9C&A\xebp/m6;\'A\xb7\x11N\x80T\x95)A\xd8\xb0\xdb>\x13;,A\xd3>\xe1\x10\x17\xdc-A\n\x0f\x9eIn\x88.A\xe9^)\xea\xcb\xca+A\xee\xab\x1b\x7f\x99\x9c,A\x8e\xb0\x96\xedZE-AMS\x99\x99\x81@+A\xe97\x82\x10eG(A\xf0\xcb\xdc\xb6\x861\'Az\xc7\'\x1f\xe6\x02(A\xa2\x97\xf2\x01}\x14*A\x12\xeb[\xb5\x84Y,A@\t\xf2\xa7`\xdf-AT\xd8\xee\xe9\x99|.A\x1b\xd7\xbe\xa5%\xba+AE\xc3`\t\x1c\x8b,A\xfd\x9e\x10.B5-A\xfa\x10\x98\xa5\\\x88+A\x0c#\xef\x13\x18\xfe(AGb\xf9\xc2>&(A\xd0K\x01|B\xd8(AZ\xf8\x9f\x95\x8e\x8d*A\x057q\xccS{,Ay\x83\x170i\xdc-A\x13o\x05s3w.A\xea\x99\xc4\x96\x85\xa9+AZ\xb7]\xb1%z,A\xfc\x81\x8c<\xac$-Adm|D\x9d\xc6+A1\xc3\xd6\xaau\x95)A\xb0\xf8\x19\xc6\xc1\xd8(A\xd7\x94\xa72q\x83)A\xce\x93\x0c;\x15\xe8*Aw\x1a\xb16\xdd\xab,Aq\x807\xdc\xf9\xcf-A\xe8/f{\xadr.A\x920\xe3P\x19\x99+A\xe5]\x8a9\xc6h,A\xbeT\x02{\xc5\x13-A\xb3\x8e\xce/"\xf2+A}\x8f6Nm\'*A\xeb\xbej\xec\xe2\x8b)A\x99\x10\r\\\x820*A\x84;f\xe6\xb0K+A(\xffA\xd9\r\xc8,A$4\x86\xcb\xb2\xda-A\xae\x95b_\xd5o.A\x99\xf4\xb7\xbc\x96\x88+A\x9b\xe1+\x1c3W,A\xd2\x9b\xab\xf2\xd8\x02-AGh\xe8\xa5Y",A<\xe3.9\x84\xac*A\xe0\x86\xb6\x90\x83\x1d*AN\xd2xo\x8a\xb3*A^\x8c\xaf\xb0\x91\xa1+A\x9eAj7\xac\xff,AR\xd2p\xe2\x8a\xe4-A\xa8\x86\x02\xf0\xcdo.A\xaaV\x85\xa9\xbaw+A\x1f\x83:\x18\xefE,A9\xbb:\xf8\xaa\xf0,Ay\x05\xa8\x0e\xdeG,A\x1d\xb6U\x19\x8c\x12+A\'\xc6\x7f\xae\x91\xa2*A\xaba\xfb\xe9\x8c\x11+A\xedF\xcb\x11\xfe\xea+A@s\x10\xef#(-Ae\x18\xe3/n\xf4-At\x10\xfaHcp.A%va\xa60g+A\x97Y}\xf8o4,A\x7f\xb5\xaf=\x8e\xe0,A\xcf\x0e\xe0\\6`,A\xec\xbd\n\x11\xa4h+A\x04\xe2\xf4\xcbU\x17+A\xdc\xb6\x06\xd3\xd5x+A\xb2\x8c\xe5\xe3PF,A\x9dV\xf0\x08\xb6?-A\xac\xcc"Y\xb8\xff-A\xd3O\xde\xb2To.AS\x88\x8c\xea\rV+A\x9aN*<\xb9",A\x12x\xd7\rF\xcf,A\x8c`\xead\x02t,A4\x83 %\x80\xb5+A\t\xcd\xd7,\xcfz+A\xfd\xcc&\x84\xe5\xda+A\x19\xb2x\x07\x18\x82,A\xeec\x15v\xfb`-ALM\xdf?m\r.ANI\x97y6j.A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_phoenix_auto_callable_note(
            coupon_payoff_type = 'CALL',
            coupon_strike=2.958,
            coupon_rate=0.12,
            start_date=datetime(2020, 2, 22),
            coupon_dates=[datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22), datetime(2020, 7, 23),datetime(2020, 8, 22)],
            day_count='ACT_365_FIXED',
            knock_out_barrier_type ='UP_OUT',
            knock_out_barrier_value=3.25380,
            knock_out_sched=[
                [datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22), datetime(2020, 7, 23),datetime(2020, 8, 22)],
                [0.0] * 6,
                [1.0] * 6
            ],
            knock_in_barrier_type ='DOWN_IN',
            knock_in_barrier_value=2.66220,
            knock_in_sched=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],  
            long_short='SELL',
            knock_in_payoff_type='PUT',
            knock_in_payoff_strike=2.514300,
            expiry=datetime(2020, 8, 19),
            delivery=datetime(2020, 8, 20),
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_phoenix_auto_callable_note_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

    def test_eq_snowball_auto_callable_note_pricer(self):
        expected = b'\n\x92\x07\t\xf8\xb3\xc5\xd1\xd2t.A\x1a\xfd\x06\n\xa7\x01\n\x05DELTA\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x80Q0\xd6\x9dw\xfc\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xa2\x80\xab\xbd*b\xf2@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x80^\x08\xc8\x0c\xba\x06\xc1\x12\x05TOTAL\x1a\x00\n\xb0\x01\n\x0eDELTA_EXPOSURE\x12e\n\x06EQUITY\x126\n\x1dDIVIDEND_DIVIDEND_CURVE_50ETF\x12\x15\n\n\n\x08\x00\x00S\xf6\x03R\'\xc0\x12\x05TOTAL\x1a\x00\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x00\xa1o]\xabf\xa1@\x12\x00\x1a\x00\x127\n\rINTEREST_RATE\x12&\n\rCNY_SHIBOR_3M\x12\x15\n\n\n\x08\x00\x00\r\x81%\x9e2\xc0\x12\x05TOTAL\x1a\x00\n6\n\x05GAMMA\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\xb3\xc9`\xd5M\xc8\x10\xc1\x12\x00\x1a\x00\n?\n\x0eGAMMA_EXPOSURE\x12-\n\x06EQUITY\x12#\n\x0fPRICE_50ETF.SSE\x12\x10\n\n\n\x08\x01 \x1f\xe9\xc4\x12n\xc0\x12\x00\x1a\x00\n$\n\x05THETA\x12\x1b\n\x05THETA\x12\x12\x12\x10\n\n\n\x08\x000\x88\xac#Q`@\x12\x00\x1a\x00\n:\n\x05VANNA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xbd\xb2\xd93\xbdi\xfc@\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVANNA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00p\xd5ck6A@\x12\x05TOTAL\x1a\x05TOTAL\n9\n\x04VEGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\xe0\x87\x9b"\xa4\x99\xf4\xc0\x12\x05TOTAL\x1a\x05TOTAL\nB\n\rVEGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\\\xa8\xbbB^\x8a\xc0\x12\x05TOTAL\x1a\x05TOTAL\n:\n\x05VOLGA\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08P)-\xb8\xf399\xc1\x12\x05TOTAL\x1a\x05TOTAL\nC\n\x0eVOLGA_EXPOSURE\x121\n\x06EQUITY\x12\'\n\t50ETF.SSE\x12\x1a\n\n\n\x08\x00\xa0\x8c\xb3Z\xaad\xc0\x12\x05TOTAL\x1a\x05TOTAL"\x00*\x03CNY2\x00\x10\x01"\xa9\x08\x08\x0b\x10\x0c\x1a\xa0\x08#r\x8bR\xacW,A?Sl\x8f0\n-A\x89n\xdd\xb3j\x8e-A\x82d\xa7k\xe2\xef-A\xff&\x1e\xfc\x16A.A\x98\x89\x97%\xc3\x87.A\xc0\xd1\xd6\x1e\x83\xbf.A\xcb\xb5+@P\xdb.AzN\x96\xef\x1c\xdf.A7#\xe1\x81\xbb\xd7.A\x17\xa2\xa8\xfcM\xd4.A\x8f\xff\xc68\x80I,A<\xce\xfe\xd3G\xfb,A\x10\x13M\x9f;\x82-A\xd8\xb9Iii\xe9-AnEFu\xc5>.A\x15\x8d\xa5\xc4\xce\x86.A\xbf\x82\xcf\xfc\xa9\xbd.A\xf1\x91\x97\x15\xe7\xd8.AN`\x94\x03/\xde.Ab\x13\x19M\x11\xd8.A\x0e\x94\tH\x98\xd4.A\\\xb5\x1d\xe0o;,A]\x19\xe1\x0b]\xed,A\xa2\x87\x1d\x0b5v-Ai\xa0d-\xf6\xe2-A\xb9!\xb4(S;.A\x86\x9f\xb6\x92\xa4\x84.A\xd3H\xab\xa0\xe8\xba.A\x98\x83}\xfd\x7f\xd6.A\x8a\xfez-\x0f\xdd.A\x15Hg\x01U\xd8.A?\x12i\xe1\xd0\xd4.A\x9f\xda^\n\xc6-,A@\x92\xcb\xeaV\xdf,A\xae5\xb8T`k-A\x16AA\x99\xe8\xda-A\xd5\x83\x82\x8f\xcc7.A\xac\xa4"0}\x81.A\xb9\x84tzB\xb7.A\xe8Q\xc1\x88\xe0\xd3.A\xad\x9ae,\xbe\xdb.Ak\xaa\xb4\x822\xd8.A\xf0\x105\xfd\xc9\xd4.AQ\xb12J\x10 ,A\xd0mZf\xc6\xd1,AJ*\xcc\t\xc7_-A\xb6\xc8bIN\xd3-A\xc2\xeb\xe9\xfa@2.Ax:>7\x9b|.A\xb4\x8e\xb8\x03*\xb3.A\x9c\xdaO\xe8\x8d\xd0.A\xa7.Mn\xbf\xd9.Aw\xdb\xa2\xd2\xad\xd8.A\xcf\x8d\xba\x0e:\xd5.A\xd1\xe2uF\xf7\x11,A\x7f*\x158\x14\xc4,A\x13nE\x98\xc3S-A\x88\xd5\x03\x94=\xcb-A\xbc\t\x13o\xd0+.A\x86H{\xbb\xfdw.A=\xf8F\x12`\xae.A\xe4\x8d\x8b\x8f~\xcc.A:k"a\xbd\xd7.A\xcb\xfc\x88\xf4\x1f\xd8.A\x01\x84\x04n$\xd5.A\xa5\x02\xe8\xf6/\x05,A\xf3\xf1H\x01\xc0\xb5,A\xca\xf2i\xb7lH-A_\x05\xe5\x83\xc5\xc1-A\xdbpF\x08h#.AS\x17\xfb\xfchq.A\xdff\xe63M\xa9.A\xa6\xa8M"*\xc8.A\xce\xef\xd4\x11j\xd5.A{]\xc7\x81I\xd7.A\xb0B\x0f\x0b\x1e\xd5.A\xb4\x8d\x7f^\xa5\xf7+A`\xf2\x01\xa2w\xa8,AIC\x11#\x96<-A\xc0\xfd4\x81\xba\xb7-Ao\xfa\xc8\x11\xdf\x1b.A\xd5\x96\xc2\x82\x83j.A\x05\xd2c\xd0\x99\xa2.AMo\xceE\x9c\xc3.A->\xb1\x9f\xc1\xd2.A\xf1\x91r7\xe5\xd6.A\xc1[\xa3D\xf5\xd4.A\x9d\xf4ic\xca\xea+A.0\x85E\x88\x9c,A:\xe3\x88\x8f\xff0-A\'n\x90r\x91\xad-AE`\xa5y\x97\x13.A\xebt\xc1\xea\nc.A1\xc5\x91\xe2\xca\x9b.A\'\xf7\x9c\xeen\xbe.A\x08\xbd\xb3\x92\xbe\xcf.A\n\x1f\xc9\x8f\xc0\xd5.A\x92\xf0q\xc5\x88\xd4.A\xbf\x13\x13\xb1?\xdd+A\t\x9c\r\xae\xa9\x8e,A\xdaD\xb4Q7%-A\x8c\r\x9b\xebE\xa2-A\x9a-\xf8\xf3\r\n.AT\xe4c\xb1\x10[.A\xe4\x01m|s\x94.A^\xce\xffg>\xb9.A\x96\x82\x9f\xb8\x83\xcc.AEO\xef\xe9\xfe\xd4.A\x87F\xb9_5\xd4.AI\x08\x11#\x07\xd0+A\xf8\xbc\xee\x9a\xbd\x81,AL\xd6\xd5\xf0Y\x19-A\xba\xd23\xbe\x10\x98-A\xe5_k\xfd\xcd\x00.A\x89\xf2\x1d\x86\xa4R.A\xde\x85\xe9\'\x92\x8c.A\xd7$w\x1b\x08\xb4.A\x8fl\xdd\xe8{\xc8.A\xb6\xd5\xff\xfaA\xd3.A\xe7\xc7.p\xd3\xd3.A]I\xc9\xc4\xda\xc2+A\x8c\x8f\xae&\x0ft,A7\x96`\xa5\x98\x0c-A\xad.=\x91\xa1\x8c-A\xaaiy\x85~\xf6-A\xbf=\x17\xd0tI.A\x9cg\'G\xdb\x84.Ac\x83\x02,2\xae.AcD\xff\x81K\xc5.A\xa02\xf7\x0eW\xd1.A\n\xb0\xdf\xf1\xac\xd2.A \x01*Z\nX\x03\t\x8a\x1fc\xee\x02@\xaa\xbc\x1d\xe1\xb4\xe0\x03@Qp\xb1\xa2\x06\xd3\x04@\xf8#EdX\xc5\x05@\x9f\xd7\xd8%\xaa\xb7\x06@F\x8bl\xe7\xfb\xa9\x07@\xed>\x00\xa9M\x9c\x08@\x94\xf2\x93j\x9f\x8e\t@;\xa6\',\xf1\x80\n@\xe2Y\xbb\xedBs\x0b@\x89\rO\xaf\x94e\x0c@2b\n`\x9a\x99\x99\x99\x99\x99\xa9\xbf\xf2\x94 O\t\xf2\xa4\xbfJ\x90\xa7\x04yJ\xa0\xbfD\x17]t\xd1E\x97\xbf\xe9\x1b\xd6\xbea\xed\x8b\xbf\x94\x12\xe4)A\x9er\xbf\xaa\x12\xe4)A\x9er?\xf4\x1b\xd6\xbea\xed\x8b?J\x17]t\xd1E\x97?M\x90\xa7\x04yJ\xa0?\xf5\x94 O\t\xf2\xa4?\x9d\x99\x99\x99\x99\x99\xa9?'

        inst = create_snowball_auto_callable_note(
            coupon_rate=0.12,
            start_date=datetime(2020, 2, 21),
            coupon_dates=[datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22), datetime(2020, 7, 23),datetime(2020, 8, 22)],
            day_count='ACT_365_FIXED',
            knock_out_barrier_type ='UP_OUT',
            knock_out_barrier_value=3.25380,
            knock_out_sched=[
                [datetime(2020, 3, 23), datetime(2020, 4, 22), datetime(2020, 5, 22), datetime(2020, 6, 22), datetime(2020, 7, 23),datetime(2020, 8, 22)],
                [0.0] * 6,
                [1.0] * 6
            ],
            knock_in_barrier_type ='DOWN_IN',
            knock_in_barrier_value=2.66220,
            knock_in_sched=[
                    [
                        datetime(2020,2,22), datetime(2020,2,23), datetime(2020,2,24),
                        datetime(2020,2,25), datetime(2020,2,26), datetime(2020,2,27),
                        datetime(2020,2,28), datetime(2020,2,29), datetime(2020,3,1),
                        datetime(2020,3,2), datetime(2020,3,3), datetime(2020,3,4),
                        datetime(2020,3,5), datetime(2020,3,6), datetime(2020,3,7),
                        datetime(2020,3,8), datetime(2020,3,9), datetime(2020,3,10),
                        datetime(2020,3,11), datetime(2020,3,12), datetime(2020,3,13),
                        datetime(2020,3,14), datetime(2020,3,15), datetime(2020,3,16),
                        datetime(2020,3,17), datetime(2020,3,18), datetime(2020,3,19),
                        datetime(2020,3,20), datetime(2020,3,21), datetime(2020,3,22),
                        datetime(2020,3,23), datetime(2020,3,24), datetime(2020,3,25),
                        datetime(2020,3,26), datetime(2020,3,27), datetime(2020,3,28),
                        datetime(2020,3,29), datetime(2020,3,30), datetime(2020,3,31),
                        datetime(2020,4,1), datetime(2020,4,2), datetime(2020,4,3),
                        datetime(2020,4,4), datetime(2020,4,5), datetime(2020,4,6),
                        datetime(2020,4,7), datetime(2020,4,8), datetime(2020,4,9),
                        datetime(2020,4,10), datetime(2020,4,11), datetime(2020,4,12),
                        datetime(2020,4,13), datetime(2020,4,14), datetime(2020,4,15),
                        datetime(2020,4,16), datetime(2020,4,17), datetime(2020,4,18),
                        datetime(2020,4,19), datetime(2020,4,20), datetime(2020,4,21),
                        datetime(2020,4,22), datetime(2020,4,23), datetime(2020,4,24),
                        datetime(2020,4,25), datetime(2020,4,26), datetime(2020,4,27),
                        datetime(2020,4,28), datetime(2020,4,29), datetime(2020,4,30),
                        datetime(2020,5,1), datetime(2020,5,2), datetime(2020,5,3),
                        datetime(2020,5,4), datetime(2020,5,5), datetime(2020,5,6),
                        datetime(2020,5,7), datetime(2020,5,8), datetime(2020,5,9),
                        datetime(2020,5,10), datetime(2020,5,11), datetime(2020,5,12),
                        datetime(2020,5,13), datetime(2020,5,14), datetime(2020,5,15),
                        datetime(2020,5,16), datetime(2020,5,17), datetime(2020,5,18),
                        datetime(2020,5,19), datetime(2020,5,20), datetime(2020,5,21),
                        datetime(2020,5,22), datetime(2020,5,23), datetime(2020,5,24),
                        datetime(2020,5,25), datetime(2020,5,26), datetime(2020,5,27),
                        datetime(2020,5,28), datetime(2020,5,29), datetime(2020,5,30),
                        datetime(2020,5,31), datetime(2020,6,1), datetime(2020,6,2),
                        datetime(2020,6,3), datetime(2020,6,4), datetime(2020,6,5),
                        datetime(2020,6,6), datetime(2020,6,7), datetime(2020,6,8),
                        datetime(2020,6,9), datetime(2020,6,10), datetime(2020,6,11),
                        datetime(2020,6,12), datetime(2020,6,13), datetime(2020,6,14),
                        datetime(2020,6,15), datetime(2020,6,16), datetime(2020,6,17),
                        datetime(2020,6,18), datetime(2020,6,19), datetime(2020,6,20),
                        datetime(2020,6,21), datetime(2020,6,22), datetime(2020,6,23),
                        datetime(2020,6,24), datetime(2020,6,25), datetime(2020,6,26),
                        datetime(2020,6,27), datetime(2020,6,28), datetime(2020,6,29),
                        datetime(2020,6,30), datetime(2020,7,1), datetime(2020,7,2),
                        datetime(2020,7,3), datetime(2020,7,4), datetime(2020,7,5),
                        datetime(2020,7,6), datetime(2020,7,7), datetime(2020,7,8),
                        datetime(2020,7,9), datetime(2020,7,10), datetime(2020,7,11),
                        datetime(2020,7,12), datetime(2020,7,13), datetime(2020,7,14),
                        datetime(2020,7,15), datetime(2020,7,16), datetime(2020,7,17),
                        datetime(2020,7,18), datetime(2020,7,19), datetime(2020,7,20),
                        datetime(2020,7,21), datetime(2020,7,22), datetime(2020,7,23),
                        datetime(2020,7,24), datetime(2020,7,25), datetime(2020,7,26),
                        datetime(2020,7,27), datetime(2020,7,28), datetime(2020,7,29),
                        datetime(2020,7,30), datetime(2020,7,31), datetime(2020,8,1),
                        datetime(2020,8,2), datetime(2020,8,3), datetime(2020,8,4),
                        datetime(2020,8,5), datetime(2020,8,6), datetime(2020,8,7),
                        datetime(2020,8,8), datetime(2020,8,9), datetime(2020,8,10),
                        datetime(2020,8,11), datetime(2020,8,12), datetime(2020,8,13),
                        datetime(2020,8,14), datetime(2020,8,15), datetime(2020,8,16),
                        datetime(2020,8,17), datetime(2020,8,18), datetime(2020,8,19)
                ],
                [0] * 180,  # All values are 0
                [1] * 180  # All weights are 1
            ],  
            long_short='SELL',
            knock_in_payoff_type='PUT',
            knock_in_payoff_strike=2.514300,
            knock_in_payoff_gearing = 1.0,
            reference_price = 2.958,
            expiry=datetime(2020, 8, 22),
            delivery=datetime(2020, 8, 22),
            settlement_days=1,
            nominal=1000000.0,
            payoff_ccy='CNY',
            underlying_type='SPOT',
            underlying_ccy='CNY',
            underlying=self.underlying
        )

        result = eq_snowball_auto_callable_note_pricer(
            instrument=inst,
            pricing_date=datetime(2020, 2, 21),
            mkt_data_set=self.mkt_data_set,
            pricing_settings=self.bsm_mc_pricing_settings,
            risk_settings=self.risk_settings,
            scn_settings=self.scenario_analysis_settings
        )
        
        self.assertEqual(result.SerializeToString(), expected)

if __name__ == "__main__":
    suite = unittest.TestLoader().loadTestsFromTestCase(TestEqAnalytics)
    unittest.TextTestRunner(verbosity=2).run(suite)


